
#ifndef __MPG_DEC_IMP_H__
#define __MPG_DEC_IMP_H__
 
#include "mpeg_decoder.h"


class FileWriter_RGB
{
private:
    int out_fd;
public:
    FileWriter_RGB( void );
    void WriteImage(const Image_RGB<Pixel>& img);
    void SetRGBStream( int fd );
};

class TiFrameOutput: public DecodedPictureSink, public ImageDataAccumulator
{
public:
    TiFrameOutput(int);
    void BeginPicture(const DecodedImageData*); 
    void ShowMBRows(DecodedImageData*); 
    void FinishedPicture();

    bool PictureAvailable() { return pic_available; }
    PTS  AskPTSOfNextToBeDisplayed() const { return nextpts; }
    void ShowPicture() { pic_available=false; }

private:
    int iframe_fd;
#if IMG_OUT_YUV
    ofstream ostr;
    FileWriter_YUV1 writer;
#else
    FileWriter_RGB writer;
#endif
    bool first;

    bool pic_available;
    PTS  nextpts;
};

class TMpegDecoderImp
{
public:
    TMpegDecoderImp( int, int, bool iframe_only, TOutputType type );
    void get_next_frame( void );
    void InitVideoDecoder1();
    void InitVideoDecoder2();
    void InitStreamSource();
    Syncer* syncer;
//    TiFrameOutput *lastoutput;
    DecodedPictureSink *lastoutput;
    void run( void );
    void Execute( void );
    void Terminate( void );

private:
    TOutputType output_type;
    pthread_t vd_thread;
    int iframe_fd;
    int mpeg_fd;
    // // stream sources
    ifstream *input_stream;
    StreamSource_IStream *strsrc_istr;
    StreamSource* streamsrc;
    // the system decoder

    SystemDecoder_MPEG1 *system_dec;
    SystemResyncer      *system_resync;
    SystemDemux *system_demux;

    // video decoding pipeline

    SystemDemuxExtractor *video_extractor;
    VideoAUConverter     *vidcon;
    PTSVideoCompleter    *video_ptsvideo;
    VideoDecoder         *video_dec;

    bool end;

};

#endif

