/* mmc.h - header file for mmap cache package
**
** Copyright  1998 by Jef Poskanzer <jef@acme.com>.
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
*/

#ifndef _MMC_H_
#define _MMC_H_

/* return codes */
#define MMC_NORMAL	(0)
#define MMC_ERROR	(-1)

/* type of parameters */
#define MMC_P_CLEANUP_TIME		0
#define MMC_P_DEFAULT_EXPIRE_AGE	1
#define MMC_P_DESIRED_MAX_MALLOC_BYTES	2
#define MMC_P_DESIRED_MAX_MALLOC_FILES	3
#define MMC_P_DESIRED_MAX_MAPPED_FILES	4
#define MMC_P_DESIRED_MAX_OPENED_FILES	5
#define MMC_P_USE_SENDFILE		6
#define MMC_P_CLOSE_ON_EXEC		7
#define MMC_P_MAX_FILE_SIZE_L0		10
#define MMC_P_MAX_FILE_SIZE_L1		11
#define MMC_P_MAX_FILE_SIZE_L2		12
#define MMC_P_MAX_FILE_SIZE_L3		13

/* type of current work parameters */
#define MMC_V_MA_MAP_COUNT		100
#define MMC_V_MA_USE_COUNT		101
#define MMC_V_MM_MAP_COUNT		110
#define MMC_V_MM_USE_COUNT		111
#define MMC_V_FD_MAP_COUNT		120
#define MMC_V_FD_USE_COUNT		121

/* Get current param value (MMC_V_*)
*/
extern int mmc_get_value( int value_type );

/* Get config. param value (MMC_P_*)
*/
extern int mmc_cfg_get_param( int param_type );

/* Set config. param value (MMC_P_*)
*/
extern int mmc_cfg_set_param( int param_type, int param_value );

/* Returns a boolean value (TRUE/FALSE) wether a file with given st_size
** should use fdMap (instead of maMap or mmMap method).
*/
extern int mmc_is_fdmap( off_t st_size );

/* Returns an mmap()ed area for the given file, or (void*) 0 on errors.
** If you have a stat buffer on the file, pass it in, otherwise pass 0.
** Same for the current time.
*/
extern int mmc_map( int* pfd, void** paddr, char* filename,
			struct stat* sbP, struct timeval* nowP );

/* Done with an mmap()ed area that was returned by mmc_map().
** If you have a stat buffer on the file, pass it in, otherwise pass 0.
** Same for the current time.
*/
extern int mmc_unmap( int fd, void* addr,
			struct stat* sbP, struct timeval* nowP );

/* Clean up the mmc package, freeing any unused storage.
** This should be called periodically, say every five minutes.
** If you have the current time, pass it in, otherwise pass 0.
*/
extern void mmc_cleanup( struct timeval* nowP );

/* Free all storage, usually in preparation for exitting. */
extern void mmc_destroy( void );

/* Generate debugging statistics syslog message. */
extern void mmc_logstats( long secs );

#endif /* _MMC_H_ */
