/*

kHTTPd -- the next generation

Pass connections to userspace-daemons

*/
/****************************************************************
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2, or (at your option)
 *	any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ****************************************************************/

/*

Purpose:

Userspace() hands all requests in the queue to the userspace-daemon, if
such beast exists.

Return value:
	The number of requests that changed status
*/
#include <linux/kernel.h>

#include <linux/errno.h>
#include <linux/malloc.h>
#include <linux/net.h>
#include <linux/sched.h>
#include <linux/skbuff.h>
#include <linux/smp_lock.h>
#include <linux/un.h>
#include <linux/unistd.h>
#include <linux/wait.h>

#include <net/ip.h>
#include <net/sock.h>
#include <net/tcp.h>

#include <asm/atomic.h>
#include <asm/semaphore.h>
#include <asm/processor.h>
#include <asm/uaccess.h>

#include <linux/file.h>


#include "structure.h"
#include "prototypes.h"
#include "sysctl.h"

/* prototypes of local, static functions */
static int AddSocketToAcceptQueue(struct socket *sock,const int Port);


int Userspace(const int CPUNR)
{
	struct http_request *CurrentRequest,**Prev,*Next;

	EnterFunction("Userspace");

	CurrentRequest = threadinfo[CPUNR].UserspaceQueue;
	Prev = &(threadinfo[CPUNR].UserspaceQueue);

	while (CurrentRequest!=NULL)
	{
		if (CurrentRequest->sock!=NULL)
		{
			int A,B;

			lock_kernel();  /* To be sure */
			if ((CurrentRequest->sock!=NULL)&&(CurrentRequest->sock->sk!=NULL))
			{
				lock_sock(CurrentRequest->sock->sk);
				remove_wait_queue(CurrentRequest->sock->sk->sleep,&(CurrentRequest->sleep));
				release_sock(CurrentRequest->sock->sk);
			};
			unlock_kernel();
		}
		else
			printk(KERN_CRIT "kHTTPd: DEBUG : Socket is NULL ! \n");


		if (AddSocketToAcceptQueue(CurrentRequest->sock,sysctl_khttpd_clientport)>=0)
		{
			(*Prev) = CurrentRequest->Next;
			Next = CurrentRequest->Next;

			CurrentRequest->sock = NULL;	/* We no longer own it */

			CleanUpRequest(CurrentRequest);

			CurrentRequest = Next;
			continue;

		}
		else
		{
			(*Prev) = CurrentRequest->Next;
			Next = CurrentRequest->Next;


			printk("kHTTPd: WOOPS : NO USERSPACE DAEMON \n");
			Send403(CurrentRequest->sock);

			CleanUpRequest(CurrentRequest);

			CurrentRequest = Next;
			continue;

		}



		Prev = &(CurrentRequest->Next);
		CurrentRequest = CurrentRequest->Next;
	}

	LeaveFunction("Userspace");
	return 0;
}

void StopUserspace(const int CPUNR)
{
	struct http_request *CurrentRequest;

	EnterFunction("StopUserspace");
	CurrentRequest = threadinfo[CPUNR].UserspaceQueue;

	while (CurrentRequest!=NULL)
	{
		CleanUpRequest(CurrentRequest);
		CurrentRequest=CurrentRequest->Next;
	}
	threadinfo[CPUNR].UserspaceQueue = NULL;

	LeaveFunction("StopUserspace");
}

extern struct sock *tcp_v4_lookup_listener(u32 daddr, unsigned short hnum, int dif);


static struct sock *FindUserspace(const unsigned short Port)
{
	EnterFunction("FindUserspace");

	return tcp_v4_lookup_listener(INADDR_ANY,Port,0);

	return NULL;
}

static void dummy_destructor(/*@unused@*/struct open_request *req)
{
}

static struct or_calltable Dummy =
{
 	NULL,
 	&dummy_destructor,
 	NULL
};

#define BACKLOG(sk) ((sk)->tp_pinfo.af_tcp.syn_backlog) /* lvalue! */
#define BACKLOGMAX(sk) sysctl_max_syn_backlog

static int AddSocketToAcceptQueue(struct socket *sock,const int Port)
{
	struct open_request *req;
	struct sock *sk;
	struct tcp_opt *tp;

	EnterFunction("AddSocketToAcceptQueue");


	sk = FindUserspace((unsigned short)Port);
	lock_sock(sk);

	if (BACKLOG(sk)>128)
	{
		printk("Backlog exceeded! \n");
		return -1;
	}

	if (sk==NULL)
	{
		return -1;
	}
	req = tcp_openreq_alloc();
	if (req==NULL)
	{
		return -1;
	}

	req->sk		= sock->sk;
	if (req->sk==NULL)
	  (void)printk(KERN_CRIT "kHTTPd: Woops, the socket-buffer is NULL \n");
	req->class	= &Dummy;
	req->sk->socket = NULL;


	tp =&(sk->tp_pinfo.af_tcp);
	tcp_synq_queue(tp,req);
	sk->ack_backlog++;
	tcp_inc_slow_timer(TCP_SLT_SYNACK);

	release_sock(sk);

	wake_up_interruptible(sk->sleep);


  /* FIXME!: Now find a way to free the socket without freeing the sock */

	if (sock->fasync_list)
		(void)printk(KERN_ERR "sock_release: fasync list not empty!\n");

/*	--sockets_in_use;*/	/*  NOT AVAILABLE TO MODULES!!!*/
	sock->file=NULL;
	iput(sock->inode);
	sock->sk = NULL;

	LeaveFunction("AddSocketToAcceptQueue");

	return +1;



}

