/* libhttpd.c - HTTP protocol library
**
** Copyright  1995,1998,1999,2000,2001 by Jef Poskanzer <jef@acme.com>.
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
*/
//typedef int bool;
//#include "../include/glb.h"
//#include "../include/client.h"
#define AUTH_IN_SERVER	1

#include "../include/cs_file_lock.h"

#define HRPC_ID_VS_GET_SYSTEM_INFO	42
#define HRPC_ID_VS_PING	98
#define HRPC_ID_VS_GET_HARDWARE_RESET 102
#define HRPC_ID_VS_GET_USER_ADVANCE_INFO 118
#define HRPC_ID_VS_IS_SUPPORT_LOCAL_STORAGE	155
#define HRPC_ID_VS_DISK_STATUS_GET	156
#define HRPC_ID_VS_DISK_FORMAT_START	157
#define HRPC_ID_VS_DISK_FORMAT_STOP	158
#define HRPC_ID_VS_DISK_CHECK_START	159
#define HRPC_ID_VS_DISK_CHECK_STOP	160
#define HRPC_ID_VS_DISK_AVAILABLE_SPACE_GET	161

#include <stdio.h>
#include <dlfcn.h>

#include "config.h"
#include "version.h"

#ifdef SHOW_SERVER_VERSION
#define EXPOSED_SERVER_SOFTWARE SERVER_SOFTWARE
#else /* SHOW_SERVER_VERSION */
#define EXPOSED_SERVER_SOFTWARE "thttpd"
#endif /* SHOW_SERVER_VERSION */

#include <sys/types.h>
#include <sys/param.h>
#include <sys/wait.h>
#include <sys/stat.h>

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <time.h>
#ifdef HAVE_MEMORY_H
#include <memory.h>
#endif /* HAVE_MEMORY_H */
#include <pwd.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>
#include <stdarg.h>

#ifdef HAVE_OSRELDATE_H
#include <osreldate.h>
#endif /* HAVE_OSRELDATE_H */

#ifdef HAVE_DIRENT_H
# include <dirent.h>
# define NAMLEN(dirent) strlen((dirent)->d_name)
#else
# define dirent direct
# define NAMLEN(dirent) (dirent)->d_namlen
# ifdef HAVE_SYS_NDIR_H
#  include <sys/ndir.h>
# endif
# ifdef HAVE_SYS_DIR_H
#  include <sys/dir.h>
# endif
# ifdef HAVE_NDIR_H
#  include <ndir.h>
# endif
#endif

#ifdef HAVE_LINUX_SENDFILE
#include <sys/sendfile.h>
#endif
#ifdef HAVE_BSD_SENDFILE
#include <sys/uio.h>
#endif

extern char* crypt( const char* key, const char* setting );

#include "libhttpd.h"
#include "mmc.h"
//#include "timers.h"
#include "match.h"
#include "tdate_parse.h"

#define ADD_HISTORY_WITH_ACTIVE 1

#ifdef STATISTICS
#ifdef ADD_HISTORY_WITH_ACTIVE
#include <sys/shm.h>
#endif
#include <userManager.h>
#endif
#if defined(NAS) || defined(DVR)
#include <config.h>
#include <NAS.h>
#include <Util.h>
#endif
#ifdef DVR
#include <eventlog.h>
#endif

#ifndef STDIN_FILENO
#define STDIN_FILENO 0
#endif
#ifndef STDOUT_FILENO
#define STDOUT_FILENO 1
#endif
#ifndef STDERR_FILENO
#define STDERR_FILENO 2
#endif

#ifndef max
#define max(a,b) ((a) > (b) ? (a) : (b))
#endif
#ifndef min
#define min(a,b) ((a) < (b) ? (a) : (b))
#endif

#ifndef	SHUT_WR
#define	SHUT_WR	1		/* for shutdown(2) */
#endif

#ifdef ALLOW_ACCESS_GRP
#define S_IRACC		S_IRGRP
#define S_IWACC		S_IWGRP
#define S_IXACC		S_IXGRP
#else
#define S_IRACC		S_IROTH
#define S_IWACC		S_IWOTH
#define S_IXACC		S_IXOTH
#endif

/* Forwards. */
static void child_reaper( ClientData client_data, struct timeval* nowP );
static int do_reap( void );
static void check_options( void );
static void free_httpd_server( httpd_server* hs );
static int initialize_listen_socket( httpd_sockaddr* saP );
static void unlisten( httpd_server* hs );
static void add_response( httpd_conn* hc, char* str );
static void send_mime( httpd_conn* hc, int status, char* title, char* encodings, char* extraheads, char* type, int length, time_t mod );
static void send_response( httpd_conn* hc, int status, char* title, char* extraheads, char* form, char* arg );
static void send_response_tail( httpd_conn* hc );
static void defang( char* str, char* dfstr, int dfsize );
#ifdef ERR_DIR
static int send_err_file( httpd_conn* hc, int status, char* title, char* extraheads, char* filename, char* arg );
#endif /* ERR_DIR */
#ifdef AUTH_FILE
static void send_authenticate( httpd_conn* hc, char* realm, int status );
static int b64_decode( const char* str, unsigned char* space, int size );
static int auth_check( httpd_conn* hc, char* dirname  );
static int auth_check2( httpd_conn* hc, char* dirname  );
#endif /* AUTH_FILE */
//void send_redirect( httpd_conn* hc, char *encodedurl );
static void send_dirredirect( httpd_conn* hc );
static int hexit( char c );
static void strdecode( char* to, char* from );
#ifdef GENERATE_INDEXES
static void strencode( char* to, int tosize, char* from );
#endif /* GENERATE_INDEXES */
#ifdef TILDE_MAP_1
static int tilde_map_1( httpd_conn* hc );
#endif /* TILDE_MAP_1 */
#ifdef TILDE_MAP_2
static int tilde_map_2( httpd_conn* hc );
#endif /* TILDE_MAP_2 */
static int vhost_map( httpd_conn* hc );
static char* expand_symlinks( char* path, char** restP, int no_symlink, int tildemapped );
static int httpd_request_reset0( httpd_conn* hc );
static char* bufgets( httpd_conn* hc );
static void de_dotdot( char* file );
static unsigned int hash_mime( char *buf, size_t len );
static void init_mime( void );
static void figure_mime( httpd_conn* hc );
#ifdef CGI_TIMELIMIT
static void cgi_kill2( ClientData client_data, struct timeval* nowP );
static void cgi_kill( ClientData client_data, struct timeval* nowP );
#endif /* CGI_TIMELIMIT */
#ifdef GENERATE_INDEXES
static off_t ls( httpd_conn* hc );
#endif /* GENERATE_INDEXES */
static char* build_env( char* fmt, char* arg );
#ifdef SERVER_NAME_LIST
static char* hostname_map( char* hostname );
#endif /* SERVER_NAME_LIST */
static char** make_envp( httpd_conn* hc );
static char** make_argp( httpd_conn* hc );
static void cgi_interpose_input( httpd_conn* hc, int wfd );
static void post_post_garbage_hack( httpd_conn* hc );
static void cgi_interpose_output( httpd_conn* hc, int rfd );
static void cgi_child( httpd_conn* hc );
static off_t cgi( httpd_conn* hc );
static int really_start_request( httpd_conn* hc, struct timeval* nowP,
		int numconn, int maxconn );
static void make_log_entry( httpd_conn* hc, struct timeval* nowP );
static int check_referer( httpd_conn* hc );
static int really_check_referer( httpd_conn* hc );
static int sockaddr_check( httpd_sockaddr* saP );
static size_t sockaddr_len( httpd_sockaddr* saP );
static int my_snprintf( char* str, size_t size, const char* format, ... );

#ifdef USE_LAYOUT
/* Layout external Vars, see thttpd.c for original declaration */
extern char *lheaderfile, *lfooterfile;
extern int lheaderfile_len, lfooterfile_len;
extern char *lheaderfile_map, *lfooterfile_map;
#endif /* USE_LAYOUT */

#if defined(NAS) || defined(DVR)
static void idle_check( ClientData client_data, struct timeval* nowP );

extern int Open_File(char *, int, FILE **);
extern int Close_File(char *, int, FILE *);

pwdcheck g_pwdcheck[MAX_CONNECT];
int g_total;
//char remoteuser[33];
//char remoteuserip[16];
#define LIBHTTPD_LOG_LOCK "libhttpd.lock"
#define LIBHTTPD_LOG_TIMEOUT 2

int __security_level=0;

//#include "../include/partition.h"
#include <stdarg.h>
void DebugOutput(char *msg, ...)
{
}

void DebugOutput2(char *msg, ...)
{
#if 0
	FILE *dout;
	char buf[2048];
	va_list args;
	CSF_LOCK_RET csflock_ret;
	CSF_LOCK_RET csfunlock_ret;

	va_start(args, msg);
	vsnprintf(buf, sizeof(buf) - 1, msg, args);
	va_end(args);
	dout = fopen("/mnt/r/tmp/thttpd.debug", "a");

	csflock_ret = CSF_Lock(LIBHTTPD_LOG_LOCK, LIBHTTPD_LOG_TIMEOUT);
	if (csflock_ret.status == CSF_LOCK_TIMEOUT)
	{
		//PRINTF("<%u> %s DISK_LOCK TIMEOUT (status=%d)\n", getpid(), "[DebugOutput LOCK]" , csflock_ret.status);
		return;
	}
	else if (csflock_ret.status != CSF_LOCK_OK)
	{
		//PRINTF("<%u> %s DISK_LOCK FAILURE (status=%d)\n", getpid(), "[DebugOutput LOCK]" , csflock_ret.status);
		return;
	}
	//PRINTF("<%u> %s DISK_LOCK OK\n", getpid(), CONTEXT);

	fputs(buf, dout);

	csfunlock_ret = CSF_Unlock(LIBHTTPD_LOG_LOCK, csflock_ret.id);
	if (csfunlock_ret.status == CSF_UNLOCK_FAIL)
	{
		//PRINTF("<%u> %s DISK_UNLOCK FAILURE\n", getpid(), "[DebugOutput LOCK]");
	}else
	{
		//PRINTF("<%u> %s DISK_UNLOCK OK\n", getpid(), "[DebugOutput LOCK]");
	}
	fclose(dout);
#endif
	return;
}
#endif

static int reap_time;

#ifdef STATISTICS
#ifdef ADD_HISTORY_WITH_ACTIVE
extern int shm_refresh_record();
extern int shm_lock(USER_HEADER *header);
extern int find_user(USER_INFO *user, USER_INFO *new, int total);
extern int free_user(USER_INFO *user, USER_INFO *new, int total);
extern void shm_unlock(USER_HEADER *header);

#ifdef AUTH_IN_SERVER
// do authentication check
#define MAX_AUTH_TAB_SIZE	128
#define MAX_CHANNEL_NO		4

static struct auth_tab_st
{
    char usr_name[128];
    char auth[256];
    char used;
    char perm;
} __auth_tab[MAX_AUTH_TAB_SIZE];

static char ocx_auth_check(httpd_conn* hc, char *authinfo);

static int do_auth_for_live_stream(httpd_conn* hc)
{
    char *p;
    int i;
    int auth_idx=-1;
    // do authentication check
    FILE *file=fopen( "/var/usr_auth", "r");

//DebugOutput2("enter!\n");
    if(file)
    {
        // invalidate all entries in __auth_tab[]
        for( i=0; i<MAX_AUTH_TAB_SIZE; i++ )
	{
		__auth_tab[i].used=0;
		__auth_tab[i].auth[0]='\0';
	}

        fclose(file);
	unlink("/var/usr_auth");
//DebugOutput2("rm /var/usr_auth!\n");
    }
    file=fopen( "/var/stm_perm", "r");
    if(file)
    {
        // invalidate all entries in __auth_tab[]
        for( i=0; i<MAX_AUTH_TAB_SIZE; i++ )
	{
		__auth_tab[i].used=0;
		__auth_tab[i].auth[0]='\0';
	}		

        fclose(file);
	unlink("/var/stm_perm");
//DebugOutput2("rm /var/stm_perm!\n");
    }
    if( hc->authorization[0]!='\0' )
    {
        p=hc->authorization+6;
        for( i=0; i<MAX_AUTH_TAB_SIZE; i++ )
        {
            if( __auth_tab[i].used && !strcmp( __auth_tab[i].auth, p))
	    {
	        auth_idx=i;
	        break;
	    }
        }
    }
    if( auth_idx<0 )
    {
	char usr_name[128];
        char perm=ocx_auth_check(hc, usr_name );
	if( !perm )
	{
            //DebugOutput2("no permission for [%s]\n", usr_name);
	    return -1;
	}
        for( i=0; i<MAX_AUTH_TAB_SIZE; i++ )
        {
            if( !__auth_tab[i].used )
            {
                __auth_tab[i].used=1;
                strcpy( __auth_tab[i].auth, p);
                strcpy( __auth_tab[i].usr_name, usr_name);
	        __auth_tab[i].perm=perm;
                //DebugOutput2("permission for [%s]=%X\n", usr_name, perm);
	        return perm&(1<<hc->current_ch)?0:-1;
	    }
        }
    }
    else
    {
        //DebugOutput2("fast permission %X for ch-%d\n", __auth_tab[auth_idx].perm, hc->current_ch);
	//DebugOutput2("auth_idx=%d, perm=%X\n", auth_idx, __auth_tab[auth_idx].perm );
        return __auth_tab[auth_idx].perm&(1<<hc->current_ch)?0:-1;
    }
//DebugOutput2("leave!\n");
    return -1;
}

#endif
int httpd_add_user(USER_INFO *new, httpd_conn* hc)
{
	int 		find = 0, shmid, ret, max_record;
        char 		*shm;
	USER_HEADER	*header;
	USER_INFO	*user;
	time_t		t;

	shm_refresh_record();
        shmid=shmget((key_t)SHMKEY, SHMSIZE, 0666);
        if (shmid==-1)
		return SHM_FAIL;
        shm=shmat(shmid, (char *)0, 0);
        if (shm==(char *)-1)
		return SHM_FAIL;
	header=(USER_HEADER *)shm;
	if (shm_lock(header)==SHM_FAIL)
		return SHM_ADD_FAIL;
	user=(USER_INFO *)(shm+sizeof(USER_HEADER));
	max_record=(SHMSIZE-sizeof(USER_HEADER))/sizeof(USER_INFO)-1;
	if ((ret=find_user(user, new, header->totalrecord))!=SHM_FAIL)
	{
		find = 1;
		user=user+ret;
		new->index=ret;
		goto copy_user;
	}
	else
	if ((ret=free_user(user, new, header->totalrecord))!=SHM_FAIL)
	{
		if (ret>max_record)
			goto full;
		user=user+ret;
		new->index=ret;
		goto copy_user;
	}
	if (header->totalrecord>SHMSIZE/sizeof(USER_INFO))	/* check if share memory is enough? */
		return SHM_NOT_ENOUGH;
	user=user+header->totalrecord;
	new->index=header->totalrecord;
	t=time(NULL);
	new->logintime=new->lasttime=t;
	header->totalrecord++;
copy_user:
	memcpy(user, new, sizeof(USER_INFO));
	shm_unlock(header);
	shmdt(shm);
        ret = SHM_SUCCESS;
        goto add_history;
full:
	shm_unlock(header);
	shmdt(shm);
	ret = SHM_FULL;
add_history:
	if (find == 0 && hc->referer[0] == '\0') {
#ifdef DVR
		pid_t pid = fork();

		if (pid == 0) { // do we need another extra timer to kill this child process if Open_Event_Log hanged???
			int hlog = -1, hlog1;
			char *plogstr[1];

			if ((plogstr[0] = (char *) malloc(128)) != NULL) {
				sprintf(plogstr[0], "00:%s:%s", hc->remoteuser, httpd_ntoa(&hc->client_addr));
				if ((hlog = Open_Event_Log(0)) > 0)
					Close_Event_Log(hlog);
				//DebugOutput("log file: %d\n", hlog);
				hlog1 = Open_Event_Log(1);
				//DebugOutput("log file1: %d\n", hlog1);
				Report_Event(hlog1, EVENTLOG_INFORMATION_TYPE, 1, plogstr);
					//DebugOutput("add historical user: %s (%s)\n", hc->remoteuser, httpd_ntoa(&hc->client_addr));
				Close_Event_Log(hlog1);
				if (hlog == 0)
					close(hlog);
				free(plogstr[0]);
			}
			exit(0);
		}
#endif
	}
	return ret;
}
#endif
void add_httpd_user(httpd_conn* hc)
{
        USER_INFO user;
#ifdef NAS
	char *ptr;
	SHARE_NAME_LIST sh_name_list[5];
#endif
	if (strlen(hc->remoteuser) == 0)
		return;
#ifdef DVR
	if (hc->mode == 0)
		return;
#endif
#ifdef NAS
	if (!strncmp(hc->encodedurl, "/cgi-bin/filemanager/share/", 27))
#endif
//#ifdef DVR
//	if (hc->send_jpeg_buffer == 1)
//#endif
	{	/* download */
		strcpy(user.username, hc->remoteuser);
        	strcpy(user.ip, httpd_ntoa(&hc->client_addr));
#ifdef NAS
		ptr = hc->encodedurl;
		ptr = ptr + 20;
                Get_Nas_Share_List_For_Path(ptr, sh_name_list, 5, 1);
		strcpy(user.sharename, sh_name_list[0].share_name);
#else
		sprintf(user.sharename, "%d", hc->mode);
#endif
        	user.app_id = shm_get_app_id("thttpd");
		user.schedule = SCHEDULE_TIMEOUT;
		user.pid = 0;
#ifdef ADD_HISTORY_WITH_ACTIVE
	        httpd_add_user(&user, hc);
#else
		shm_add_user(&user);
#endif
	}
}
#endif

#if defined(NAS) || defined(DVR)
#if 0
void DEBUG_connecttion(httpd_conn* hc)
{
	DebugOutput("server->binding_hostname=[%s]\n", hc->hs->binding_hostname);
	DebugOutput("server->server_hostname=[%s]\n", hc->hs->server_hostname);
	DebugOutput("server->cgi_pattern=[%s]\n", hc->hs->cgi_pattern);
	DebugOutput("server->charset=[%s]\n", hc->hs->charset);
	DebugOutput("server->cwd=[%s]\n", hc->hs->cwd);
	DebugOutput("server->url_pattern=[%s]\n", hc->hs->url_pattern);
	DebugOutput("server->local_pattern=[%s]\n", hc->hs->local_pattern);
	DebugOutput("read_buf=[%s]\n", hc->read_buf);
	DebugOutput("method=[%d]\n", hc->method);
	DebugOutput("status=[%d]\n", hc->status);
	DebugOutput("encodedurl=[%s]\n", hc->encodedurl);
	DebugOutput("decodedurl=[%s]\n", hc->decodedurl);
	DebugOutput("protocol=[%s]\n", hc->protocol);
	DebugOutput("origfilename=[%s]\n", hc->origfilename);
	DebugOutput("expnfilename=[%s]\n", hc->expnfilename);
	DebugOutput("encodings=[%s]\n", hc->encodings);
	DebugOutput("pathinfo=[%s]\n", hc->pathinfo);
	DebugOutput("query=[%s]\n", hc->query);
	DebugOutput("referer=[%s]\n", hc->referer);
	DebugOutput("useragent=[%s]\n", hc->useragent);
	DebugOutput("accept=[%s]\n", hc->accept);
	DebugOutput("accepte=[%s]\n", hc->accepte);
	DebugOutput("cookie=[%s]\n", hc->cookie);
	DebugOutput("contenttype=[%s]\n", hc->contenttype);
	DebugOutput("reqhost=[%s]\n", hc->reqhost);
	DebugOutput("hdrhost=[%s]\n", hc->hdrhost);
	DebugOutput("hostdir=[%s]\n", hc->hostdir);
	DebugOutput("authrization=[%s]\n", hc->authorization);
	DebugOutput("remoteuser=[%s]\n", hc->remoteuser);
	DebugOutput("response=[%s]\n", hc->response);
	DebugOutput("mime_flag=[%d]\n", hc->mime_flag);
	DebugOutput("conn_fd=[%d]\n", hc->conn_fd);
	DebugOutput("file_address=[%s]\n", hc->file_address);
}
#endif
#if 0
double timeGet(void)
{
  	struct timeval tv;
  	double curtime;

  	/* get wallclock time */
  	gettimeofday(&tv, NULL);
  	curtime = (double) tv.tv_sec + 1.0e-6 * (double) tv.tv_usec;
  	return curtime;
}
#endif
#ifdef DVR
void save_jpeg_file(unsigned char *jpeg_data, int size, int current_card, int current_ch, int count, int time)
{
#if 0
	FILE *output_file = NULL;
   	char outfilename[64];

	sprintf(outfilename, "/home/kevin/test%d-%d-%d-%d.jpg", current_card, current_ch, count, time);
   	if ((output_file = fopen(outfilename, "w")) == NULL)
   		return;
        fwrite(jpeg_data, size, 1, output_file);
        fclose(output_file);
#endif
	return;
}
#endif
#endif

static void
child_reaper( ClientData client_data, struct timeval* nowP )
    {
    int child_count;
    static int prev_child_count = 0;

    child_count = do_reap();

    /* Reschedule reaping, with adaptively changed time. */
    if ( child_count > prev_child_count * 3 / 2 )
	reap_time = max( reap_time / 2, MIN_REAP_TIME );
    else if ( child_count < prev_child_count * 2 / 3 )
	reap_time = min( reap_time * 5 / 4, MAX_REAP_TIME );
    if ( tmr_create( nowP, child_reaper, JunkClientData,
		reap_time * 1000L, 0 ) == (Timer*) 0 )
	{
	syslog( LOG_CRIT, "tmr_create(child_reaper) failed" );
	exit( 100 );
	}
    }

static int
do_reap( void )
    {
    int child_count;
    pid_t pid;
    int status;

    /* Reap defunct children until there aren't any more. */
    for ( child_count = 0; ; ++child_count )
	{
#ifdef HAVE_WAITPID
	pid = waitpid( (pid_t) -1, &status, WNOHANG );
#else /* HAVE_WAITPID */
	pid = wait3( &status, WNOHANG, (struct rusage*) 0 );
#endif /* HAVE_WAITPID */
	if ( (int) pid == 0 )           /* none left */
	    break;
	if ( (int) pid < 0 )
	    {
	    if ( errno == EINTR )       /* because of ptrace */
		continue;
	    /* ECHILD shouldn't happen with the WNOHANG option, but with
	    ** some kernels it does anyway.  Ignore it.
	    */
	    if ( errno != ECHILD )
		syslog( LOG_ERR, "waitpid - %m" );
	    break;
	    }
	}
    return child_count;
    }


static void
check_options( void )
    {
#if defined(TILDE_MAP_1) && defined(TILDE_MAP_2)
    syslog( LOG_CRIT, "both TILDE_MAP_1 and TILDE_MAP_2 are defined" );
    exit( 101 );
#endif /* both */
    }


static void
free_httpd_server( httpd_server* hs )
    {
    if ( hs->binding_hostname != (char*) 0 )
	free( (void*) hs->binding_hostname );
    if ( hs->cwd != (char*) 0 )
	free( (void*) hs->cwd );
    if ( hs->cgi_pattern != (char*) 0 )
	free( (void*) hs->cgi_pattern );
    if ( hs->charset != (char*) 0 )
	free( (void*) hs->charset );
    if ( hs->url_pattern != (char*) 0 )
	free( (void*) hs->url_pattern );
    if ( hs->local_pattern != (char*) 0 )
	free( (void*) hs->local_pattern );
#ifdef DVR
	if (hs->prules) {
		free(hs->prules);
		hs->prules = NULL;
		hs->rule_count = 0;
	}
#endif
    free( (void*) hs );
    }
#ifdef DVR
void load_ip_security_rules(httpd_server* hs)
{
	int type = Is_IPSEC_Enabled();

	if (hs->prules)
		free(hs->prules);
	hs->prules = NULL;
	hs->rule_count = 0;
	if (type == 1) {
		Load_All_IPSEC_Rules(NULL, &hs->rule_count);
		if (hs->rule_count > 0 && (hs->prules = (IPSEC_RULE_EX*) malloc(hs->rule_count * sizeof(IPSEC_RULE_EX))) != NULL && Load_All_IPSEC_Rules(hs->prules, &hs->rule_count) != 0) {
			free(hs->prules);
			hs->prules = NULL;
			hs->rule_count = 0;
		}
		hs->rule_type = 1;
	}
	else if (type == 2) {
		Load_All_IPSEC_Deny_Rules(NULL, &hs->rule_count);
		if (hs->rule_count > 0 && (hs->prules = (IPSEC_RULE_EX*) malloc(hs->rule_count * sizeof(IPSEC_RULE_EX))) != NULL && Load_All_IPSEC_Deny_Rules(hs->prules, &hs->rule_count) != 0) {
			free(hs->prules);
			hs->prules = NULL;
			hs->rule_count = 0;
		}
		hs->rule_type = 2;
	}
	return;
}
#endif
#if defined(NAS) || defined(DVR)
static int set_user_ip_login(char *name, char *ip, int timer_index, int mode)
{
	if (timer_index >= 0)
		timer_index = ((timer_index << 4) + mode);
	return Set_Private_Profile_Integer(name, ip, timer_index, CHECK_USER_CONF);
}

static int my_strip_EOL(char *str)
{
	if (str[strlen(str) - 1] == '\n') {
		str[strlen(str) - 1] = '\0';
		if (str[strlen(str) - 1] == '\r')
			str[strlen(str) - 1] = '\0';
		return 1;
	}
	return 0;
}

static char *my_skip_white_space(char *str)
{
	char *p = str;

	while (*p == ' ' || *p == '\t')
		p++;
	return p;
}

static char *my_stripe_white_space_tail(char *str)
{
	if (strlen(str) > 0) {
		char *p = str + strlen(str) - 1;

		while (*p == ' ' || *p == '\t')
			p--;
		*(p + 1) = '\0';
	}
	return str;
}

static int get_string_by_delimiter(char *str, char ch, char *buf, int buf_size, char *buf1, int buf_size1)
{
	char *p = strrchr(str, ch), *q;
	int i, j;

	if (p == NULL)
		return -1;
	if ((i = (int) (p - str)) >= buf_size)
		i = buf_size - 1;
	if (buf1 != NULL) {
		if ((p + 1) == NULL)
			return -1;
		q = my_skip_white_space(p + 1);
		if ((j = (strlen(str) - (int) (q - str))) >= buf_size1)
			j = buf_size1 - 1;
		strncpy(buf1, q, j);
		buf1[j] = '\0';
		my_stripe_white_space_tail(buf1);
	}
	strncpy(buf, str, i);
	buf[i] = '\0';
	my_stripe_white_space_tail(buf);
	return 0;
}
#if 0
static void remove_all_ip(char *ip, char *name, char *from, char *to)
{
	FILE *fp = NULL, *tmpfp = NULL;
	char buf[128], field_name[20], value[20], user_name[64];
	int i, dirty = 0;

	if (Open_File(CHECK_USER_CONF, O_RDONLY, &fp) != SUCCESS)
		return;
	rewind(fp);
	tmpfp = tmpfile();
	if (NULL != fp) {
		while (fgets(buf, sizeof(buf), fp) != NULL && my_strip_EOL(buf) == 1) {
			char *q = my_skip_white_space(buf);

			if (*q == '[') {
				if (name != NULL && get_string_by_delimiter(q + 1, ']', user_name, sizeof(user_name), NULL, 0) != 0)
					strcpy(user_name, "");
			}
			else if (*q != '#' && *q != ';') {
				if (get_string_by_delimiter(q, '=', field_name, sizeof(field_name), value, sizeof(value)) == 0) {
					if (strcasecmp(field_name, ip) == 0) {
						if (name != NULL && strcasecmp(name, user_name) == 0) {
							dirty = 1;
							continue;
						}
						else if (strcasecmp(value, from) == 0) {
							sprintf(buf, "%s = %s", ip, to);
							fputs(buf, tmpfp);
							fputs("\n", tmpfp);
							dirty = 1;
							continue;
						}
					}
				}
			}
			fputs(q, tmpfp);
			fputs("\n", tmpfp);
		}
		Close_File(CHECK_USER_CONF, O_RDONLY, fp);
		if (dirty) {
			if (Open_File(CHECK_USER_CONF, O_WRONLY, &fp) == SUCCESS) {
				rewind(tmpfp);
				while ((i = fread(buf, 1, sizeof(buf), tmpfp)) > 0)
					fwrite(buf, 1, i, fp);
				Close_File(CHECK_USER_CONF, O_WRONLY, fp);
			}
		}
		fclose(tmpfp);
	}
	return;
}
#endif
#if 0
static void remove_user_ip(char *name, char *ip)
{
	int timer_index = Get_Private_Profile_Integer(name, ip, -1, CHECK_USER_CONF);

	if (timer_index == -2)
		Conf_Remove_Field(CHECK_USER_CONF, name, ip);
	return;
}
#endif
static void get_user_ip_login(char *name, char *ip, int *timer_index, int *mode)
{
	if ((*timer_index = Get_Private_Profile_Integer(name, ip, -1, CHECK_USER_CONF)) >= 0) {
		*mode = (0x0000000f & (*timer_index));
		*timer_index = (*timer_index >> 4);
	}
	else
		*mode = 0;
	return ;
}

static void idle_check(ClientData client_data, struct timeval* nowP)
{
	int index = client_data.i;

	//DebugOutput("logout: %s %s, timer: 0x%x\n", g_pwdcheck[index].name, g_pwdcheck[index].ip, g_pwdcheck[index].idle_timer);
	set_user_ip_login(g_pwdcheck[index].name, g_pwdcheck[index].ip, -2, 0);
	g_pwdcheck[index].idle_timer = (Timer *) 0;
	g_total--;
	shm_del_user(g_pwdcheck[index].name, g_pwdcheck[index].ip, shm_get_app_id("thttpd"));
	return;
}

static void init_check_user()
{
	char name[64], ip[32], text2[20], buf[128];
	struct stat status;
	FILE *fp = NULL;
	int i, index, mode;
	struct timeval tv;
	ClientData client_data;
	USER_INFO user;

	//DebugOutput("init_check_user\n");
	memset(g_pwdcheck, 0, sizeof(g_pwdcheck));
	g_total = 0;
	//if (0 != stat(SESSION_ID_LOG_FILE, &status)) {
	//	sprintf(buf, "echo > %s", SESSION_ID_LOG_FILE);
	//	system(buf);
	//}
	if (0 != stat(CHECK_USER_CONF, &status)) {
		sprintf(buf, "echo > %s", CHECK_USER_CONF);
		system(buf);
		return;
	}
	if ((i = Open_File(CHECK_USER_CONF, O_RDONLY, &fp)) != SUCCESS)
		return;
	while (fgets(buf, sizeof(buf), fp) != NULL && my_strip_EOL(buf) == 1) {
		char *q = my_skip_white_space(buf);

		if (*q == '[') {
			if (get_string_by_delimiter(q + 1, ']', name, sizeof(name), NULL, 0) != 0)
				strcpy(name, "");
		}
		else if (*q != '#' && *q != ';') {
			if (get_string_by_delimiter(q, '=', ip, sizeof(ip), text2, sizeof(text2)) == 0) {
				//DebugOutput("name = %s, ip = %s, value = %s\n", name, ip, text2);
				if ((index = atoi(text2)) >= 0 && (mode = (0x0000000f & index)) != 0) {
					index = (index >> 4);
					strncpy(g_pwdcheck[index].name, name, sizeof(g_pwdcheck[index].name));
					strncpy(g_pwdcheck[index].ip, ip, sizeof(g_pwdcheck[index].ip));
					(void) gettimeofday(&tv, (struct timezone*) 0);
					client_data.i = index;
					g_pwdcheck[index].idle_timer = tmr_create((struct timeval*) 0, idle_check, client_data, MAX_TIME * 1000L, 0);
					g_total++;
					strncpy(user.username, name, sizeof(user.username));
					strncpy(user.ip, ip, sizeof(user.ip));
					sprintf(user.sharename, "%d", mode);
       					user.app_id = shm_get_app_id("thttpd");
					user.schedule = SCHEDULE_TIMEOUT;
					user.pid = 0;
					shm_add_user(&user);
				}
			}
		}
	}
	Close_File(CHECK_USER_CONF, O_RDONLY, fp);
	return;
}
#endif
httpd_server*
httpd_initialize(
    char* hostname, httpd_sockaddr* sa4P, httpd_sockaddr* sa6P, int port,
    char* cgi_pattern, char* charset, char* cwd, int no_log, FILE* logfp,
    int no_symlink, int vhost, int global_passwd, char* url_pattern,
    char* local_pattern, int no_empty_referers )
    {
    httpd_server* hs;
    static char ghnbuf[256];
    char* cp;

    check_options();

    /* Set up child-process reaper. */
    reap_time = min( MIN_REAP_TIME * 4, MAX_REAP_TIME );
    if ( tmr_create( (struct timeval*) 0, child_reaper, JunkClientData, reap_time * 1000L, 0 ) == (Timer*) 0 )
	{
	syslog( LOG_CRIT, "tmr_create(child_reaper) failed" );
	return (httpd_server*) 0;
	}

    hs = NEW( httpd_server, 1 );
    if ( hs == (httpd_server*) 0 )
	{
	syslog( LOG_CRIT, "out of memory allocating an httpd_server" );
	return (httpd_server*) 0;
	}

    if ( hostname != (char*) 0 )
	{
	hs->binding_hostname = strdup( hostname );
	if ( hs->binding_hostname == (char*) 0 )
	    {
	    syslog( LOG_CRIT, "out of memory copying hostname" );
	    return (httpd_server*) 0;
	    }
	hs->server_hostname = hs->binding_hostname;
	}
    else
	{
	hs->binding_hostname = (char*) 0;
	hs->server_hostname = (char*) 0;
	if ( gethostname( ghnbuf, sizeof(ghnbuf) ) < 0 )
	    ghnbuf[0] = '\0';
#ifdef SERVER_NAME_LIST
	if ( ghnbuf[0] != '\0' )
	    hs->server_hostname = hostname_map( ghnbuf );
#endif /* SERVER_NAME_LIST */
	if ( hs->server_hostname == (char*) 0 )
	    {
#ifdef SERVER_NAME
	    hs->server_hostname = SERVER_NAME;
#else /* SERVER_NAME */
	    if ( ghnbuf[0] != '\0' )
		hs->server_hostname = ghnbuf;
#endif /* SERVER_NAME */
	    }
	}

    hs->port = port;
    if ( cgi_pattern == (char*) 0 )
	hs->cgi_pattern = (char*) 0;
    else
	{
	/* Nuke any leading slashes. */
	if ( cgi_pattern[0] == '/' )
	    ++cgi_pattern;
	hs->cgi_pattern = strdup( cgi_pattern );
	if ( hs->cgi_pattern == (char*) 0 )
	    {
	    syslog( LOG_CRIT, "out of memory copying cgi_pattern" );
	    return (httpd_server*) 0;
	    }
	/* Nuke any leading slashes in the cgi pattern. */
	while ( ( cp = strstr( hs->cgi_pattern, "|/" ) ) != (char*) 0 )
	    (void) strcpy( cp + 1, cp + 2 );
	}
    hs->charset = strdup( charset );
    hs->cwd = strdup( cwd );
    if ( hs->cwd == (char*) 0 )
	{
	syslog( LOG_CRIT, "out of memory copying cwd" );
	return (httpd_server*) 0;
	}
    if ( url_pattern == (char*) 0 )
	hs->url_pattern = (char*) 0;
    else
	{
	hs->url_pattern = strdup( url_pattern );
	if ( hs->url_pattern == (char*) 0 )
	    {
	    syslog( LOG_CRIT, "out of memory copying url_pattern" );
	    return (httpd_server*) 0;
	    }
	}
    if ( local_pattern == (char*) 0 )
	hs->local_pattern = (char*) 0;
    else
	{
	hs->local_pattern = strdup( local_pattern );
	if ( hs->local_pattern == (char*) 0 )
	    {
	    syslog( LOG_CRIT, "out of memory copying local_pattern" );
	    return (httpd_server*) 0;
	    }
	}
    hs->no_log = no_log;
    hs->logfp = (FILE*) 0;
    httpd_set_logfp( hs, logfp );
    hs->no_symlink = no_symlink;
    hs->vhost = vhost;
    hs->global_passwd = global_passwd;
    hs->no_empty_referers = no_empty_referers;

    /* Initialize listen sockets.  Try v6 first because of a Linux peculiarity;
    ** unlike other systems, it has magical v6 sockets that also listen for v4,
    ** but if you bind a v4 socket first then the v6 bind fails.
    */
    if ( sa6P == (httpd_sockaddr*) 0 )
	hs->listen6_fd = -1;
    else
	hs->listen6_fd = initialize_listen_socket( sa6P );
    if ( sa4P == (httpd_sockaddr*) 0 )
	hs->listen4_fd = -1;
    else
	hs->listen4_fd = initialize_listen_socket( sa4P );
    /* If we didn't get any valid sockets, fail. */
    if ( hs->listen4_fd == -1 && hs->listen6_fd == -1 )
	{
	free_httpd_server( hs );
	return (httpd_server*) 0;
	}

    init_mime();


#ifdef DVR
	load_ip_security_rules(hs);
#endif
#if defined(NAS) || defined(DVR)
	init_check_user();
#endif
    /* Done initializing. */
    if ( hs->binding_hostname == (char*) 0 )
	syslog( LOG_NOTICE, "%.80s starting on port %d", SERVER_SOFTWARE, hs->port );
    else
	syslog(
	    LOG_NOTICE, "%.80s starting on %.80s, port %d", SERVER_SOFTWARE,
	    httpd_ntoa( hs->listen4_fd != -1 ? sa4P : sa6P ), hs->port );
    return hs;
    }


static int
initialize_listen_socket( httpd_sockaddr* saP )
    {
    int listen_fd;
    int on, flags;

    /* Check sockaddr. */
    if ( ! sockaddr_check( saP ) )
	{
	syslog( LOG_CRIT, "unknown sockaddr family on listen socket" );
	return -1;
	}

    /* Create socket. */
    listen_fd = socket( saP->sa.sa_family, SOCK_STREAM, 0 );
    if ( listen_fd < 0 )
	{
	syslog( LOG_CRIT, "socket %.80s - %m", httpd_ntoa( saP ) );
	return -1;
	}
    (void) fcntl( listen_fd, F_SETFD, 1 );

    /* Allow reuse of local addresses. */
    on = 1;
    if ( setsockopt(
	     listen_fd, SOL_SOCKET, SO_REUSEADDR, (char*) &on,
	     sizeof(on) ) < 0 )
	syslog( LOG_CRIT, "setsockopt SO_REUSEADDR - %m" );

    /* Use accept filtering, if available. */
#ifdef SO_ACCEPTFILTER
    {
#if ( __FreeBSD_version >= 411000 )
#define ACCEPT_FILTER_NAME "httpready"
#else
#define ACCEPT_FILTER_NAME "dataready"
#endif
    struct accept_filter_arg af;
    (void) bzero( &af, sizeof(af) );
    (void) strcpy( af.af_name, ACCEPT_FILTER_NAME );
    (void) setsockopt(
	listen_fd, SOL_SOCKET, SO_ACCEPTFILTER, (char*) &af, sizeof(af) );
    }
#endif /* SO_ACCEPTFILTER */

    /* Bind to it. */
    if ( bind( listen_fd, &saP->sa, sockaddr_len( saP ) ) < 0 )
	{
	syslog(
	    LOG_CRIT, "bind %.80s - %m", httpd_ntoa( saP ) );
	(void) close( listen_fd );
	return -1;
	}

    /* Set the listen file descriptor to no-delay mode. */
    flags = fcntl( listen_fd, F_GETFL, 0 );
    if ( flags == -1 )
	{
	syslog( LOG_CRIT, "fcntl F_GETFL - %m" );
	(void) close( listen_fd );
	return -1;
	}
    if ( fcntl( listen_fd, F_SETFL, flags | O_NDELAY ) < 0 )
	{
	syslog( LOG_CRIT, "fcntl O_NDELAY - %m" );
	(void) close( listen_fd );
	return -1;
	}

    /* Start a listen going. */
    if ( listen( listen_fd, LISTEN_BACKLOG ) < 0 )
	{
	syslog( LOG_CRIT, "listen - %m" );
	(void) close( listen_fd );
	return -1;
	}

    return listen_fd;
    }


void
httpd_set_logfp( httpd_server* hs, FILE* logfp )
    {
    if ( hs->logfp != (FILE*) 0 )
	(void) fclose( hs->logfp );
    hs->logfp = logfp;
    }


void
httpd_terminate( httpd_server* hs )
    {
    unlisten( hs );
    if ( hs->logfp != (FILE*) 0 )
	{
	(void) fclose( hs->logfp );
	hs->logfp = (FILE*) 0;
	}
    free_httpd_server( hs );
    }


static void
unlisten( httpd_server* hs )
    {
    if ( hs->listen4_fd != -1 )
    {
	(void) close( hs->listen4_fd );
	hs->listen4_fd = -1;
    }
    if ( hs->listen6_fd != -1 )
    {
	(void) close( hs->listen6_fd );
	hs->listen6_fd = -1;
    }
    }


/* Conditional macro to allow two alternate forms for use in the built-in
** error pages.  If EXPLICIT_ERROR_PAGES is defined, the second and more
** explicit error form is used; otherwise, the first and more generic
** form is used.
*/
#ifdef EXPLICIT_ERROR_PAGES
#define ERROR_FORM(a,b) b
#else /* EXPLICIT_ERROR_PAGES */
#define ERROR_FORM(a,b) a
#endif /* EXPLICIT_ERROR_PAGES */


static char  ok200title[]  = "OK";
static char  ok206title[]  = "Partial Content";

static char  err302title[] = "Found";
static char  err302form[]  = "The actual URL is '%.80s'.\n";

static char  err304title[] = "Not Modified";

char  httpd_err400title[]  = "Bad Request";
char  httpd_err400form[]   =
    "Your request has bad syntax or is inherently impossible to satisfy.\n";

#ifdef AUTH_FILE
static char  err401title[] = "Unauthorized";
static char  err401form[]  =
#if defined(NAS) || defined(DVR)
#ifdef DVR
	"Authorization fail.\n";
#else
	"<SCRIPT Language=\"JavaScript\">document.write(ERR401_FORM)</script>";
#endif
#else
    "Authorization required for the URL '%.80s'.\n";
#endif
#endif /* AUTH_FILE */

static char  err403title[] = "Forbidden";
#ifndef DVR
static char  err403form[]  =
    "You do not have permission to get URL '%.80s' from this server.\n";
#endif
static char  err404title[] = "Not Found";
static char  err404form[]  =
#ifdef DVR
	"The path or file you requested was not found on the system!\n";
#else
    "The requested URL '%.80s' was not found on this server.\n";
#endif

char  httpd_err408title[] = "Request Timeout";
char  httpd_err408form[]  =
    "No request appeared within a reasonable time period.\n";

static char  err500title[] = "Internal Error";
static char  err500form[]  =
    "There was an unusual problem serving the requested URL '%.80s'.\n";

static char  err501title[] = "Not Implemented";
static char  err501form[]  =
    "The requested method '%.80s' is not implemented by this server.\n";

char  httpd_err503title[] = "Service Temporarily Overloaded";
char  httpd_err503form[] =
#if defined(NAS) || defined(DVR)
	"The system is currently overloaded! Please try again later.\n";
#else
    "The requested URL '%.80s' is temporarily overloaded. Please try again later.\n";
#endif
//static char  err503form2[] =
//    "Overload (%.80s).  Please try again later.\n";

#ifdef NAS
char httpd_err504title[] = "Execute Error";
char httpd_err504form[] = "You can't execute file %s\n";
char httpd_err999title[] = "Web Filemanager";
char httpd_err999form[] = "<SCRIPT Language=\"JavaScript\">document.write(ERR999_FORM)</script>";
#endif
#ifdef DVR
char httpd_err999title[] = "Forbidden";
char httpd_err999form[] = "You do not have enough privilege!\n";
#endif
#if defined(NAS) || defined(DVR)
char httpd_errunknowntitle[] = "Unknown Error";
char httpd_errunknownform[] = "Unknown error!\n";
#endif

/* Append a string to the buffer waiting to be sent as response. */
static void
add_response( httpd_conn* hc, char* str )
    {
    int len;

    len = strlen( str );
    httpd_realloc_str( &hc->response, &hc->maxresponse, hc->responselen + len );
    (void) memcpy( &(hc->response[hc->responselen]), str, len );
    hc->responselen += len;
    }

/* Clear the buffered response */
void
httpd_clear_response( httpd_conn* hc )
    {
	hc->responselen = 0;
    }

/* Send the buffered response in blocking / delay mode */
void
httpd_write_blk_response( httpd_conn* hc )
    {
    /* First turn off NDELAY mode. */
    httpd_clear_ndelay( hc->conn_fd );
    /* send response, if necessary. */
    /* NOTE: caller has eventually to call httpd_set_ndelay() */
    if ( hc->responselen > 0 )
	{
	if (write( hc->conn_fd, hc->response, hc->responselen ) == -1 &&
	    errno == EINTR)
	{
	    (void) write( hc->conn_fd, hc->response, hc->responselen );
	}
	hc->responselen = 0;
	}
    }


/* Set NDELAY mode on a socket. */
void
httpd_set_ndelay( int fd )
    {
    int flags, newflags;

    flags = fcntl( fd, F_GETFL, 0 );
    if ( flags != -1 )
	{
	newflags = flags | (int) O_NDELAY;
	if ( newflags != flags )
	    (void) fcntl( fd, F_SETFL, newflags );
	}
    }


/* Clear NDELAY mode on a socket. */
void
httpd_clear_ndelay( int fd )
    {
    int flags, newflags;

    flags = fcntl( fd, F_GETFL, 0 );
    if ( flags != -1 )
	{
	newflags = flags & ~ (int) O_NDELAY;
	if ( newflags != flags )
	    (void) fcntl( fd, F_SETFL, newflags );
	}
    }


/* return a string about available sendfile().
** NOTE: keep these #defines syncronized with those in httpd_sendfile().
*/

const char *
httpd_typeof_sendfile( void )
    {
#if defined(HAVE_LINUX_SENDFILE)
    return "Linux";
#elif defined(HAVE_BSD_SENDFILE)
    return "BSD";
#else
    return "emulated";
#endif
    }


/* send "bytes" from "fdin" (input file) to "fdout" (output socket)
** starting at input file "offset".
** NOTE: if native implemementation of sendfile() does not work well
**       in non-blocking mode then use emulation by read() + write().
*/
ssize_t
httpd_sendfile( int fdout, int fdin, off_t offset, size_t bytes )
    {

#if defined(HAVE_LINUX_SENDFILE)

    return sendfile( fdout, fdin, &offset, bytes );

#elif defined(HAVE_BSD_SENDFILE)

    off_t nbytes = 0;

    if ( sendfile( fdin, fdout, offset, bytes, NULL, &nbytes, 0 ) == -1 )
	{
	if ( errno == EAGAIN && nbytes > 0 )
	    /* OK, partial write */
	    return nbytes;
	return -1;
	}
    return nbytes;

#else

//#warning using slow sendfile() !

#define IO_BUFSIZE	12288

    char buf[IO_BUFSIZE];
    ssize_t nread;

    if ( lseek( fdin, offset, SEEK_SET) == -1 )
	return -1;

    if (bytes == 0 )
	return 0;

    /* read data */
    nread = read( fdin, buf, (bytes < IO_BUFSIZE) ? bytes : IO_BUFSIZE );
    if (nread <= 0)
	/* -1 or 0 */
	return nread;

    /* write it out */
    /* -1, 0, 1 - nread (a partial write is OK) */
    return write( fdout, buf, nread );

#endif /* SENDFILE */
    }

#if 0
void show_server_time(char *nowbuf)
{
	//time_t t;
  	//struct tm *ptm;
  	char date_time[32], month[3], day[3], year[5], hour[3], min[3], sec[3];

	Get_Current_Date_Time(date_time);
	//DebugOutput("nowbuf = %s\n", date_time);
	memcpy(month, date_time, 2);
	month[2] = 0;
	memcpy(day, &(date_time[2]), 2);
	day[2] = 0;
	memcpy(year, &(date_time[4]), 4);
	year[4] = 0;
	memcpy(hour, &(date_time[8]), 2);
	hour[2] = 0;
	memcpy(min, &(date_time[10]), 2);
	min[2] = 0;
	memcpy(sec, &(date_time[12]), 2);
	sec[2] = 0;
	//time(&t);
	//ptm = localtime(&t);
	sprintf(nowbuf, "%s/%s/%s %s:%s:%s", month, day, year, hour, min, sec);
	//DebugOutput("%s %s %s %s %s %s %s %s\n", date_time, month, day, year, hour, min, sec, nowbuf);
	return;
}
#endif
static void
send_mime( httpd_conn* hc, int status, char* title, char* encodings, char* extraheads, char* type, int length, time_t mod )
    {
    time_t now;
    const char  rfc1123fmt[] = "%a, %d %b %Y %H:%M:%S GMT";
    char nowbuf[100];
    char modbuf[100];
    char fixed_type[500];
    char buf[1000];

#if defined(NAS) || defined(DVR)
	if (status == 450)
		hc->status = 401;
	else
#endif
    hc->status = status;
    hc->bytes_to_send = length;

#ifdef USE_LAYOUT
    if ( length > 0 )
	{
	/* use hdr + ftr only if body is not empty */
	if (hc->layout)
	    { /* are layout modifications needed ? */
	    if ( lheaderfile_len > 0 )
		{
		length += lheaderfile_len;
		hc->lheaderfile_len = lheaderfile_len;
		}
	    else
		hc->lheaderfile_len = 0;
	    if ( lfooterfile_len > 0 )
		{
		length += lfooterfile_len;
		hc->lfooterfile_len = lfooterfile_len;
		}
	    else
		hc->lfooterfile_len = 0;
	    }
	}
#endif /* USE_LAYOUT */

    if ( hc->mime_flag )
	{
	if ( status == 200 && hc->got_range && length > 0 &&
	     ( hc->end_byte_loc >= hc->init_byte_loc ) &&
	     ( ( hc->end_byte_loc != length - 1 ) ||
	       ( hc->init_byte_loc != 0 ) ) &&
	     ( hc->range_if == (time_t) -1 ||
	       hc->range_if == hc->sb.st_mtime ) )
	    {
	    /* partial_content = 1, hc->got_range == 1 */
	    hc->status = status = 206;
	    title = ok206title;
	    }
	else
	    {
	    /* partial_content = 0, disable got_range */
	    hc->got_range = 0;
	    }
#ifdef DVR
	if (hc->send_jpeg_buffer != 0) {
		strcpy(nowbuf, "");
		strcpy(modbuf, "");
		(void) my_snprintf(fixed_type, sizeof(fixed_type), type, "iso8859-1");
	}
	else {
#endif
	now = time( (time_t*) 0 );
	if ( mod == (time_t) 0 )
	    mod = now;
	(void) strftime( nowbuf, sizeof(nowbuf), rfc1123fmt, gmtime( &now ) );
	(void) strftime( modbuf, sizeof(modbuf), rfc1123fmt, gmtime( &mod ) );
#if defined(NAS) || defined(DVR)
	if (strstr(hc->cookie, "+nas_lang=TCH"))
		(void) my_snprintf(fixed_type, sizeof(fixed_type), type, "big5");
	else if (strstr(hc->cookie, "+nas_lang=JPN"))
		(void) my_snprintf(fixed_type, sizeof(fixed_type), type, "Shift_JIS");
	else if (strstr(hc->cookie, "+nas_lang=ENG"))
		(void) my_snprintf(fixed_type, sizeof(fixed_type), type, "iso8859-1");
	else if (strstr(hc->cookie, "+nas_lang=CHS"))
		(void) my_snprintf(fixed_type, sizeof(fixed_type), type, "gb2312");
// Ben Add 2005.02.01 for Polish
	else if (strstr(hc->cookie, "+nas_lang=POL"))
		(void) my_snprintf(fixed_type, sizeof(fixed_type), type, "windows-1250");
// Ben Add 2005.02.01 for Polish
#if 0
	else if (strstr(hc->cookie, "nas_lang=TCH"))
		(void) my_snprintf(fixed_type, sizeof(fixed_type), type, "big5");
	else if (strstr(hc->cookie, "nas_lang=JPN"))
		(void) my_snprintf(fixed_type, sizeof(fixed_type), type, "Shift_JIS");
	else if (strstr(hc->cookie, "nas_lang=ENG"))
		(void) my_snprintf(fixed_type, sizeof(fixed_type), type, "iso8859-1");
	else if (strstr(hc->cookie, "nas_lang=CHS"))
		(void) my_snprintf(fixed_type, sizeof(fixed_type), type, "gb2312");
// Ben Add 2005.02.01 for Polish
	else if (strstr(hc->cookie, "nas_lang=POL"))
		(void) my_snprintf(fixed_type, sizeof(fixed_type), type, "windows-1250");
// Ben Add 2005.02.01 for Polish
#endif
	else
#endif
	(void) my_snprintf(
	    fixed_type, sizeof(fixed_type), type, hc->hs->charset );
#ifdef DVR
	}
#endif

//#if defined(NAS) || defined(DVR)
#if 0
	(void) my_snprintf( buf, sizeof(buf),
		"%.20s %d %s\r\nContent-Type: %s\r\nDate: %s\r\nLast-Modified: %s\r\nAccept-Ranges: bytes\r\n",
		hc->protocol, status, title, fixed_type, nowbuf, modbuf);
#else
	(void) my_snprintf( buf, sizeof(buf),
	    "%.20s %d %s\r\nServer: %s\r\nContent-Type: %s\r\nDate: %s\r\nLast-Modified: %s\r\nAccept-Ranges: bytes\r\n",
	    hc->protocol, hc->status, title, EXPOSED_SERVER_SOFTWARE, fixed_type,
	    nowbuf, modbuf);
#endif
	add_response( hc, buf );
	if ( encodings[0] != '\0' )
	    {
	    (void) my_snprintf( buf, sizeof(buf),
		"Content-Encoding: %s\r\n", encodings );
	    add_response( hc, buf );
	    }
	if ( hc->got_range )
	    {
	    (void) my_snprintf( buf, sizeof(buf),
		"Content-Range: bytes %ld-%ld/%d\r\nContent-Length: %ld\r\n",
		(long) hc->init_byte_loc, (long) hc->end_byte_loc, length,
		(long) ( hc->end_byte_loc - hc->init_byte_loc + 1 ) );
	    add_response( hc, buf );
	    }
	else if ( length >= 0 )
	    {
	    (void) my_snprintf( buf, sizeof(buf),
		"Content-Length: %d\r\n", length );
	    add_response( hc, buf );
	    }
	else
	    {
	    hc->do_keep_alive = 0;
	    }
	if (hc->do_keep_alive)
	    {
	    add_response( hc, "Connection: keep-alive\r\n" );
	    }
	else
	    {
	    add_response( hc, "Connection: close\r\n" );
	    }
	if ( extraheads[0] != '\0' )
	    add_response( hc, extraheads );
#if defined(NAS) || defined(DVR)
	if (status == 450) {
		//DebugOutput("%s\n", hc->remoteuser);
		if (strlen(hc->remoteuser) > 0) {
			//(void) my_snprintf(buf, sizeof(buf), "Set-Cookie: %s=TEST; path=/; expires=Thursday, 31-Dec-2099 23:59:59 GMT\r\n", hc->remoteuser);
			(void) my_snprintf(buf, sizeof(buf), "Set-Cookie: %s=TEST; path=/\r\n", hc->remoteuser);
			add_response(hc, buf);
			//add_response(hc, "Set-Cookie: +OK=FALSE; path=/; expires=Thursday, 31-Dec-2099 23:59:59 GMT\r\n");
			//add_response(hc, "Set-Cookie: +OK=FALSE; path=/\r\n");
		}
		else {
			if (strlen(hc->session) > 0) {
				(void) my_snprintf(buf, sizeof(buf), "Set-Cookie: +ID=%s; path=/\r\n", hc->session);
				add_response(hc, buf);
			}
			//add_response(hc, "Set-Cookie: +OK=TRUE; path=/; expires=Thursday, 31-Dec-2099 23:59:59 GMT\r\n");
			//add_response(hc, "Set-Cookie: +OK=TRUE; path=/\r\n");
		}
	}
#if 0
	else {
		if (strlen(hc->session) > 0) {
			(void) my_snprintf(buf, sizeof(buf), "Set-Cookie: +ID=%s; path=/; expires=Thursday, 31-Jan-1980 23:59:59 GMT\r\n", hc->session);
			add_response(hc, buf);
		}
		//add_response(hc, "Set-Cookie: +OK=FALSE; path=/; expires=Thursday, 31-Dec-2099 23:59:59 GMT\r\n");
		//add_response(hc, "Set-Cookie: +OK=FALSE; path=/\r\n");
	}
#endif
#endif
	add_response( hc, "\r\n" );
	}
	//DebugOutput("Response: %s\n", hc->response);
    }


static int str_alloc_count = 0;
static long str_alloc_size = 0;

void
httpd_realloc_str( char** strP, int* maxsizeP, int size )
    {
    if ( *maxsizeP == 0 )
	{
	*maxsizeP = MAX( 200, size );   /* arbitrary */
	*strP = NEW( char, *maxsizeP + 4 );
	++str_alloc_count;
	str_alloc_size += *maxsizeP;
	}
    else if ( size > *maxsizeP )
	{
	str_alloc_size -= *maxsizeP;
	*maxsizeP = MAX( *maxsizeP * 2, size * 5 / 4 );
	*strP = RENEW( *strP, char, *maxsizeP + 4 );
	str_alloc_size += *maxsizeP;
	}
    else
	return;
    if ( *strP == (char*) 0 )
	{
	syslog(
	    LOG_ERR, "out of memory reallocating a string to %d bytes",
	    *maxsizeP );
	exit( 102 );
	}
    }

#if defined(NAS) || defined(DVR)
static void lock_pwd(char *username, char *ip)
{
	int index, mode;

	get_user_ip_login(username, ip, &index, &mode);
	if (index >= 0 && g_pwdcheck[index].idle_timer != (Timer *) 0 && !strncmp(g_pwdcheck[index].name, username, sizeof(g_pwdcheck[index].name)) && !strncmp(g_pwdcheck[index].ip, ip, sizeof(g_pwdcheck[index].ip))) {
		set_user_ip_login(username, ip, -2, 0);
		tmr_cancel(g_pwdcheck[index].idle_timer);
		g_pwdcheck[index].idle_timer = (Timer *) 0;
		g_total--;
		shm_del_user(username, ip, shm_get_app_id("thttpd"));
	}
	return;
}

static int get_lock_pwd(httpd_conn *hc, char *username, char *ip, int *index)
{
	int ret = 1, mode;

	get_user_ip_login(username, ip, index, &mode);
	//DebugOutput("index: %d, mode: %d, timer: 0x%x, timer0: 0x%x\n", *index, mode, g_pwdcheck[*index].idle_timer, g_pwdcheck[0].idle_timer);
	if (*index >= 0) {
		if (g_pwdcheck[*index].idle_timer != (Timer *) 0)
			ret = 0;
		else {
			*index = -1;
			Conf_Remove_Field(CHECK_USER_CONF, username, ip);
		}
	}
	else if (*index == -1) {
		if (g_total < MAX_CONNECT)
			ret = 0;
	}
#if 0
	else if (*index == -2) {
#if 0
		remove_user_ip(username, ip);
#else
		set_user_ip_login(username, ip, -3, 0);
#endif
	}
#endif
	else {
		if (g_total < MAX_CONNECT) {
			if (strlen(hc->session) > 0) {
				//remove_all_ip(ip, username, "-3", "-2");
				//Conf_Remove_Field(SESSION_ID_LOG_FILE, NULL, hc->session);
				ret = 0;
			}
			else {
				char text[128];

				sprintf(text, "%s=TEST", username);
				if (strstr(hc->cookie, text)) {
					//remove_all_ip(ip, username, "-3", "-2");
					ret = 0;
				}
			}
		}
	}
	return ret;
}

static int unlock_pwd(char *username, char *ip, int *index, int mode)
{
	int i, ret = 503, real_index = *index;
	struct timeval tv;
	ClientData client_data;

	(void) gettimeofday(&tv, (struct timezone*) 0);
	if (real_index < 0) {
		for (i = 0; i < MAX_CONNECT; i++) {
			if (g_pwdcheck[i].idle_timer != (Timer *) 0 && !strncmp(g_pwdcheck[i].name, username, sizeof(g_pwdcheck[i].name)) && !strncmp(g_pwdcheck[i].ip, ip, sizeof(g_pwdcheck[i].ip))) {
				real_index = i;
				break;
			}
		}
	}
	else {
		if (g_pwdcheck[real_index].idle_timer == (Timer *) 0 || strncmp(g_pwdcheck[real_index].name, username, sizeof(g_pwdcheck[real_index].name)) || strncmp(g_pwdcheck[real_index].ip, ip, sizeof(g_pwdcheck[real_index].ip)))
			real_index = -1;
	}
	if (real_index >= 0) {
		tmr_reset(&tv, g_pwdcheck[real_index].idle_timer);
		set_user_ip_login(username, ip, real_index, mode);
		ret = 0;
	}
	else {
		for (i = 0; i < MAX_CONNECT; i++) {
			if (g_pwdcheck[i].idle_timer == (Timer *) 0)
				break;
		}
		if (i < MAX_CONNECT) {
			client_data.i = i;
			if ((g_pwdcheck[i].idle_timer = tmr_create((struct timeval*) 0, idle_check, client_data, MAX_TIME * 1000L, 0)) != (Timer*) 0) {
				//DebugOutput("name: %s, ip: %s, index: %d, mode: %d, timer: 0x%x\n", username, ip, i, mode, g_pwdcheck[i].idle_timer);
				*index = i;
		    		strncpy(g_pwdcheck[i].name, username, sizeof(g_pwdcheck[i].name));
				strncpy(g_pwdcheck[i].ip, ip, sizeof(g_pwdcheck[i].ip));
		    		set_user_ip_login(username, ip, i, mode);
		    		g_total++;
		    		ret = 0;
		    	}
		}
	}
	//DebugOutput("unlock_pwd timer: 0x%x\n", g_pwdcheck[0].idle_timer);
	return ret;
}
#endif

static void
send_response( httpd_conn* hc, int status, char* title, char* extraheads, char* form, char* arg )
    {
    char defanged_arg[1000], buf[2000];
#if defined(NAS) || defined(DVR)
	hc->response[0] = 0x0;
#endif
    send_mime( hc, status, title, "", extraheads, "text/html", -1, 0 );
    (void) my_snprintf( buf, sizeof(buf),
	"<HTML><HEAD><TITLE>%d %s</TITLE></HEAD>\n<BODY BGCOLOR=\"#cc9999\"><H2>%d %s</H2>\n",
	status, title, status, title );
    add_response( hc, buf );
    defang( arg, defanged_arg, sizeof(defanged_arg) );
    (void) my_snprintf( buf, sizeof(buf), form, defanged_arg );
    add_response( hc, buf );
#if 0
    if (status == 450) {
#if 0
	remove_user_ip(remoteuser, remoteuserip);
#else
	if (strlen(hc->remoteuser) > 0)
		set_user_ip_login(hc->remoteuser, httpd_ntoa(&hc->client_addr), -3, 0);
	else
		remove_all_ip(httpd_ntoa(&hc->client_addr), NULL, "-2", "-3");
#endif
    }
#endif
    if ( match( "**MSIE**", hc->useragent ) )
	{
	int n;
	add_response( hc, "<!--\n" );
	for ( n = 0; n < 6; ++n )
	    add_response( hc, "Padding so that MSIE deigns to show this error instead of its own canned one.\n");
	add_response( hc, "-->\n" );
	}
    send_response_tail( hc );
    }


static void
send_response_tail( httpd_conn* hc )
    {
#if !defined(NAS) && !defined(DVR)
//#if 1
    char buf[1000];

    (void) my_snprintf( buf, sizeof(buf),
	"<HR>\n<ADDRESS><A HREF=\"%s\">%s</A></ADDRESS>\n</BODY></HTML>\n",
	SERVER_ADDRESS, EXPOSED_SERVER_SOFTWARE );
    add_response( hc, buf );
#endif
    }


static void
defang( char* str, char* dfstr, int dfsize )
    {
    char* cp1;
    char* cp2;

    for ( cp1 = str, cp2 = dfstr;
	  *cp1 != '\0' && cp2 - dfstr < dfsize - 5;
	  ++cp1, ++cp2 )
	{
	switch ( *cp1 )
	    {
	    case '<':
	    *cp2++ = '&';
	    *cp2++ = 'l';
	    *cp2++ = 't';
	    *cp2 = ';';
	    break;
	    case '>':
	    *cp2++ = '&';
	    *cp2++ = 'g';
	    *cp2++ = 't';
	    *cp2 = ';';
	    break;
	    default:
	    *cp2 = *cp1;
	    break;
	    }
	}
    *cp2 = '\0';
    }


void
httpd_send_err( httpd_conn* hc, int status, char* title, char* extraheads, char* form, char* arg )
    {
#ifdef ERR_DIR
    char filename[1000];
#if defined(NAS) || defined(DVR)
        char errhtml[100];

	strcpy(errhtml, "error_thttpd.html");
#endif
    /* Try virtual host error page. */
    if ( hc->hs->vhost && hc->hostdir[0] != '\0' )
	{
#if defined(NAS) || defined(DVR)
	(void) my_snprintf(filename, sizeof(filename), "%s/%s/%s", hc->hostdir, ERR_DIR, errhtml);
#else
	(void) my_snprintf( filename, sizeof(filename),
	    "%s/%s/err%d.html", hc->hostdir, ERR_DIR, status );
#endif
	//DebugOutput("error page: %s\n", filename);
	if ( send_err_file( hc, status, title, extraheads, filename, arg ) )
	    return;
	}

    /* Try server-wide error page. */
#if defined(NAS) || defined(DVR)
//#ifdef NAS
	(void) my_snprintf(filename, sizeof(filename), "%s/%s", ERR_DIR, errhtml);
#else
    (void) my_snprintf( filename, sizeof(filename),
	"%s/err%d.html", ERR_DIR, status );
#endif
    if ( send_err_file( hc, status, title, extraheads, filename, arg ) )
	return;

    /* Fall back on built-in error page. */
    send_response( hc, status, title, extraheads, form, arg );

#else /* ERR_DIR */

    send_response( hc, status, title, extraheads, form, arg );

#endif /* ERR_DIR */
    }


#ifdef ERR_DIR
static int
send_err_file( httpd_conn* hc, int status, char* title, char* extraheads, char* filename, char* arg )
    {
    FILE* fp;
    char buf[1024+sizeof(long)];
#if defined(NAS) || defined(DVR)
//#ifdef NAS
	char *ptr = httpd_errunknownform, model[32], charset[256], temp[256];

	char color_2108r_1[] = {"<td width=\"97%\" bgcolor=\"#764C90\">"};
	char color_2108r_2[] = {"<td bgcolor=\"#764C90\" height=\"77\" valign=\"bottom\"><img src=\"/v2_menu/images/b4a.gif\"></td>"};
        char color_104r_1[] = {"<td width=\"97%\" bgcolor=\"#0095C7\">"};
        char color_104r_2[] = {"<td bgcolor=\"#0095C7\" height=\"77\" valign=\"bottom\"><img src=\"/v2_menu/images/b4a.gif\"></td>"};
        char color_4000_1[] = {"<td width=\"97%\" bgcolor=\"#6666FF\">"};
        char color_4000_2[] = {"<td bgcolor=\"#6666FF\" height=\"77\" valign=\"bottom\"><img src=\"/v2_menu/images/b4a.gif\"></td>"};
#endif

    fp = fopen( filename, "r" );
    if ( fp == (FILE*) 0 )
	return 0;
    send_mime( hc, status, title, "", extraheads, "text/html", -1, 0 );
    for (;;)
	{
#if defined(NAS) || defined(DVR)
//#ifdef NAS
	buf[0] = 0x0;
	fgets(buf, 1000, fp);
	if (buf[0] == 0x0)
		break;
	if (strstr(buf, "$server_name$")) {
    		Get_Server_Name(charset, sizeof(charset));
    		my_snprintf(buf, sizeof(buf), charset, arg);
	}
	else if (strstr(buf, "$server_version$")) {
		Get_System_Version(temp, sizeof(temp));
		strcpy(charset, temp);
		strcat(charset, " Build ");
		Get_Build_Number(temp, sizeof(temp));
		strcat(charset, temp);
		my_snprintf(buf, sizeof(buf), charset, arg);
	}
	else if (strstr(buf, "$content_link1$")) {
		if (status == 401 || status == 450)
    			my_snprintf(buf, sizeof(buf), "SYSTEM_MESSAGE_LOGOUT2", arg);
    		else
    			my_snprintf(buf, sizeof(buf), "EVT_USR_BACK", arg);
    	}
    	else if (strstr(buf, "$cookie_name$")) {
		if (status == 450 && strlen(hc->remoteuser) > 0)
    			sprintf(buf, "\"%s\"", hc->remoteuser);
    		else
    			strcpy(buf, "\"\"");
    	}
    	else if (strstr(buf, "$error_link1$")) {
    		if (status == 401 || status == 450)
    			my_snprintf(buf, sizeof(buf), "window.location = \"/enter.html\";\n", arg);
    		else
    			my_snprintf(buf, sizeof(buf), "history.back();\n", arg);
    	}
    	else if (strstr(buf, "$title_text1$"))
    		my_snprintf(buf, sizeof(buf), "SYSTEM_MESSAGE_TITLE", arg);
	else if (strstr(buf, "$content_pic1$"))
    		my_snprintf(buf, sizeof(buf), "src=\"/cgi-bin/data/images/error.gif\"", arg);
    	else if (strstr(buf, "$title_pic1$"))
    		my_snprintf(buf, sizeof(buf), "src=\"/cgi-bin/data/images/icon-logout.gif\"", arg);
        else if (strstr(buf, "HEADER_BG1")) {
        	Get_Model_Name(model, sizeof(model));
                if (!strcmp(model, "NAS-2108R"))
                        ptr = color_2108r_1;
                else
                if (!strcmp(model, "NAS-104R"))
                        ptr = color_104r_1;
                else
                if (!strcmp(model, "NAS-4000"))
                        ptr = color_4000_1;
                else
                        ptr = color_4000_1;
		my_snprintf(buf, sizeof(buf), ptr, arg);
        }
        else if (strstr(buf, "HEADER_BG2")) {
        	Get_Model_Name(model, sizeof(model));
                if (!strcmp(model, "NAS-2108R"))
                        ptr = color_2108r_2;
                else
                if (!strcmp(model, "NAS-104R"))
                        ptr = color_104r_2;
                else
                if (!strcmp(model, "NAS-4000"))
                        ptr = color_4000_2;
                else
                        ptr = color_4000_2;
                my_snprintf(buf, sizeof(buf), ptr, arg);
        }
	else if (strstr(buf, "#charset#") || strstr(buf, "$charset$")) {
		ptr = hc->cookie;
		if (strstr(ptr, "+nas_lang=TCH"))
			strcpy(charset, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=big5\">\n");
		else
		if (strstr(ptr, "+nas_lang=ENG"))
			strcpy(charset, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n");
		else
		if (strstr(ptr, "+nas_lang=JPN"))
			strcpy(charset, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=Shift_JIS\">\n");
		else
		if (strstr(ptr, "+nas_lang=CHS"))
			strcpy(charset, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=gb2312\">\n");
// Ben Add 2005.02.01 for Polish
		else
		if (strstr(ptr, "+nas_lang=POL"))
			strcpy(charset, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=windows-1250\">\n");
// Ben Add 2005.02.01 for Polish
#if 0
		else
		if (strstr(ptr, "nas_lang=TCH"))
			strcpy(charset, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=big5\">\n");
		else
		if (strstr(ptr, "nas_lang=ENG"))
			strcpy(charset, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n");
		else
		if (strstr(ptr, "nas_lang=JPN"))
			strcpy(charset, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=Shift_JIS\">\n");
		else
		if (strstr(ptr, "nas_lang=CHS"))
			strcpy(charset, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=gb2312\">\n");
// Ben Add 2005.02.01 for Polish
		else
		if (strstr(ptr, "nas_lang=POL"))
			strcpy(charset, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=windows-1250\">\n");
// Ben Add 2005.02.01 for Polish
#endif
		my_snprintf(buf, sizeof(buf), charset, arg);
	}
        else if (strstr(buf, "#error#") || strstr(buf, "$content_text1$")) {
		switch (status) {
			case 302:
				ptr = err302form;
				break;

			case 400:
				ptr = httpd_err400form;
				break;

			case 450:
#if 0
#if 0
				remove_user_ip(remoteuser, remoteuserip);
#else
				if (strlen(hc->remoteuser) > 0)
					set_user_ip_login(hc->remoteuser, httpd_ntoa(&hc->client_addr), -3, 0);
				else
					remove_all_ip(httpd_ntoa(&hc->client_addr), NULL, "-2", "-3");
#endif
#endif
			case 401:
#ifdef NAS
				ptr = err401form;
#else
				ptr = "SYSTEM_MESSAGE_ERROR401";
#endif
				break;

			case 403:
#ifdef NAS
				ptr = err403form;
#else
				ptr = "SYSTEM_MESSAGE_ERROR403";
#endif
				break;

			case 404:
#ifdef DVR
				ptr = "SYSTEM_MESSAGE_ERROR404";
#else
				ptr = err404form;
#endif
				break;

			case 408:
				ptr = httpd_err408form;
				break;

			case 500:
				ptr = err500form;
				break;

			case 501:
				ptr = err501form;
				break;

			case 503:
#ifdef DVR
				ptr = "SYSTEM_MESSAGE_ERROR503";
#else
				ptr = httpd_err503form;
#endif
				break;

			case STATUS_LOCAL_DISK_MODULE_ERROR:
#ifdef DVR
				//ptr = httpd_err997form;
				ptr = "SYSTEM_MESSAGE_ERROR997";
				DebugOutput("STATUS_LOCAL_DISK_MODULE_ERROR %s ...\n", ptr);
#endif
				break;

			case STATUS_LOCAL_DISK_BUSY_REJECT:
#ifdef DVR
				//ptr = httpd_err998form;
				ptr = "SYSTEM_MESSAGE_ERROR998";
				DebugOutput("STATUS_LOCAL_DISK_BUSY_REJECT %s ...\n", ptr);
#endif
				break;

			case 999:
#ifdef NAS
				ptr = httpd_err999form;
#else
				ptr = "SYSTEM_MESSAGE_ERROR999";
#endif
				break;
		}
		my_snprintf(buf, sizeof(buf), ptr, arg);
	}
#else
	int r;

	r = fread( buf, 1, sizeof(buf) - sizeof(long), fp );
	if ( r <= 0 )
	    break;
	buf[r] = '\0';
#endif
	add_response( hc, buf );
	}
    (void) fclose( fp );

#ifdef ERR_APPEND_SERVER_INFO
    send_response_tail( hc );
#endif /* ERR_APPEND_SERVER_INFO */

    return 1;
    }
#endif /* ERR_DIR */

#ifdef AUTH_FILE

static void
send_authenticate( httpd_conn* hc, char* realm, int status )
    {
    static char* header;
    static int maxheader = 0;
    static char headstr[] = "WWW-Authenticate: Basic realm=\"";

    httpd_realloc_str(
	&header, &maxheader, sizeof(headstr) + strlen( realm ) + 3 );
    (void) my_snprintf( header, maxheader, "%s%s\"\r\n", headstr, realm );
#if defined(NAS) || defined(DVR)
	httpd_send_err( hc, status, err401title, header, err401form, "" );
#else
    httpd_send_err( hc, status, err401title, header, err401form, hc->encodedurl );
#endif
    /* If the request was a POST then there might still be data to be read,
    ** so we need to do a lingering close.
    */
    if ( hc->method == METHOD_POST )
	hc->should_linger = 1;
    }

/* Base-64 decoding.  This represents binary data as printable ASCII
** characters.  Three 8-bit binary bytes are turned into four 6-bit
** values, like so:
**
**   [11111111]  [22222222]  [33333333]
**
**   [111111] [112222] [222233] [333333]
**
** Then the 6-bit values are represented using the characters "A-Za-z0-9+/".
*/

static int b64_decode_table[256] = {
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 00-0F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 10-1F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,62,-1,-1,-1,63,  /* 20-2F */
    52,53,54,55,56,57,58,59,60,61,-1,-1,-1,-1,-1,-1,  /* 30-3F */
    -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,  /* 40-4F */
    15,16,17,18,19,20,21,22,23,24,25,-1,-1,-1,-1,-1,  /* 50-5F */
    -1,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,  /* 60-6F */
    41,42,43,44,45,46,47,48,49,50,51,-1,-1,-1,-1,-1,  /* 70-7F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 80-8F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 90-9F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* A0-AF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* B0-BF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* C0-CF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* D0-DF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* E0-EF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1   /* F0-FF */
    };

/* Do base-64 decoding on a string.  Ignore any non-base64 bytes.
** Return the actual number of bytes generated.  The decoded size will
** be at most 3/4 the size of the encoded, and may be smaller if there
** are padding characters (blanks, newlines).
*/
static int
b64_decode( const char* str, unsigned char* space, int size )
    {
    const char* cp;
    int space_idx, phase;
    int d, prev_d = 0;
    unsigned char c;

    space_idx = 0;
    phase = 0;
    for ( cp = str; *cp != '\0'; ++cp )
	{
	d = b64_decode_table[(int) *cp];
	if ( d != -1 )
	    {
	    switch ( phase )
		{
		case 0:
		++phase;
		break;
		case 1:
		c = ( ( prev_d << 2 ) | ( ( d & 0x30 ) >> 4 ) );
		if ( space_idx < size )
		    space[space_idx++] = c;
		++phase;
		break;
		case 2:
		c = ( ( ( prev_d & 0xf ) << 4 ) | ( ( d & 0x3c ) >> 2 ) );
		if ( space_idx < size )
		    space[space_idx++] = c;
		++phase;
		break;
		case 3:
		c = ( ( ( prev_d & 0x03 ) << 6 ) | d );
		if ( space_idx < size )
		    space[space_idx++] = c;
		phase = 0;
		break;
		}
	    prev_d = d;
	    }
	}
    return space_idx;
    }


/* Returns -1 == unauthorized, 0 == no auth file, 1 = authorized. */
static int
auth_check( httpd_conn* hc, char* dirname  )
    {
    if ( hc->hs->global_passwd )
	{
	char* topdir;
	if ( hc->hs->vhost && hc->hostdir[0] != '\0' )
	    topdir = hc->hostdir;
	else
	    topdir = ".";
	switch ( auth_check2( hc, topdir ) )
	    {
	    case -1:
	    return -1;
	    case -401:
	    return -401;
	    case 1:
	    return 1;
	    }
	}
    return auth_check2( hc, dirname );
    }

#ifdef DVR
static char ocx_auth_check(httpd_conn* hc, char *usr_name )
{
	char authinfo[128];
    	char *authpass;
    	int l, i;
    	char ret = 0;
    	char line[128];
	usr_name[0]='\0';

	DebugOutput("ocx_auth_check1: %s\n", hc->authorization);
    	if (hc->authorization[0] == '\0' || strncmp( hc->authorization, "Basic ", 6 ) != 0)
	{
//DebugOutput2("ocx_auth_check1: auth=[%s]\n", hc->authorization);
		return ret;
	}
	DebugOutput("ocx_auth_check2\n");
	/* Decode it. */
    	l = b64_decode(&(hc->authorization[6]), (unsigned char*) authinfo, sizeof(authinfo) - 1);
    	authinfo[l] = '\0';
    	/* Split into user and password. */
    	authpass = strchr(authinfo, ':');
    	if (authpass == (char*) 0)
		return ret;
	strcpy( usr_name, authinfo );
	DebugOutput("ocx_auth_check3\n");
    	*authpass++ = '\0';
	Get_Exact_NAS_User_Name(authinfo);
	if (strcasecmp(authinfo, "guest") == 0 || strcasecmp(authinfo, "dvr_ocx_ftp") == 0)
		return ret;
	DebugOutput("ocx_auth_check4\n");
	if (Check_Local_User_Password(authinfo, authpass) == SUCCESS) {
		DebugOutput("ocx_auth_check: %s\n", authinfo);
		for (i = 0; i < 4; i++) {
			sprintf(line, "monitor%d", i);
			if (User_Belongs_To_Group(authinfo, line))
				ret |= (1 << i);
		}
		DebugOutput("ocx_auth_check: %d\n", ret);
	}
	return ret;
}
#endif

/* Returns -1 == unauthorized, 0 == no auth file, 1 = authorized. */
static int
auth_check2( httpd_conn* hc, char* dirname  )
    {
    char authinfo[128];
    char* authpass;
    int l;
    char line[128];
#if !defined(NAS) && !defined(DVR)
    static char* authpath;
    static int maxauthpath = 0;
    FILE* fp;
    char* cryp;
    static char* prevauthpath;
    static int maxprevauthpath = 0;
    static time_t prevmtime;
    static char* prevuser;
    static int maxprevuser = 0;
    static char* prevcryp;
    static int maxprevcryp = 0;
    struct stat sb;
#endif
    //DebugOutput("auth_check2: %s\n", hc->encodedurl);
#if defined(NAS) || defined(DVR)
#ifdef NAS
	if (strstr(hc->encodedurl, "cgi-bin/filemanager/filemanager.cgi?")) {
		if (Is_Webfs_Enabled() == FALSE)
			return -999;
	}
	if (strstr(hc->encodedurl, "cgi-bin/filemanager/share") == 0 && strstr(hc->encodedurl, "cgi-bin/filemanager/filemanager.cgi?") == 0 && strstr(hc->encodedurl, "cgi-bin/filemanager/function.cgi") == 0)
		return 1;
#endif
	if (/*strstr(hc->encodedurl, "cgi-bin") == 0 || */strstr(hc->encodedurl, ".cgi") == 0)
		return 1;
	if (((authpass = strstr(hc->cookie, "+ID=")) != NULL) && (strlen(authpass) > 4)) {
		//DebugOutput("1\n");
		for (l = 0; l < (strlen(authpass) - 4); l++) {
			if (authpass[4 + l] == ' ')
				break;
			else
				authinfo[l] = authpass[4 + l];
		}
		authinfo[l] = '\0';
		//if (Conf_Get_Field(SESSION_ID_LOG_FILE, NULL, authinfo, NULL, 0) != ERROR_NOT_FOUND) {
			//DebugOutput("2\n");
			httpd_realloc_str(&hc->session, &hc->maxsession, strlen(authinfo));
			strcpy(hc->session, authinfo);
		//}
	}
#else
    /* Construct auth filename. */
    httpd_realloc_str(
	&authpath, &maxauthpath, strlen( dirname ) + 1 + sizeof(AUTH_FILE) );
    (void) my_snprintf( authpath, maxauthpath,
	"%s/%s", dirname, AUTH_FILE );

    /* Does this directory have an auth file? */
    if ( stat( authpath, &sb ) < 0 )
	/* Nope, let the request go through. */
	return 0;
#endif
    /* Does this request contain basic authorization info? */
    if ( hc->authorization[0] == '\0' ||
	 strncmp( hc->authorization, "Basic ", 6 ) != 0 )
	{
#if defined(NAS) || defined(DVR)
		if (strlen(hc->session) == 0) {
			sprintf(line, "%d", (int) time(NULL));
			//Conf_Set_Field(SESSION_ID_LOG_FILE, NULL, line, "");
			httpd_realloc_str(&hc->session, &hc->maxsession, strlen(line));
			strcpy(hc->session, line);
			//DebugOutput("3\n");
		}
#endif
	/* Nope, return a 401 Unauthorized. */
	send_authenticate( hc, dirname, 450 );
	//DebugOutput("send 401 error\n");
#if defined(NAS) || defined(DVR)
	return -401;
#else
	return -1;
#endif
	}
	//DebugOutput("4\n");
    /* Decode it. */
    l = b64_decode( &(hc->authorization[6]), authinfo, sizeof(authinfo) - 1 );
    authinfo[l] = '\0';
    /* Split into user and password. */
    authpass = strchr( authinfo, ':' );
    if ( authpass == (char*) 0 )
	{
		//DebugOutput("2\n");
	/* No colon?  Bogus auth info. */
	send_authenticate( hc, dirname, 401 );
#if defined(NAS) || defined(DVR)
	return -401;
#else
	return -1;
#endif
	}
    *authpass++ = '\0';
#if defined(NAS) || defined(DVR)
	//DebugOutput("5\n");
	Get_Exact_NAS_User_Name(authinfo);
	if (strcasecmp(authinfo, "guest") == 0 || strcasecmp(authinfo, "dvr_ocx_ftp") == 0) {
		send_authenticate(hc, dirname, 401);
		//return -401;
		return -1;
	}
#ifdef DVR
	if (Check_Local_User_Password(authinfo, authpass) == SUCCESS) {
#else
	if (Check_NAS_User_Password(authinfo, authpass) == SUCCESS) {
#endif
		httpd_realloc_str(&hc->remoteuser, &hc->maxremoteuser, strlen(authinfo));
		strcpy(hc->remoteuser, authinfo);
		if (strstr(hc->encodedurl, "logout=yes")) {
	        	//DebugOutput("2\n");
	                lock_pwd(authinfo, httpd_ntoa(&hc->client_addr));
			//strcpy(hc->authorization, "B");
	                return 1;
	        }
	    	if (get_lock_pwd(hc, authinfo, httpd_ntoa(&hc->client_addr), &hc->timer_index) == 1) {
	    		//DebugOutput("1\n");
	    		strcpy(hc->authorization, "B");
	    		send_authenticate(hc, dirname, 450);
			//return -401;
			return -1;
		}
		//strcpy(remoteuser, authinfo);
		//strcpy(remoteuserip, httpd_ntoa(&hc->client_addr));
#ifdef DVR
		if (strstr(hc->encodedurl, "monitor")/* || strstr(hc->encodedurl, "ch_info_update")*/)
			hc->mode = 1;
		else if (strstr(hc->encodedurl, "play") || strstr(hc->encodedurl, "vdo_query") || strstr(hc->encodedurl, "input")) {
#if 0
			int show_play = User_Belongs_To_Group(hc->remoteuser, "playback0") || User_Belongs_To_Group(hc->remoteuser, "playback1") || User_Belongs_To_Group(hc->remoteuser, "playback2") || User_Belongs_To_Group(hc->remoteuser, "playback3");

			if (!show_play) {
				return -999;
			}
#endif
			hc->mode = 2;
		}
		else if (strstr(hc->encodedurl, "cfg_")) {
			if (!User_Belongs_To_Group(hc->remoteuser, "administrators")) {
				return -999;
			}
			hc->mode = 3;
		}
		else if (strstr(hc->encodedurl, "ptz")) {
			int show_ptz = (User_Belongs_To_Group(hc->remoteuser, "monitor0") || User_Belongs_To_Group(hc->remoteuser, "monitor1") || User_Belongs_To_Group(hc->remoteuser, "monitor2") || User_Belongs_To_Group(hc->remoteuser, "monitor3")) && User_Belongs_To_Group(hc->remoteuser, "ptz");

			if (!show_ptz) {
				return -999;
			}
			hc->mode = 1;
		}
		else if (strstr(hc->encodedurl, "monitor_ch")) {
			hc->mode = 1;
			hc->broadcast = 1;
		}
		//else if (strstr(hc->encodedurl, "sacc"))
		//	hc->mode = 4;
		if (strstr(hc->encodedurl, "sacc") == NULL && unlock_pwd(hc->remoteuser, httpd_ntoa(&hc->client_addr), &hc->timer_index, (strstr(hc->encodedurl, "ch_info_update") ? 4 : hc->mode)) != 0)
#else
		if (unlock_pwd(hc->remoteuser, httpd_ntoa(&hc->client_addr), &hc->timer_index, hc->mode) != 0)
#endif
			return -503;
	}
	else {
#ifdef DVR
		if (strstr(hc->encodedurl, "ch_info_update"))
			send_redirect(hc, "/cgi-bin/data/cmp/ch_info_fail.html");
		else
#endif
			send_authenticate(hc, dirname, 401);
		//return -401;
		return -1;
	}
	return 1;
#else
    /* See if we have a cached entry and can use it. */
    if ( maxprevauthpath != 0 &&
	 strcmp( authpath, prevauthpath ) == 0 &&
	 sb.st_mtime == prevmtime &&
	 strcmp( authinfo, prevuser ) == 0 )
	{
	/* Yes.  Check against the cached encrypted password. */
	if ( strcmp( crypt( authpass, prevcryp ), prevcryp ) == 0 )
	    {
	    /* Ok! */
	    httpd_realloc_str(
		&hc->remoteuser, &hc->maxremoteuser, strlen( authinfo ) );
	    (void) strcpy( hc->remoteuser, authinfo );
	    return 1;
	    }
	else
	    {
	    /* No. */
	    send_authenticate( hc, dirname, 401 );
	    return -1;
	    }
	}

    /* Open the password file. */
    fp = fopen( authpath, "r" );
    if ( fp == (FILE*) 0 )
	{
	/* The file exists but we can't open it?  Disallow access. */
	syslog(
	    LOG_ERR, "%.80s auth file %.80s could not be opened - %m",
	    httpd_ntoa( &hc->client_addr ), authpath );
	httpd_send_err(
	    hc, 403, err403title, "",
	    ERROR_FORM( err403form, "The requested URL '%.80s' is protected by an authentication file, but the authentication file cannot be opened.\n" ),
	    hc->encodedurl );
	return -1;
	}

    /* Read it. */
    while ( fgets( line, sizeof(line), fp ) != (char*) 0 )
	{
	/* Nuke newline. */
	l = strlen( line );
	if ( line[l - 1] == '\n' )
	    line[l - 1] = '\0';
	/* Split into user and encrypted password. */
	cryp = strchr( line, ':' );
	if ( cryp == (char*) 0 )
	    continue;
	*cryp++ = '\0';
	/* Is this the right user? */
	if ( strcmp( line, authinfo ) == 0 )
	    {
	    /* Yes. */
	    (void) fclose( fp );
	    /* So is the password right? */
	    if ( strcmp( crypt( authpass, cryp ), cryp ) == 0 )
		{
		/* Ok! */
		httpd_realloc_str(
		    &hc->remoteuser, &hc->maxremoteuser, strlen( line ) );
		(void) strcpy( hc->remoteuser, line );
		/* And cache this user's info for next time. */
		httpd_realloc_str(
		    &prevauthpath, &maxprevauthpath, strlen( authpath ) );
		(void) strcpy( prevauthpath, authpath );
		prevmtime = sb.st_mtime;
		httpd_realloc_str(
		    &prevuser, &maxprevuser, strlen( authinfo ) );
		(void) strcpy( prevuser, authinfo );
		httpd_realloc_str( &prevcryp, &maxprevcryp, strlen( cryp ) );
		(void) strcpy( prevcryp, cryp );
		return 1;
		}
	    else
		{
		/* No. */
		send_authenticate( hc, dirname, 401 );
		return -1;
		}
	    }
	}

    /* Didn't find that user.  Access denied. */
    (void) fclose( fp );
    send_authenticate( hc, dirname, 401 );
    return -1;
#endif
    }

#endif /* AUTH_FILE */


void
send_redirect( httpd_conn* hc, char *encodedurl )
    {
    static char* header;
    static size_t maxheader = 0;
    static char headstr[] = "Location: ";
    size_t url_len;

    if ( encodedurl == ((char *) 0) || *encodedurl == '\0' )
	encodedurl = hc->encodedurl;

    url_len = strlen( encodedurl );
    httpd_realloc_str(
	&header, &maxheader, sizeof(headstr) + url_len + 2 );
    (void) my_snprintf( header, maxheader,
	"%s%s\r\n", headstr, encodedurl );
    send_response( hc, 302, err302title, header, err302form, encodedurl );
    }


static void
send_dirredirect( httpd_conn* hc )
    {
    static char* location;
    static size_t maxlocation = 0;
    size_t url_len;

    url_len = strlen( hc->encodedurl );
    httpd_realloc_str(
	    &location, &maxlocation, url_len + 1 );
    strcpy( location, hc->encodedurl );
    location[url_len++] = '/';
    location[url_len]   = '\0';
    send_redirect( hc, location );
    }


char*
httpd_method_str( int method )
    {
    switch ( method )
	{
	case METHOD_GET: return "GET";
	case METHOD_HEAD: return "HEAD";
	case METHOD_POST: return "POST";
	default: return "UNKNOWN";
	}
    }


static int
hexit( char c )
    {
    if ( c >= '0' && c <= '9' )
	return c - '0';
    if ( c >= 'a' && c <= 'f' )
	return c - 'a' + 10;
    if ( c >= 'A' && c <= 'F' )
	return c - 'A' + 10;
    return 0;           /* shouldn't happen, we're guarded by isxdigit() */
    }


/* Copies and decodes a string.  It's ok for from and to to be the
** same string.
*/
static void
strdecode( char* to, char* from )
    {
    for ( ; *from != '\0'; ++to, ++from )
	{
	if ( from[0] == '%' && isxdigit( from[1] ) && isxdigit( from[2] ) )
	    {
	    *to = hexit( from[1] ) * 16 + hexit( from[2] );
	    from += 2;
	    }
	else
	    *to = *from;
	}
    *to = '\0';
    }


#ifdef GENERATE_INDEXES
/* Copies and encodes a string. */
static void
strencode( char* to, int tosize, char* from )
    {
    int tolen;

    for ( tolen = 0; *from != '\0' && tolen + 4 < tosize; ++from )
	{
	if ( isalnum(*from) || strchr( "/_.-~", *from ) != (char*) 0 )
	    {
	    *to = *from;
	    ++to;
	    ++tolen;
	    }
	else
	    {
	    (void) sprintf( to, "%%%02x", (int) *from & 0xff );
	    to += 3;
	    tolen += 3;
	    }
	}
    *to = '\0';
    }
#endif /* GENERATE_INDEXES */


#ifdef TILDE_MAP_1
/* Map a ~username/whatever URL into <prefix>/username. */
static int
tilde_map_1( httpd_conn* hc )
    {
    static char* temp;
    static int maxtemp = 0;
    int len;
    static char* prefix = TILDE_MAP_1;

    len = strlen( hc->expnfilename ) - 1;
    httpd_realloc_str( &temp, &maxtemp, len );
    (void) strcpy( temp, &hc->expnfilename[1] );
    httpd_realloc_str(
	&hc->expnfilename, &hc->maxexpnfilename, strlen( prefix ) + 1 + len );
    (void) strcpy( hc->expnfilename, prefix );
    if ( prefix[0] != '\0' )
	(void) strcat( hc->expnfilename, "/" );
    (void) strcat( hc->expnfilename, temp );
    return 1;
    }
#endif /* TILDE_MAP_1 */

#ifdef TILDE_MAP_2
/* Map a ~username/whatever URL into <user's homedir>/<postfix>. */
static int
tilde_map_2( httpd_conn* hc )
    {
    static char* temp;
    static int maxtemp = 0;
    static char* postfix = TILDE_MAP_2;
    char* cp;
    struct passwd* pw;
    char* alt;
    char* rest;

    /* Get the username. */
    httpd_realloc_str( &temp, &maxtemp, strlen( hc->expnfilename ) - 1 );
    (void) strcpy( temp, &hc->expnfilename[1] );
    cp = strchr( temp, '/' );
    if ( cp != (char*) 0 )
	*cp++ = '\0';
    else
	cp = "";

    /* Get the passwd entry. */
    pw = getpwnam( temp );
    if ( pw == (struct passwd*) 0 )
	return 0;

    /* Set up altdir. */
    httpd_realloc_str(
	&hc->altdir, &hc->maxaltdir,
	strlen( pw->pw_dir ) + 1 + strlen( postfix ) );
    (void) strcpy( hc->altdir, pw->pw_dir );
    if ( postfix[0] != '\0' )
	{
	(void) strcat( hc->altdir, "/" );
	(void) strcat( hc->altdir, postfix );
	}
    alt = expand_symlinks( hc->altdir, &rest, 0, 1 );
    if ( rest[0] != '\0' )
	return 0;
    httpd_realloc_str( &hc->altdir, &hc->maxaltdir, strlen( alt ) );
    (void) strcpy( hc->altdir, alt );

    /* And the filename becomes altdir plus the post-~ part of the original. */
    httpd_realloc_str(
	&hc->expnfilename, &hc->maxexpnfilename,
	strlen( hc->altdir ) + 1 + strlen( cp ) );
    (void) my_snprintf( hc->expnfilename, hc->maxexpnfilename,
	"%s/%s", hc->altdir, cp );

    /* For this type of tilde mapping, we want to defeat vhost mapping. */
    hc->tildemapped = 1;

    return 1;
    }
#endif /* TILDE_MAP_2 */


/* Virtual host mapping. */
static int
vhost_map( httpd_conn* hc )
    {
    httpd_sockaddr sa;
    int sz;
    static char* tempfilename;
    static int maxtempfilename = 0;
    char* cp1;
    int len;
#ifdef VHOST_DIRLEVELS
    int i;
    char* cp2;
#endif /* VHOST_DIRLEVELS */

    /* Figure out the virtual hostname. */
    if ( hc->reqhost[0] != '\0' )
	hc->hostname = hc->reqhost;
    else if ( hc->hdrhost[0] != '\0' )
	hc->hostname = hc->hdrhost;
    else
	{
	sz = sizeof(sa);
	if ( getsockname( hc->conn_fd, &sa.sa, &sz ) < 0 )
	    {
	    syslog( LOG_ERR, "getsockname - %m" );
	    return 0;
	    }
	hc->hostname = httpd_ntoa( &sa );
	}
    /* Pound it to lower case. */
    for ( cp1 = hc->hostname; *cp1 != '\0'; ++cp1 )
	if ( isupper( *cp1 ) )
	    *cp1 = tolower( *cp1 );

    if ( hc->tildemapped )
	return 1;

    /* Figure out the host directory. */
#ifdef VHOST_DIRLEVELS
    httpd_realloc_str(
	&hc->hostdir, &hc->maxhostdir,
	strlen( hc->hostname ) + 2 * VHOST_DIRLEVELS );
    if ( strncmp( hc->hostname, "www.", 4 ) == 0 )
	cp1 = &hc->hostname[4];
    else
	cp1 = hc->hostname;
    for ( cp2 = hc->hostdir, i = 0; i < VHOST_DIRLEVELS; ++i )
	{
	/* Skip dots in the hostname.  If we don't, then we get vhost
	** directories in higher level of filestructure if dot gets
	** involved into path construction.  It's `while' used here instead
	** of `if' for it's possible to have a hostname formed with two
	** dots at the end of it.
	*/
	while ( *cp1 == '.' )
	    ++cp1;
	/* Copy a character from the hostname, or '_' if we ran out. */
	if ( *cp1 != '\0' )
	    *cp2++ = *cp1++;
	else
	    *cp2++ = '_';
	/* Copy a slash. */
	*cp2++ = '/';
	}
    (void) strcpy( cp2, hc->hostname );
#else /* VHOST_DIRLEVELS */
    httpd_realloc_str( &hc->hostdir, &hc->maxhostdir, strlen( hc->hostname ) );
    (void) strcpy( hc->hostdir, hc->hostname );
#endif /* VHOST_DIRLEVELS */

    /* Prepend hostdir to the filename. */
    len = strlen( hc->expnfilename );
    httpd_realloc_str( &tempfilename, &maxtempfilename, len );
    (void) strcpy( tempfilename, hc->expnfilename );
    httpd_realloc_str(
	&hc->expnfilename, &hc->maxexpnfilename,
	strlen( hc->hostdir ) + 1 + len );
    (void) strcpy( hc->expnfilename, hc->hostdir );
    (void) strcat( hc->expnfilename, "/" );
    (void) strcat( hc->expnfilename, tempfilename );
    return 1;
    }


/* Expands all symlinks in the given filename, eliding ..'s and leading /'s.
** Returns the expanded path (pointer to static string), or (char*) 0 on
** errors.  Also returns, in the string pointed to by restP, any trailing
** parts of the path that don't exist.
**
** This is a fairly nice little routine.  It handles any size filenames
** without excessive mallocs.
*/
static char*
expand_symlinks( char* path, char** restP, int no_symlink, int tildemapped )
    {
    static char* checked;
    static char* rest;
    char link[5000];
    static int maxchecked = 0, maxrest = 0;
    int checkedlen, restlen, linklen, prevcheckedlen, prevrestlen, nlinks, i;
    char* r;
    char* cp1;
    char* cp2;

    if ( no_symlink )
	{
	/* If we are chrooted, we can actually skip the symlink-expansion,
	** since it's impossible to get out of the tree.  However, we still
	** need to do the pathinfo check, and the existing symlink expansion
	** code is a pretty reasonable way to do this.  So, what we do is
	** a single stat() of the whole filename - if it exists, then we
	** return it as is with nothing in restP.  If it doesn't exist, we
	** fall through to the existing code.
	**
	** One side-effect of this is that users can't symlink to central
	** approved CGIs any more.  The workaround is to use the central
	** URL for the CGI instead of a local symlinked one.
	*/
	struct stat sb;
	if ( stat( path, &sb ) != -1 )
	    {
	    httpd_realloc_str( &checked, &maxchecked, strlen( path ) );
	    (void) strcpy( checked, path );
	    httpd_realloc_str( &rest, &maxrest, 0 );
	    rest[0] = '\0';
	    *restP = rest;
	    return checked;
	    }
	}

    /* Start out with nothing in checked and the whole filename in rest. */
    httpd_realloc_str( &checked, &maxchecked, 1 );
    checked[0] = '\0';
    checkedlen = 0;
    restlen = strlen( path );
    httpd_realloc_str( &rest, &maxrest, restlen );
    (void) strcpy( rest, path );
    if ( restlen > 0 && rest[restlen - 1] == '/' )
	rest[--restlen] = '\0';         /* trim trailing slash */
    if ( ! tildemapped )
	/* Remove any leading slashes. */
	while ( rest[0] == '/' )
	    {
	    (void) strcpy( rest, &(rest[1]) );
	    --restlen;
	    }
    r = rest;
    nlinks = 0;

    /* While there are still components to check... */
    while ( restlen > 0 )
	{
	/* Save current checkedlen in case we get a symlink.  Save current
	** restlen in case we get a non-existant component.
	*/
	prevcheckedlen = checkedlen;
	prevrestlen = restlen;

	/* Grab one component from r and transfer it to checked. */
	cp1 = strchr( r, '/' );
	if ( cp1 != (char*) 0 )
	    {
	    i = cp1 - r;
	    if ( i == 0 )
		{
		/* Special case for absolute paths. */
		httpd_realloc_str( &checked, &maxchecked, checkedlen + 1 );
		(void) strncpy( &checked[checkedlen], r, 1 );
		checkedlen += 1;
		}
	    else if ( strncmp( r, "..", MAX( i, 2 ) ) == 0 )
		{
		/* Ignore ..'s that go above the start of the path. */
		if ( checkedlen != 0 )
		    {
		    cp2 = strrchr( checked, '/' );
		    if ( cp2 == (char*) 0 )
			checkedlen = 0;
		    else if ( cp2 == checked )
			checkedlen = 1;
		    else
			checkedlen = cp2 - checked;
		    }
		}
	    else
		{
		httpd_realloc_str( &checked, &maxchecked, checkedlen + 1 + i );
		if ( checkedlen > 0 && checked[checkedlen-1] != '/' )
		    checked[checkedlen++] = '/';
		(void) strncpy( &checked[checkedlen], r, i );
		checkedlen += i;
		}
	    checked[checkedlen] = '\0';
	    r += i + 1;
	    restlen -= i + 1;
	    }
	else
	    {
	    /* No slashes remaining, r is all one component. */
	    if ( strcmp( r, ".." ) == 0 )
		{
		/* Ignore ..'s that go above the start of the path. */
		if ( checkedlen != 0 )
		    {
		    cp2 = strrchr( checked, '/' );
		    if ( cp2 == (char*) 0 )
			checkedlen = 0;
		    else if ( cp2 == checked )
			checkedlen = 1;
		    else
			checkedlen = cp2 - checked;
		    checked[checkedlen] = '\0';
		    }
		}
	    else
		{
		httpd_realloc_str(
		    &checked, &maxchecked, checkedlen + 1 + restlen );
		if ( checkedlen > 0 && checked[checkedlen-1] != '/' )
		    checked[checkedlen++] = '/';
		(void) strcpy( &checked[checkedlen], r );
		checkedlen += restlen;
		}
	    r += restlen;
	    restlen = 0;
	    }

	/* Try reading the current filename as a symlink */
	if ( checked[0] == '\0' )
	    continue;
	linklen = readlink( checked, link, sizeof(link) );
	if ( linklen == -1 )
	    {
	    if ( errno == EINVAL )
		continue;               /* not a symlink */
	    if ( errno == EACCES || errno == ENOENT || errno == ENOTDIR )
		{
		/* That last component was bogus.  Restore and return. */
		*restP = r - ( prevrestlen - restlen );
		if ( prevcheckedlen == 0 )
		    (void) strcpy( checked, "." );
		else
		    checked[prevcheckedlen] = '\0';
		return checked;
		}
	    syslog( LOG_ERR, "readlink %.80s - %m", checked );
	    return (char*) 0;
	    }
	++nlinks;
	if ( nlinks > MAX_LINKS )
	    {
	    syslog( LOG_ERR, "too many symlinks in %.80s", path );
	    return (char*) 0;
	    }
	link[linklen] = '\0';
	if ( link[linklen - 1] == '/' )
	    link[--linklen] = '\0';     /* trim trailing slash */

	/* Insert the link contents in front of the rest of the filename. */
	if ( restlen != 0 )
	    {
	    (void) strcpy( rest, r );
	    httpd_realloc_str( &rest, &maxrest, restlen + linklen + 1 );
	    for ( i = restlen; i >= 0; --i )
		rest[i + linklen + 1] = rest[i];
	    (void) strcpy( rest, link );
	    rest[linklen] = '/';
	    restlen += linklen + 1;
	    r = rest;
	    }
	else
	    {
	    /* There's nothing left in the filename, so the link contents
	    ** becomes the rest.
	    */
	    httpd_realloc_str( &rest, &maxrest, linklen );
	    (void) strcpy( rest, link );
	    restlen = linklen;
	    r = rest;
	    }

	if ( rest[0] == '/' )
	    {
	    /* There must have been an absolute symlink - zero out checked. */
	    checked[0] = '\0';
	    checkedlen = 0;
	    }
	else
	    {
	    /* Re-check this component. */
	    checkedlen = prevcheckedlen;
	    checked[checkedlen] = '\0';
	    }
	}

    /* Ok. */
    *restP = r;
    if ( checked[0] == '\0' )
	(void) strcpy( checked, "." );
    return checked;
    }


static int
httpd_request_reset0( httpd_conn* hc )
{
/*
** Don't reset the following fields because the caller has to deal with them.
**
**  hc->read_idx = 0;
**  hc->checked_idx = 0;
**  hc->checked_state = CHST_FIRSTWORD;
*/
    hc->method = METHOD_UNKNOWN;
    hc->status = 0;
    hc->bytes_to_send = 0;
    hc->bytes_sent = 0;
    hc->encodedurl = "";
    hc->decodedurl[0] = '\0';
    hc->protocol = "UNKNOWN";
    hc->origfilename[0] = '\0';
    hc->expnfilename[0] = '\0';
    hc->encodings[0] = '\0';
    hc->pathinfo[0] = '\0';
    hc->query[0] = '\0';
    hc->referer = "";
    hc->useragent = "";
    hc->accept[0] = '\0';
    hc->accepte[0] = '\0';
    hc->acceptl = "";
    hc->cookie = "";
    hc->contenttype = "";
    hc->reqhost[0] = '\0';
    hc->hdrhost = "";
    hc->hostdir[0] = '\0';
    hc->authorization = "";
    hc->remoteuser[0] = '\0';
    hc->response[0] = '\0';
#ifdef TILDE_MAP_2
    hc->altdir[0] = '\0';
#endif /* TILDE_MAP_2 */
    hc->responselen = 0;
    hc->if_modified_since = (time_t) -1;
    hc->range_if = (time_t) -1;
    hc->contentlength = -1;
    hc->type = "";
    hc->hostname = (char*) 0;
    hc->mime_flag = 1;
    hc->one_one = 0;
    hc->got_range = 0;
    hc->tildemapped = 0;
    hc->init_byte_loc = 0;
    hc->end_byte_loc = -1;
    hc->keep_alive = 0;
    hc->do_keep_alive = 0;
    hc->should_linger = 0;
    hc->file_fd = EOF;
    hc->file_address = (char*) 0;
#if defined(NAS) || defined(DVR)
    	hc->session[0] = '\0';
#endif
#ifdef DVR
	hc->send_jpeg_buffer = 0;
	hc->current_ch = 0;
	hc->current_card = 0;
	hc->mode = 0;
	hc->broadcast = 0;
#endif
    return GC_OK;
}


int
httpd_request_reset(httpd_conn* hc )
{
    hc->read_idx = 0;
    hc->checked_idx = 0;
    hc->checked_state = CHST_FIRSTWORD;
    return( httpd_request_reset0( hc ) );
}


int
httpd_request_reset2(httpd_conn* hc )
{
    if ( hc->checked_idx >= hc->read_idx )
	{
	hc->read_idx = 0;
	hc->checked_idx = 0;
	hc->checked_state = CHST_FIRSTWORD;
	}
    else
	{  /* move contents to the front of buffer */
	hc->read_idx -= hc->checked_idx;
	if ( hc->read_idx > 0  && hc->checked_idx > 0 )
	    (void) memmove( hc->read_buf, &hc->read_buf[hc->checked_idx],
				hc->read_idx );
	hc->checked_idx = 0;
	hc->checked_state = CHST_FIRSTWORD;
	}
    return( httpd_request_reset0( hc ) );
}


int
httpd_get_conn( httpd_server* hs, int listen_fd, httpd_conn* hc )
    {
    httpd_sockaddr sa;
    int sz;

    if ( ! hc->initialized )
	{
	hc->read_size = 0;
	//httpd_realloc_str( &hc->read_buf, &hc->read_size, 500 );
	httpd_realloc_str( &hc->read_buf, &hc->read_size, 1536 );
	hc->maxdecodedurl =
	    hc->maxorigfilename = hc->maxexpnfilename = hc->maxencodings =
	    hc->maxpathinfo = hc->maxquery = hc->maxaccept =
	    hc->maxaccepte = hc->maxreqhost = hc->maxhostdir =
	    hc->maxremoteuser = hc->maxresponse = 0;
#ifdef TILDE_MAP_2
	hc->maxaltdir = 0;
#endif /* TILDE_MAP_2 */
	httpd_realloc_str( &hc->decodedurl, &hc->maxdecodedurl, 1 );
	httpd_realloc_str( &hc->origfilename, &hc->maxorigfilename, 1 );
	httpd_realloc_str( &hc->expnfilename, &hc->maxexpnfilename, 0 );
	httpd_realloc_str( &hc->encodings, &hc->maxencodings, 0 );
	httpd_realloc_str( &hc->pathinfo, &hc->maxpathinfo, 0 );
	httpd_realloc_str( &hc->query, &hc->maxquery, 0 );
	httpd_realloc_str( &hc->accept, &hc->maxaccept, 0 );
	httpd_realloc_str( &hc->accepte, &hc->maxaccepte, 0 );
	httpd_realloc_str( &hc->reqhost, &hc->maxreqhost, 0 );
	httpd_realloc_str( &hc->hostdir, &hc->maxhostdir, 0 );
	httpd_realloc_str( &hc->remoteuser, &hc->maxremoteuser, 0 );
	httpd_realloc_str( &hc->response, &hc->maxresponse, 0 );
#if defined(NAS) || defined(DVR)
	hc->maxsession = 0;
    	httpd_realloc_str(&hc->session, &hc->maxsession, 0);
#endif
#ifdef TILDE_MAP_2
	httpd_realloc_str( &hc->altdir, &hc->maxaltdir, 0 );
#endif /* TILDE_MAP_2 */
	hc->initialized = 1;
	}

    /* Accept the new connection. */
    sz = sizeof(sa);
    hc->conn_fd = accept( listen_fd, &sa.sa, &sz );
    if ( hc->conn_fd < 0 )
	{
	if ( errno == EWOULDBLOCK )
	    return GC_NO_MORE;
	syslog( LOG_ERR, "accept - %m" );
	return GC_FAIL;
	}
    if ( ! sockaddr_check( &sa ) )
	{
	syslog( LOG_ERR, "unknown sockaddr family" );
	(void) close( hc->conn_fd );
	hc->conn_fd = -1;
	return GC_FAIL;
	}
    if ( hs->cgi_pattern != (char*) 0 )
	/* close-on-exec */
	(void) fcntl( hc->conn_fd, F_SETFD, 1 );
    hc->hs = hs;
    memset( &hc->client_addr, 0, sizeof(hc->client_addr) );
    memcpy( &hc->client_addr, &sa, sockaddr_len( &sa ) );

    return httpd_request_reset(hc);
    }


/* Checks hc->read_buf to see whether a complete request has been read so far;
** either the first line has two words (an HTTP/0.9 request), or the first
** line has three words and there's a blank line present.
**
** hc->read_idx is how much has been read in; hc->checked_idx is how much we
** have checked so far; and hc->checked_state is the current state of the
** finite state machine.
*/
int
httpd_got_request( httpd_conn* hc )
    {
    char c;

    for ( ; hc->checked_idx < hc->read_idx; ++hc->checked_idx )
	{
	c = hc->read_buf[hc->checked_idx];
	switch ( hc->checked_state )
	    {
	    case CHST_FIRSTWORD:
	    switch ( c )
		{
		case ' ': case '\t':
		if ( hc->checked_idx < 2 )
		    {
		    hc->checked_state = CHST_BOGUS;
		    return GR_BAD_REQUEST;
		    }
		hc->checked_state = CHST_FIRSTWS;
		break;
		case '\n': case '\r':
		hc->checked_state = CHST_BOGUS;
		return GR_BAD_REQUEST;
		}
	    break;
	    case CHST_FIRSTWS:
	    switch ( c )
		{
		case ' ': case '\t':
		break;
		case '\n': case '\r':
		hc->checked_state = CHST_BOGUS;
		return GR_BAD_REQUEST;
		default:
		hc->checked_state = CHST_SECONDWORD;
		break;
		}
	    break;
	    case CHST_SECONDWORD:
	    switch ( c )
		{
		case ' ': case '\t':
		hc->checked_state = CHST_SECONDWS;
		break;
		case '\n': case '\r':
		/* The first line has only two words - an HTTP/0.9 request. */
		return GR_GOT_REQUEST;
		}
	    break;
	    case CHST_SECONDWS:
	    switch ( c )
		{
		case ' ': case '\t':
		break;
		case '\n': case '\r':
		hc->checked_state = CHST_BOGUS;
		return GR_BAD_REQUEST;
		default:
		hc->checked_state = CHST_THIRDWORD;
		break;
		}
	    break;
	    case CHST_THIRDWORD:
	    switch ( c )
		{
		case ' ': case '\t':
		hc->checked_state = CHST_THIRDWS;
		break;
		case '\n':
		hc->checked_state = CHST_LF;
		break;
		case '\r':
		hc->checked_state = CHST_CR;
		break;
		}
	    break;
	    case CHST_THIRDWS:
	    switch ( c )
		{
		case ' ': case '\t':
		break;
		case '\n':
		hc->checked_state = CHST_LF;
		break;
		case '\r':
		hc->checked_state = CHST_CR;
		break;
		default:
		hc->checked_state = CHST_BOGUS;
		return GR_BAD_REQUEST;
		}
	    break;
	    case CHST_LINE:
	    switch ( c )
		{
		case '\n':
		hc->checked_state = CHST_LF;
		break;
		case '\r':
		hc->checked_state = CHST_CR;
		break;
		}
	    break;
	    case CHST_LF:
	    switch ( c )
		{
		case '\n':
		/* Two newlines in a row - a blank line - end of request. */
		return GR_GOT_REQUEST;
		case '\r':
		hc->checked_state = CHST_CR;
		break;
		default:
		hc->checked_state = CHST_LINE;
		break;
		}
	    break;
	    case CHST_CR:
	    switch ( c )
		{
		case '\n':
		hc->checked_state = CHST_CRLF;
		break;
		case '\r':
		/* Two returns in a row - end of request. */
		return GR_GOT_REQUEST;
		default:
		hc->checked_state = CHST_LINE;
		break;
		}
	    break;
	    case CHST_CRLF:
	    switch ( c )
		{
		case '\n':
		/* Two newlines in a row - end of request. */
		return GR_GOT_REQUEST;
		case '\r':
		hc->checked_state = CHST_CRLFCR;
		break;
		default:
		hc->checked_state = CHST_LINE;
		break;
		}
	    break;
	    case CHST_CRLFCR:
	    switch ( c )
		{
		case '\n': case '\r':
		/* Two CRLFs or two CRs in a row - end of request. */
		return GR_GOT_REQUEST;
		default:
		hc->checked_state = CHST_LINE;
		break;
		}
	    break;
	    case CHST_BOGUS:
	    return GR_BAD_REQUEST;
	    }
	}
    return GR_NO_REQUEST;
    }


int
httpd_parse_request( httpd_conn* hc )
    {
    char* buf;
    char* method_str;
    char* url;
    char* protocol;
    char* reqhost;
    char* eol;
    char* cp;
    char* pi;

    hc->checked_idx = 0;	/* reset */
    method_str = bufgets( hc );
    url = strpbrk( method_str, " \t\n\r" );
    if ( url == (char*) 0 )
	{
	httpd_send_err( hc, 400, httpd_err400title, "", httpd_err400form, "" );
	return -1;
	}
    *url++ = '\0';
    url += strspn( url, " \t\n\r" );
    protocol = strpbrk( url, " \t\n\r" );
    if ( protocol == (char*) 0 )
	{
	protocol = "HTTP/0.9";
	hc->mime_flag = 0;
	}
    else
	{
	*protocol++ = '\0';
	protocol += strspn( protocol, " \t\n\r" );
	if ( *protocol == '\0' )
	    {	/* trailing spaces, ignore them */
		/* NOTE: httpd_got_request() should prevent this to happen */
		protocol = "HTTP/0.9";
		hc->mime_flag = 0;
	    }
	else
	    {
	    char *pszTmp;
	    int httpMajorVersion = 1;
	    int httpMinorVersion = 0;

	    eol = strpbrk( protocol, " \t\n\r" );
	    if ( eol != (char*) 0 )
		*eol = '\0';

	    if ( strncasecmp( protocol, "HTTP/", 5 ) != 0 ||
		 !isdigit( protocol[5] ) )
		{   /* unknown protocol, reply */
		hc->protocol = "HTTP/1.0";
		httpd_send_err( hc, 400, httpd_err400title, "",
			httpd_err400form, "" );
		return -1;
		}

	    httpMajorVersion = atoi( &protocol[5] );
	    if ( ( pszTmp = strchr( &protocol[5], '.' ) ) != NULL &&
		isdigit( pszTmp[1] ) )
		{
		httpMinorVersion = atoi( &pszTmp[1] );
		}

	    if (  httpMajorVersion  < 0  ||
		  httpMajorVersion  > 99 ||
		  httpMinorVersion  < 0  ||
		  httpMinorVersion  > 9999 ||
		( httpMajorVersion == 0 &&
		  httpMinorVersion != 9 ) )
		{    /* unsupported protocol */
		hc->protocol = "HTTP/1.0";
		httpd_send_err( hc, 400, httpd_err400title, "",
			httpd_err400form, "" );
		return -1;
		}

	    if (   httpMajorVersion > 1 ||
		 ( httpMajorVersion == 1 && httpMinorVersion >= 1 ) )
		{   /* force HTTP version to what is supported */
		    protocol = "HTTP/1.1";
		    hc->one_one = 1;
		    /* by default it has a persistent connection */
		    hc->keep_alive = 1;
		    hc->do_keep_alive = 1;
		}
	    else
	    if ( httpMajorVersion == 1 )
		/* downgrade HTTP version to requested HTTP/1.0 */
		 protocol = "HTTP/1.0";
	    else
		{
		/* downgrade HTTP version to requested HTTP/0.9 */
		 protocol = "HTTP/0.9";
		hc->mime_flag = 0;
		}
	    }
	}

    hc->protocol = protocol;

    /* Check for HTTP/1.1 absolute URL. */
    if ( strncasecmp( url, "http://", 7 ) == 0 )
	{
	if ( ! hc->one_one )
	    {
	    httpd_send_err( hc, 400, httpd_err400title, "",
		httpd_err400form, "" );
	    return -1;
	    }
	reqhost = url + 7;
	url = strchr( reqhost, '/' );
	if ( url == (char*) 0 )
	    {
	    httpd_send_err( hc, 400, httpd_err400title, "", httpd_err400form, "" );
	    return -1;
	    }
	*url = '\0';
	httpd_realloc_str( &hc->reqhost, &hc->maxreqhost, strlen( reqhost ) );
	(void) strcpy( hc->reqhost, reqhost );
	*url = '/';
	}

    if ( strcasecmp( method_str, httpd_method_str( METHOD_GET ) ) == 0 )
	hc->method = METHOD_GET;
    else if ( strcasecmp( method_str, httpd_method_str( METHOD_HEAD ) ) == 0 )
	hc->method = METHOD_HEAD;
    else if ( strcasecmp( method_str, httpd_method_str( METHOD_POST ) ) == 0 )
	hc->method = METHOD_POST;
    else
	{
	httpd_send_err( hc, 501, err501title, "", err501form, method_str );
	return -1;
	}

    hc->encodedurl = url;
    httpd_realloc_str(
	&hc->decodedurl, &hc->maxdecodedurl, strlen( hc->encodedurl ) );
    strdecode( hc->decodedurl, hc->encodedurl );

	//DebugOutput("encodeurl: %s, decodeurl: %s\n", hc->encodedurl, hc->decodedurl);

    de_dotdot( hc->decodedurl );
    if ( hc->decodedurl[0] != '/' || hc->decodedurl[1] == '/' ||
	 ( hc->decodedurl[1] == '.' && hc->decodedurl[2] == '.' &&
	   ( hc->decodedurl[3] == '\0' || hc->decodedurl[3] == '/' ) ) )
	{
	httpd_send_err( hc, 400, httpd_err400title, "", httpd_err400form, "" );
	return -1;
	}
#ifdef DVR
	//DebugOutput("encodeurl: %s, useragent: %s\n", hc->encodedurl, hc->useragent ? hc->useragent : "");
	//if (strstr(hc->encodedurl, "/IEIDVR?CH=") && hc->useragent && strstr(hc->useragent, "IEI-DVR-Client")) {
	if (strstr(hc->encodedurl, "/IEIDVR?CH=")) {
		char temp[32], length = strlen(hc->encodedurl);
		hc->send_jpeg_buffer = 1;
		hc->mode = 1;
		hc->current_ch = 0;
		hc->current_card = 0;
		if (length > 11) {
			strncpy(temp, &hc->encodedurl[11], ((length > 12) ? 2 : 1));
			hc->current_ch = atoi(temp);
			if (length > 18) {
				strncpy(temp, &hc->encodedurl[18], ((length > 19) ? 2 : 1));
				hc->current_card = atoi(temp);
			}
		}
		if ((hc->current_ch < 0) || (hc->current_ch >= MAX_CHANNEL_PER_CHIP))
			hc->current_ch = 0;
		if ((hc->current_card < 0) || (hc->current_card >= MAX_CHIP_NUMBER))
			hc->current_card = 0;
		//DebugOutput("channel: %d\n", hc->current_ch);
	}
	else if (strstr(hc->encodedurl, "/IEIDVRS"))
		hc->send_jpeg_buffer = 2;

#endif

    httpd_realloc_str(
	&hc->origfilename, &hc->maxorigfilename, strlen( hc->decodedurl ) );
    (void) strcpy( hc->origfilename, &hc->decodedurl[1] );
    /* Special case for top-level URL. */
    if ( hc->origfilename[0] == '\0' )
	(void) strcpy( hc->origfilename, "." );
#ifdef DVR
	if (hc->send_jpeg_buffer == 0) {
#endif
    /* Extract query string from encoded URL. */
    cp = strchr( hc->encodedurl, '?' );
    //if ( cp != (char*) 0 && cp[1] )
    if ( cp != (char*) 0 )
	{
	++cp;
	httpd_realloc_str( &hc->query, &hc->maxquery, strlen( cp ) );
	(void) strcpy( hc->query, cp );
	/* 07-JAN-2002 Cameron Gregory */
	/* And remove query from filename. */
	cp = strchr( hc->origfilename, '?' );
	if ( cp != (char*) 0 )
	    *cp = '\0';
	}
#ifdef DVR
	}
#endif
	//DebugOutput("encodeurl: %s, decodeurl: %s, origfilename: %s\n", hc->encodedurl, hc->decodedurl, hc->origfilename);
    if ( hc->mime_flag )
	{
	/* Read the MIME headers. */
	while ( ( buf = bufgets( hc ) ) != (char*) 0 )
	    {
	    if ( buf[0] == '\0' )
		break;
	    if ( strncasecmp( buf, "Referer:", 8 ) == 0 )
		{
		cp = &buf[8];
		cp += strspn( cp, " \t" );
		hc->referer = cp;
		}
	    else if ( strncasecmp( buf, "User-Agent:", 11 ) == 0 )
		{
		cp = &buf[11];
		cp += strspn( cp, " \t" );
		hc->useragent = cp;
		}
	    else if ( strncasecmp( buf, "Host:", 5 ) == 0 )
		{
		cp = &buf[5];
		cp += strspn( cp, " \t" );
		hc->hdrhost = cp;
		cp = strchr( hc->hdrhost, ':' );
		if ( cp != (char*) 0 )
		    *cp = '\0';
		}
	    else if ( strncasecmp( buf, "Accept:", 7 ) == 0 )
		{
		cp = &buf[7];
		cp += strspn( cp, " \t" );
		if ( hc->accept[0] != '\0' )
		    {
		    if ( strlen( hc->accept ) > 5000 )
			{
			syslog(
			    LOG_ERR, "%.80s way too much Accept: data",
			    httpd_ntoa( &hc->client_addr ) );
			continue;
			}
		    httpd_realloc_str(
			&hc->accept, &hc->maxaccept,
			strlen( hc->accept ) + 2 + strlen( cp ) );
		    (void) strcat( hc->accept, ", " );
		    }
		else
		    httpd_realloc_str(
			&hc->accept, &hc->maxaccept, strlen( cp ) );
		(void) strcat( hc->accept, cp );
		}
	    else if ( strncasecmp( buf, "Accept-Encoding:", 16 ) == 0 )
		{
		cp = &buf[16];
		cp += strspn( cp, " \t" );
		if ( hc->accepte[0] != '\0' )
		    {
		    if ( strlen( hc->accepte ) > 5000 )
			{
			syslog(
			    LOG_ERR, "%.80s way too much Accept-Encoding: data",
			    httpd_ntoa( &hc->client_addr ) );
			continue;
			}
		    httpd_realloc_str(
			&hc->accepte, &hc->maxaccepte,
			strlen( hc->accepte ) + 2 + strlen( cp ) );
		    (void) strcat( hc->accepte, ", " );
		    }
		else
		    httpd_realloc_str(
			&hc->accepte, &hc->maxaccepte, strlen( cp ) );
		(void) strcpy( hc->accepte, cp );
		}
	    else if ( strncasecmp( buf, "Accept-Language:", 16 ) == 0 )
		{
		cp = &buf[16];
		cp += strspn( cp, " \t" );
		hc->acceptl = cp;
		}
	    else if ( strncasecmp( buf, "If-Modified-Since:", 18 ) == 0 )
		{
		cp = &buf[18];
		hc->if_modified_since = tdate_parse( cp );
		if ( hc->if_modified_since == (time_t) -1 )
		    syslog( LOG_DEBUG, "unparsable time: %.80s", cp );
		}
	    else if ( strncasecmp( buf, "Cookie:", 7 ) == 0 )
		{
		cp = &buf[7];
		cp += strspn( cp, " \t" );
		hc->cookie = cp;
		}
	    else if ( strncasecmp( buf, "Range:", 6 ) == 0 )
		{
		/* Only support %d- and %d-%d, not %d-%d,%d-%d or -%d. */
		if ( strchr( buf, ',' ) == (char*) 0 )
		    {
		    char* cp_dash;
		    cp = strpbrk( buf, "=" );
		    if ( cp != (char*) 0 )
			{
			cp_dash = strchr( cp + 1, '-' );
			if ( cp_dash != (char*) 0 && cp_dash != cp + 1 )
			    {
			    *cp_dash = '\0';
			    hc->init_byte_loc = atol( cp + 1 );
			    if ( isdigit( (int) cp_dash[1] ) )
				hc->end_byte_loc = atol( cp_dash + 1 );
			    else
				hc->end_byte_loc = -1L;
			    if ( hc->init_byte_loc <  0L ||
				 hc->end_byte_loc  < -1L ||
				(hc->end_byte_loc  >  0L &&
				 hc->end_byte_loc  < hc->init_byte_loc) )
				hc->got_range = 0;
			    else
				hc->got_range = 1;
			    }
			}
		    }
		}
	    else if ( strncasecmp( buf, "Range-If:", 9 ) == 0 ||
		      strncasecmp( buf, "If-Range:", 9 ) == 0 )
		{
		cp = &buf[9];
		hc->range_if = tdate_parse( cp );
		if ( hc->range_if == (time_t) -1 )
		    syslog( LOG_DEBUG, "unparsable time: %.80s", cp );
		}
	    else if ( strncasecmp( buf, "Content-Type:", 13 ) == 0 )
		{
		cp = &buf[13];
		cp += strspn( cp, " \t" );
		hc->contenttype = cp;
		}
	    else if ( strncasecmp( buf, "Content-Length:", 15 ) == 0 )
		{
		cp = &buf[15];
		hc->contentlength = atol( cp );
		}
	    else if ( strncasecmp( buf, "Authorization:", 14 ) == 0 )
		{
		cp = &buf[14];
		cp += strspn( cp, " \t" );
		hc->authorization = cp;
		}
	    else if ( strncasecmp( buf, "Connection:", 11 ) == 0 )
		{
		cp = &buf[11];
		cp += strspn( cp, " \t" );
		if ( strcasecmp( cp, "close" ) == 0 )
		    {
		    /* we should not need linger but we are very cautios */
		    /* because client might send some trailing CRLF */
		    hc->should_linger = hc->one_one;
		    hc->keep_alive = 0;
		    hc->do_keep_alive = 0;
		    }
		else
		if ( strcasecmp( cp, "Keep-Alive" ) == 0 )
		    {
		    /* there can be pipelined data requests */
		    hc->keep_alive = 1;
		    /* only HTTP/1.1 allows persistent connections */
		    hc->do_keep_alive = hc->one_one;
		    }
		else
		    {
		    /* unknown, but there could be pipelined data requests */
		    hc->keep_alive = 1;
		    hc->do_keep_alive = 0;
		    }
		}
#ifdef LOG_UNKNOWN_HEADERS
	    else if ( strncasecmp( buf, "Accept-Charset:", 15 ) == 0 ||
		      strncasecmp( buf, "Accept-Language:", 16 ) == 0 ||
		      strncasecmp( buf, "Agent:", 6 ) == 0 ||
		      strncasecmp( buf, "Cache-Control:", 14 ) == 0 ||
		      strncasecmp( buf, "Cache-Info:", 11 ) == 0 ||
		      strncasecmp( buf, "Charge-To:", 10 ) == 0 ||
		      strncasecmp( buf, "Client-IP:", 10 ) == 0 ||
		      strncasecmp( buf, "Date:", 5 ) == 0 ||
		      strncasecmp( buf, "Extension:", 10 ) == 0 ||
		      strncasecmp( buf, "Forwarded:", 10 ) == 0 ||
		      strncasecmp( buf, "From:", 5 ) == 0 ||
		      strncasecmp( buf, "HTTP-Version:", 13 ) == 0 ||
		      strncasecmp( buf, "Max-Forwards:", 13 ) == 0 ||
		      strncasecmp( buf, "Message-Id:", 11 ) == 0 ||
		      strncasecmp( buf, "MIME-Version:", 13 ) == 0 ||
		      strncasecmp( buf, "Negotiate:", 10 ) == 0 ||
		      strncasecmp( buf, "Pragma:", 7 ) == 0 ||
		      strncasecmp( buf, "Proxy-Agent:", 12 ) == 0 ||
		      strncasecmp( buf, "Proxy-Connection:", 17 ) == 0 ||
		      strncasecmp( buf, "Security-Scheme:", 16 ) == 0 ||
		      strncasecmp( buf, "Session-Id:", 11 ) == 0 ||
		      strncasecmp( buf, "UA-Color:", 9 ) == 0 ||
		      strncasecmp( buf, "UA-CPU:", 7 ) == 0 ||
		      strncasecmp( buf, "UA-Disp:", 8 ) == 0 ||
		      strncasecmp( buf, "UA-OS:", 6 ) == 0 ||
		      strncasecmp( buf, "UA-Pixels:", 10 ) == 0 ||
		      strncasecmp( buf, "User:", 5 ) == 0 ||
		      strncasecmp( buf, "Via:", 4 ) == 0 ||
		      strncasecmp( buf, "X-", 2 ) == 0 )
		; /* ignore */
	    else
		syslog( LOG_DEBUG, "unknown request header: %.80s", buf );
#endif /* LOG_UNKNOWN_HEADERS */
	    }
	}

    if ( hc->one_one )
	{
	/* Check that HTTP/1.1 requests specify a host, as required. */
	if ( hc->reqhost[0] == '\0' && hc->hdrhost[0] == '\0' )
	    {
	    httpd_send_err( hc, 400, httpd_err400title, "", httpd_err400form, "" );
	    return -1;
	    }

	/* If the client wants to do keep-alives, it might also be doing
	** pipelining.  There's no way for us to tell without doubts.
	** Since we may disable keep-alive, if we close such a connection
	** there might be unread pipelined requests waiting.
	** So, we have to do a lingering close.
	*/
	if ( hc->keep_alive )
	    hc->should_linger = 1;

	/*
	**  Disable keep alive support for bad browsers,
	**    list taken from Apache 1.3.19
	*/
	if ( hc->do_keep_alive &&
	    ( strstr(hc->useragent, "Mozilla/2") != NULL ||
	      strstr(hc->useragent, "MSIE 4.0b2;") != NULL ) )
	    hc->do_keep_alive = 0;
	}

    /* Ok, the request has been parsed.  Now we resolve stuff that
    ** may require the entire request.
    */

    /* Copy original filename to expanded filename. */
    httpd_realloc_str(
	&hc->expnfilename, &hc->maxexpnfilename, strlen( hc->origfilename ) );
    (void) strcpy( hc->expnfilename, hc->origfilename );

    /* Tilde mapping. */
    if ( hc->expnfilename[0] == '~' )
	{
#ifdef TILDE_MAP_1
	if ( ! tilde_map_1( hc ) )
	    {
	    httpd_send_err( hc, 404, err404title, "", err404form, hc->encodedurl );
	    return -1;
	    }
#endif /* TILDE_MAP_1 */
#ifdef TILDE_MAP_2
	if ( ! tilde_map_2( hc ) )
	    {
	    httpd_send_err( hc, 404, err404title, "", err404form, hc->encodedurl );
	    return -1;
	    }
#endif /* TILDE_MAP_2 */
	}

    /* Virtual host mapping. */
    if ( hc->hs->vhost )
	if ( ! vhost_map( hc ) )
	    {
	    httpd_send_err( hc, 500, err500title, "", err500form, hc->encodedurl );
	    return -1;
	    }

    /* Expand all symbolic links in the filename.  This also gives us
    ** any trailing non-existing components, for pathinfo.
    */
    cp = expand_symlinks( hc->expnfilename, &pi, hc->hs->no_symlink, hc->tildemapped );
    if ( cp == (char*) 0 )
	{
	httpd_send_err( hc, 500, err500title, "", err500form, hc->encodedurl );
	return -1;
	}
    httpd_realloc_str( &hc->expnfilename, &hc->maxexpnfilename, strlen( cp ) );
    (void) strcpy( hc->expnfilename, cp );
    httpd_realloc_str( &hc->pathinfo, &hc->maxpathinfo, strlen( pi ) );
    (void) strcpy( hc->pathinfo, pi );

    /* Remove pathinfo stuff from the original filename too. */
    if ( hc->pathinfo[0] != '\0' )
	{
	int i;
	i = strlen( hc->origfilename ) - strlen( hc->pathinfo );
	if ( i > 0 && strcmp( &hc->origfilename[i], hc->pathinfo ) == 0 )
	    hc->origfilename[i - 1] = '\0';
	}

    /* If the expanded filename is an absolute path, check that it's still
    ** within the current directory or the alternate directory.
    */
    if ( hc->expnfilename[0] == '/' )
	{
	if ( strncmp(
		 hc->expnfilename, hc->hs->cwd, strlen( hc->hs->cwd ) ) == 0 )
	    {
	    /* Elide the current directory. */
	    (void) strcpy(
		hc->expnfilename, &hc->expnfilename[strlen( hc->hs->cwd )] );
	    }
#ifdef TILDE_MAP_2
	else if ( hc->altdir[0] != '\0' &&
		  ( strncmp(
		       hc->expnfilename, hc->altdir,
		       strlen( hc->altdir ) ) == 0 &&
		    ( hc->expnfilename[strlen( hc->altdir )] == '\0' ||
		      hc->expnfilename[strlen( hc->altdir )] == '/' ) ) )
	    {}
#endif /* TILDE_MAP_2 */
	else
	    {
#if defined(NAS) || defined(DVR)
		//===============================================================
		//	modify by Kent 2001/06/13
		//	to prevent 403 error
		//===============================================================
		return 0;
#endif
	    syslog(
		LOG_NOTICE, "%.80s URL \"%.80s\" goes outside the web tree",
		httpd_ntoa( &hc->client_addr ), hc->encodedurl );
	    httpd_send_err(
		hc, 403, err403title, "",
		ERROR_FORM( err403form, "The requested URL '%.80s' resolves to a file outside the permitted web server directory tree.\n" ),
		hc->encodedurl );
	    return -1;
	    }
	}

	//DebugOutput("encodeurl: %s, decodeurl: %s, origfilename: %s\n", hc->encodedurl, hc->decodedurl, hc->origfilename);

    return 0;
    }


static char*
bufgets( httpd_conn* hc )
    {
    int i;
    char c;

    for ( i = hc->checked_idx; hc->checked_idx < hc->read_idx; ++hc->checked_idx )
	{
	c = hc->read_buf[hc->checked_idx];
	if ( c == '\n' || c == '\r' )
	    {
	    hc->read_buf[hc->checked_idx] = '\0';
	    ++hc->checked_idx;
	    if ( c == '\r' && hc->checked_idx < hc->read_idx &&
		 hc->read_buf[hc->checked_idx] == '\n' )
		{
		hc->read_buf[hc->checked_idx] = '\0';
		++hc->checked_idx;
		}
	    return &(hc->read_buf[i]);
	    }
	}
    return (char*) 0;
    }


static void
de_dotdot( char* file )
    {
    char* cp;
    char* cp2;
    int l;

    /* Collapse any multiple / sequences. */
    while ( ( cp = strstr( file, "//") ) != (char*) 0 )
	{
	for ( cp2 = cp + 2; *cp2 == '/'; ++cp2 )
	    continue;
	(void) strcpy( cp + 1, cp2 );
	}

    /* Elide any xxx/../ sequences. */
    while ( ( cp = strstr( file, "/../" ) ) != (char*) 0 )
	{
	for ( cp2 = cp - 1; cp2 >= file && *cp2 != '/'; --cp2 )
	    continue;
	if ( cp2 < file )
	    break;
	(void) strcpy( cp2, cp + 3 );
	}

    /* Also elide any xxx/.. at the end. */
    while ( ( l = strlen( file ) ) > 3 &&
	    strcmp( ( cp = file + l - 3 ), "/.." ) == 0 )
	{
	for ( cp2 = cp - 1; cp2 >= file && *cp2 != '/'; --cp2 )
	    continue;
	if ( cp2 < file )
	    break;
	*cp2 = '\0';
	}
    }


void
httpd_complete_request( httpd_conn* hc, struct timeval* nowP, int logit )
    {
    if (logit)
	    make_log_entry( hc, nowP );

    if ( hc->file_fd != EOF || hc->file_address != (char*) 0 )
	{
#ifdef DVR
	if (hc->send_jpeg_buffer != 0) {
		if (hc->file_address)
			free(hc->file_address);
	}
	else
#endif
	{
		mmc_unmap( hc->file_fd, hc->file_address, &(hc->sb), nowP );
		hc->file_fd = EOF;
	}
	hc->file_address = (char*) 0;
	}
    }


void
httpd_close_conn_wr( httpd_conn* hc )
    {
    if ( hc->conn_fd >= 0 )
	{
	/* half close output stream of this connection */

	if (shutdown( hc->conn_fd, SHUT_WR ) == -1 && errno == EINTR)
	    /* got a signal, retry */
	    /* NOTE: nowadays this check is not necessary because */
	    /* shutdown() always succeds if fd is valid, anyway */
	    /* things might change in the near future */
	    (void) shutdown( hc->conn_fd, SHUT_WR );
	}
    }


void
httpd_close_conn( httpd_conn* hc, struct timeval* nowP )
    {

    if ( hc->conn_fd >= 0 )
	{
	if (close( hc->conn_fd ) == -1 && errno == EINTR)
	    /* NOTE: nowadays this check is not necessary because */
	    /* close() always succeds if fd is valid, anyway */
	    /* things might change in the near future */
	    (void) close( hc->conn_fd );
	hc->conn_fd = -1;
	}
    }

void
httpd_destroy_conn( httpd_conn* hc )
    {
    if ( hc->initialized )
	{
	free( (void*) hc->read_buf );
	free( (void*) hc->decodedurl );
	free( (void*) hc->origfilename );
	free( (void*) hc->expnfilename );
	free( (void*) hc->encodings );
	free( (void*) hc->pathinfo );
	free( (void*) hc->query );
	free( (void*) hc->accept );
	free( (void*) hc->accepte );
	free( (void*) hc->reqhost );
	free( (void*) hc->hostdir );
	free( (void*) hc->remoteuser );
	free( (void*) hc->response );
#if defined(NAS) || defined(DVR)
	free((void*) hc->session);
#endif
#ifdef TILDE_MAP_2
	free( (void*) hc->altdir );
#endif /* TILDE_MAP_2 */
	hc->initialized = 0;
	}
    }


struct mime_entry {
    char* ext;
    size_t ext_len;
    char* val;
    size_t val_len;
    };
static struct mime_entry enc_tab[] = {
#include "mime_encodings.h"
    };
static const int n_enc_tab = sizeof(enc_tab) / sizeof(*enc_tab);
static struct mime_entry typ_tab[] = {
#include "mime_types.h"
    };
static const int n_typ_tab = sizeof(typ_tab) / sizeof(*typ_tab);

struct mime_hash {
    struct mime_entry *pmime;
    struct mime_hash  *pnext;
    };

#define TYP_HASH_BITS	8	/* 8 = 256 entries, 9 = 512 entries, ecc. */
#define	TYP_HASH_SIZE	( 1 << TYP_HASH_BITS )
#define	TYP_HASH_MASK	( TYP_HASH_SIZE - 1 )

static size_t max_enc_ext_len;
static size_t max_typ_ext_len;
static struct mime_hash *free_mime_hash;
static struct mime_hash	*typ_hash_tab[TYP_HASH_SIZE];

#define DEFAULT_MIME_TYPE	"text/plain; charset=%s"

/* case-insensitive hash */

static unsigned int
hash_mime( char *buf, size_t len )
    {
    unsigned int h;

    for ( h = 5381; len > 0; --len, ++buf )
	{
	h += h << 5;
	if ( isupper( (int) *buf ) == 0 )
	    /* common case (fast) */
	    h ^= (int) *buf;
	else
	    /* uncommon case (slow) */
	    h ^= tolower( (int) *buf );
	}

    return( h & TYP_HASH_MASK );
    }

static void
init_mime( void )
    {
    int i;
    unsigned int h = 0;
    struct mime_hash *pmh1, *pmh2;

    /* Fill in the lengths. */
    for ( i = 0; i < n_enc_tab; ++i )
	{
	enc_tab[i].ext_len = strlen( enc_tab[i].ext );
	enc_tab[i].val_len = strlen( enc_tab[i].val );
	if ( enc_tab[i].ext_len > max_enc_ext_len )
	     max_enc_ext_len = enc_tab[i].ext_len;
	}
    for ( i = 0; i < n_typ_tab; ++i )
	{
	typ_tab[i].ext_len = strlen( typ_tab[i].ext );
	typ_tab[i].val_len = strlen( typ_tab[i].val );
	if ( typ_tab[i].ext_len > max_typ_ext_len )
	     max_typ_ext_len = typ_tab[i].ext_len;
	}

    if ( ( free_mime_hash = calloc( n_typ_tab, sizeof(struct mime_hash) ) )
	== NULL)
	{
	    syslog( LOG_CRIT, "init_mime: calloc failed, errno %m" );
	    exit( 102 );
	}
    for ( i = 0; i < n_typ_tab; ++i)
	{
	h = hash_mime( typ_tab[i].ext, typ_tab[i].ext_len );

	pmh1 = &free_mime_hash[i];
	pmh1->pmime = &typ_tab[i];

	if( typ_hash_tab[h] == NULL )
	    {
	    typ_hash_tab[h] = pmh1;
	    }
	else
	    {
	    pmh2 = typ_hash_tab[h];
	    typ_hash_tab[h] = pmh1;
	    pmh1->pnext = pmh2;
	    }
	}
    }


static void
figure_mime( httpd_conn* hc )
    {
    char* prev_dot;
    char* dot;
    char* ext;
    size_t ext_len, encodings_len;
    int i;

#ifdef DVR
	if (hc->send_jpeg_buffer != 0) {
		hc->type = "image/jpeg-v2";
		return;
	}
#endif

    hc->encodings[0] = '\0';
    encodings_len = 0;

    /* Peel off encoding extensions until there aren't any more. */
    for ( prev_dot = &hc->expnfilename[strlen(hc->expnfilename)]; ;
	  prev_dot = dot )
	{
	for ( dot = prev_dot - 1;
	      dot >= hc->expnfilename && *dot != '.';
	    --dot )
	    ;
	if ( dot < hc->expnfilename )
	    {
	    /* No dot found.  No more encoding extensions, and no type
	    ** extension either.
	    */
	    hc->type = DEFAULT_MIME_TYPE;
	    return;
	    }
	ext = dot + 1;
	ext_len = prev_dot - ext;

	if ( ext_len > max_enc_ext_len )
	    /* No encoding extension can be found. */
	    /* Break and look for a type extension. */
	    break;

	/* Search the encodings table.  Linear search is fine here, there
	** are only a few entries.
	*/

	for ( i = 0; i < n_enc_tab; ++i )
	    {
	    if ( ext_len == enc_tab[i].ext_len &&
		strncasecmp( ext, enc_tab[i].ext, ext_len ) == 0 )
		{
		httpd_realloc_str(
		    &hc->encodings, &hc->maxencodings,
		    enc_tab[i].val_len + 1 );
		if ( hc->encodings[0] != '\0' )
		    {
		    (void) strcpy( &hc->encodings[encodings_len], ";" );
		    ++encodings_len;
		    }
		(void) strcpy( &hc->encodings[encodings_len], enc_tab[i].val );
		encodings_len += enc_tab[i].val_len;
		break;
		}
	    }
	if ( i >= n_enc_tab )
	    /* No encoding extension found. */
	    /* Break and look for a type extension. */
	    break;
	}

    if ( ext_len > max_typ_ext_len )
	{
	hc->type = DEFAULT_MIME_TYPE;
	return;
	}
    /* else */
	{
	unsigned int h;
	struct mime_hash *pmh1;

	h = hash_mime( ext, ext_len );

	for ( pmh1 = typ_hash_tab[h]; pmh1 != NULL; pmh1 = pmh1->pnext )
	    {
	    if ( pmh1->pmime->ext_len == ext_len &&
		strncasecmp( ext, pmh1->pmime->ext, ext_len ) == 0 )
		{
		hc->type = pmh1->pmime->val;
		return;
		}
	    }
	}
    hc->type = DEFAULT_MIME_TYPE;
    return;

    }

#ifdef CGI_TIMELIMIT
static void
cgi_kill2( ClientData client_data, struct timeval* nowP )
    {
    pid_t pid;

    /* Before trying to kill the CGI process, reap any zombie processes.
    ** That may get rid of the CGI process.
    */
    (void) do_reap();

    pid = (pid_t) client_data.i;
    if ( kill( pid, SIGKILL ) == 0 )
    	//DebugOutput("cgi_kill\n");
	syslog( LOG_ERR, "hard-killed CGI process %d", pid );
    }

static void
cgi_kill( ClientData client_data, struct timeval* nowP )
    {
    pid_t pid;

    /* Before trying to kill the CGI process, reap any zombie processes.
    ** That may get rid of the CGI process.
    */
    (void) do_reap();

    pid = (pid_t) client_data.i;
    if ( kill( pid, SIGINT ) == 0 )
	{
	syslog( LOG_ERR, "killed CGI process %d", pid );
	//DebugOutput("cgi_kill\n");
	/* In case this isn't enough, schedule an uncatchable kill. */
	if ( tmr_create( nowP, cgi_kill2, client_data, 5 * 1000L, 0 ) == (Timer*) 0 )
	    {
	    syslog( LOG_CRIT, "tmr_create(cgi_kill2) failed" );
	    exit( 103 );
	    }
	}
    }
#endif /* CGI_TIMELIMIT */


#ifdef GENERATE_INDEXES

/* qsort comparison routine - declared old-style on purpose, for portability. */
static int
name_compare( a, b )
    char** a;
    char** b;
    {
    return strcmp( *a, *b );
    }


static off_t
ls( httpd_conn* hc )
    {
    DIR* dirp;
    struct dirent* de;
    int namlen;
    static int maxnames = 0;
    int nnames;
    static char* names;
    static char** nameptrs;
    static char* name;
    static int maxname = 0;
    static char* rname;
    static int maxrname = 0;
    static char* encrname;
    static int maxencrname = 0;
    FILE* fp;
    int i, r;
    struct stat sb;
    struct stat lsb;
    char modestr[20];
    char* linkprefix;
    char link[MAXPATHLEN];
    int linklen;
    char* fileclass;
    time_t now;
    char* timestr;
    ClientData client_data;

    dirp = opendir( hc->expnfilename );
    if ( dirp == (DIR*) 0 )
	{
	syslog( LOG_ERR, "opendir %.80s - %m", hc->expnfilename );
	httpd_send_err( hc, 404, err404title, "", err404form, hc->encodedurl );
	return -1;
	}

    if ( hc->method == METHOD_HEAD )
	{
	send_mime( hc, 200, ok200title, "", "", "text/html", -1,
		hc->sb.st_mtime );
	closedir( dirp );
	}
    else if ( hc->method == METHOD_GET )
	{
	r = fork( );
	if ( r < 0 )
	    {
	    syslog( LOG_ERR, "fork - %m" );
	    httpd_send_err( hc, 500, err500title, "", err500form,
			hc->encodedurl );
	    closedir( dirp );
	    return -1;
	    }
	if ( r == 0 )
	    {
	    /* Child process. */
	    unlisten( hc->hs );

	    send_mime( hc, 200, ok200title, "", "", "text/html", -1,
			hc->sb.st_mtime );
	    /* then set blocking mode */
	    /* and write response */
	    httpd_write_blk_response( hc );

#ifdef CGI_NICE
	    /* Set priority. */
	    (void) nice( CGI_NICE );
#endif /* CGI_NICE */

	    /* Open a stdio stream so that we can use fprintf, which is more
	    ** efficient than a bunch of separate write()s.  We don't have
	    ** to worry about double closes or file descriptor leaks cause
	    ** we're in a subprocess.
	    */
	    fp = fdopen( hc->conn_fd, "w" );
	    if ( fp == (FILE*) 0 )
		{
		syslog( LOG_ERR, "fdopen - %m" );
		httpd_send_err(
		    hc, 500, err500title, "", err500form, hc->encodedurl );
		httpd_write_blk_response( hc );
		closedir( dirp );
		exit( 104 );
		}

	    (void) fprintf( fp, "\
<HTML><HEAD><TITLE>Index of %.80s</TITLE></HEAD>\n\
<BODY BGCOLOR=\"#99cc99\">\n\
<H2>Index of %.80s</H2>\n\
<PRE>\n\
mode  links  bytes  last-changed  name\n\
<HR>",
		hc->encodedurl, hc->encodedurl );

	    /* Read in names. */
	    nnames = 0;
	    while ( ( de = readdir( dirp ) ) != 0 )     /* dirent or direct */
		{
		if ( nnames >= maxnames )
		    {
		    if ( maxnames == 0 )
			{
			maxnames = 100;
			names = NEW( char, maxnames * MAXPATHLEN );
			nameptrs = NEW( char*, maxnames );
			}
		    else
			{
			maxnames *= 2;
			names = RENEW( names, char, maxnames * MAXPATHLEN );
			nameptrs = RENEW( nameptrs, char*, maxnames );
			}
		    if ( names == (char*) 0 || nameptrs == (char**) 0 )
			{
			syslog( LOG_ERR, "out of memory reallocating directory names" );
			exit( 105 );
			}
		    for ( i = 0; i < maxnames; ++i )
			nameptrs[i] = &names[i * MAXPATHLEN];
		    }
		namlen = NAMLEN(de);
		(void) strncpy( nameptrs[nnames], de->d_name, namlen );
		nameptrs[nnames][namlen] = '\0';
		++nnames;
		}
	    closedir( dirp );

	    /* Sort the names. */
	    qsort( nameptrs, nnames, sizeof(*nameptrs), name_compare );

	    /* Generate output. */
	    for ( i = 0; i < nnames; ++i )
		{
		httpd_realloc_str(
		    &name, &maxname,
		    strlen( hc->expnfilename ) + 1 + strlen( nameptrs[i] ) );
		httpd_realloc_str(
		    &rname, &maxrname,
		    strlen( hc->origfilename ) + 1 + strlen( nameptrs[i] ) );
		if ( hc->expnfilename[0] == '\0' ||
		     strcmp( hc->expnfilename, "." ) == 0 )
		    {
		    (void) strcpy( name, nameptrs[i] );
		    (void) strcpy( rname, nameptrs[i] );
		    }
		else
		    {
		    (void) my_snprintf( name, maxname,
			"%s/%s", hc->expnfilename, nameptrs[i] );
		    if ( strcmp( hc->origfilename, "." ) == 0 )
			(void) my_snprintf( rname, maxrname,
			    "%s", nameptrs[i] );
		    else
			(void) my_snprintf( rname, maxrname,
			    "%s%s", hc->origfilename, nameptrs[i] );
		    }
		httpd_realloc_str(
		    &encrname, &maxencrname, 3 * strlen( rname ) + 1 );
		strencode( encrname, maxencrname, rname );

		if ( stat( name, &sb ) < 0 || lstat( name, &lsb ) < 0 )
		    continue;

		linkprefix = "";
		link[0] = '\0';
		/* Break down mode word.  First the file type. */
		switch ( lsb.st_mode & S_IFMT )
		    {
		    case S_IFIFO:  modestr[0] = 'p'; break;
		    case S_IFCHR:  modestr[0] = 'c'; break;
		    case S_IFDIR:  modestr[0] = 'd'; break;
		    case S_IFBLK:  modestr[0] = 'b'; break;
		    case S_IFREG:  modestr[0] = '-'; break;
		    case S_IFSOCK: modestr[0] = 's'; break;
		    case S_IFLNK:  modestr[0] = 'l';
		    linklen = readlink( name, link, sizeof(link) );
		    if ( linklen != -1 )
			{
			link[linklen] = '\0';
			linkprefix = " -> ";
			}
		    break;
		    default:       modestr[0] = '?'; break;
		    }
		/* Now the world permissions.  Owner and group permissions
		** are not of interest to web clients.
		*/
		modestr[1] = ( lsb.st_mode & S_IRACC ) ? 'r' : '-';
		modestr[2] = ( lsb.st_mode & S_IWACC ) ? 'w' : '-';
		modestr[3] = ( lsb.st_mode & S_IXACC ) ? 'x' : '-';
		modestr[4] = '\0';

		/* We also leave out the owner and group name, they are
		** also not of interest to web clients.  Plus if we're
		** running under chroot(), they would require a copy
		** of /etc/passwd and /etc/group, which we want to avoid.
		*/

		/* Get time string. */
		now = time( (time_t*) 0 );
		timestr = ctime( &lsb.st_mtime );
		timestr[ 0] = timestr[ 4];
		timestr[ 1] = timestr[ 5];
		timestr[ 2] = timestr[ 6];
		timestr[ 3] = ' ';
		timestr[ 4] = timestr[ 8];
		timestr[ 5] = timestr[ 9];
		timestr[ 6] = ' ';
		if ( now - lsb.st_mtime > 60*60*24*182 )        /* 1/2 year */
		    {
		    timestr[ 7] = ' ';
		    timestr[ 8] = timestr[20];
		    timestr[ 9] = timestr[21];
		    timestr[10] = timestr[22];
		    timestr[11] = timestr[23];
		    }
		else
		    {
		    timestr[ 7] = timestr[11];
		    timestr[ 8] = timestr[12];
		    timestr[ 9] = ':';
		    timestr[10] = timestr[14];
		    timestr[11] = timestr[15];
		    }
		timestr[12] = '\0';

		/* The ls -F file class. */
		switch ( sb.st_mode & S_IFMT )
		    {
		    case S_IFDIR:  fileclass = "/"; break;
		    case S_IFSOCK: fileclass = "="; break;
		    case S_IFLNK:  fileclass = "@"; break;
		    default:
		    fileclass = ( sb.st_mode & S_IXACC ) ? "*" : "";
		    break;
		    }

		/* And print. */
		(void)  fprintf( fp,
		   "%s %3ld  %8ld  %s  <A HREF=\"/%.500s%s\">%.80s</A>%s%s%s\n",
		    modestr, (long) lsb.st_nlink, (long) lsb.st_size, timestr,
		    encrname, S_ISDIR(sb.st_mode) ? "/" : "",
		    nameptrs[i], linkprefix, link, fileclass );
		}

	    (void) fprintf( fp, "</PRE></BODY></HTML>\n" );
	    (void) fclose( fp );
	    exit( 0 );
	    }

	/* Parent process. */

	httpd_clear_response( hc );
	closedir( dirp );
	syslog( LOG_INFO, "spawned indexing process %d for directory '%.200s'", r, hc->expnfilename );
#ifdef CGI_TIMELIMIT
	/* Schedule a kill for the child process, in case it runs too long */
	client_data.i = r;
	if ( tmr_create( (struct timeval*) 0, cgi_kill, client_data, CGI_TIMELIMIT * 1000L, 0 ) == (Timer*) 0 )
	    {
	    syslog( LOG_CRIT, "tmr_create(cgi_kill) failed" );
	    exit( 106 );
	    }
#endif /* CGI_TIMELIMIT */
	hc->status = 200;
	hc->bytes_sent = CGI_BYTECOUNT;
	hc->should_linger = 0;
	}
    else
	{
	closedir( dirp );
	httpd_send_err( hc, 501, err501title, "", err501form,
			httpd_method_str( hc->method ) );
	return -1;
	}

    return 0;
    }

#endif /* GENERATE_INDEXES */


static char*
build_env( char* fmt, char* arg )
    {
    char* cp;
    int size;
    static char* buf;
    static int maxbuf = 0;

    size = strlen( fmt ) + strlen( arg );
    if ( size > maxbuf )
	httpd_realloc_str( &buf, &maxbuf, size );
    (void) my_snprintf( buf, maxbuf,
	fmt, arg );
    cp = strdup( buf );
    if ( cp == (char*) 0 )
	{
	syslog( LOG_ERR, "out of memory copying environment variable" );
	exit( 107 );
	}
    return cp;
    }


#ifdef SERVER_NAME_LIST
static char*
hostname_map( char* hostname )
    {
    int len, n;
    static char* list[] = { SERVER_NAME_LIST };

    len = strlen( hostname );
    for ( n = sizeof(list) / sizeof(*list) - 1; n >= 0; --n )
	if ( strncasecmp( hostname, list[n], len ) == 0 )
	    if ( list[n][len] == '/' )  /* check in case of a substring match */
		return &list[n][len + 1];
    return (char*) 0;
    }
#endif /* SERVER_NAME_LIST */


/* Set up environment variables. Be real careful here to avoid
** letting malicious clients overrun a buffer.  We don't have
** to worry about freeing stuff since we're a sub-process.
*/
static char**
make_envp( httpd_conn* hc )
    {
    static char* envp[50];
    int envn;
    char* cp;
    char buf[256];
	//int i;

    envn = 0;
    envp[envn++] = build_env( "PATH=%s", CGI_PATH );
#ifdef CGI_LD_LIBRARY_PATH
    envp[envn++] = build_env( "LD_LIBRARY_PATH=%s", CGI_LD_LIBRARY_PATH );
#endif /* CGI_LD_LIBRARY_PATH */
//#if !defined(NAS) && !defined(DVR)
//#if 1
    envp[envn++] = build_env( "SERVER_SOFTWARE=%s", SERVER_SOFTWARE );
//#endif
    /* If vhosting, use that server-name here. */
    if ( hc->hs->vhost && hc->hostname != (char*) 0 )
	cp = hc->hostname;
    else
	cp = hc->hs->server_hostname;
    if ( cp != (char*) 0 )
	envp[envn++] = build_env( "SERVER_NAME=%s", cp );
    envp[envn++] = "GATEWAY_INTERFACE=CGI/1.1";
    envp[envn++] = build_env("SERVER_PROTOCOL=%s", hc->protocol);
    (void) my_snprintf( buf, sizeof(buf),
	"%d", hc->hs->port );
    envp[envn++] = build_env( "SERVER_PORT=%s", buf );
    envp[envn++] = build_env(
	"REQUEST_METHOD=%s", httpd_method_str( hc->method ) );
    if ( hc->pathinfo[0] != '\0' )
	{
	char* cp2;
	int l;
	envp[envn++] = build_env( "PATH_INFO=/%s", hc->pathinfo );
	l = strlen( hc->hs->cwd ) + strlen( hc->pathinfo ) + 1;
	cp2 = NEW( char, l );
	if ( cp2 != (char*) 0 )
	    {
	    (void) my_snprintf( cp2, l,
		"%s%s", hc->hs->cwd, hc->pathinfo );
	    envp[envn++] = build_env( "PATH_TRANSLATED=%s", cp2 );
	    }
	}
    envp[envn++] = build_env(
	"SCRIPT_NAME=/%s", strcmp( hc->origfilename, "." ) == 0 ?
	"" : hc->origfilename );
    if ( hc->query[0] != '\0')
	envp[envn++] = build_env( "QUERY_STRING=%s", hc->query );
    envp[envn++] = build_env(
	"REMOTE_ADDR=%s", httpd_ntoa( &hc->client_addr ) );
    if ( hc->referer[0] != '\0' )
	envp[envn++] = build_env( "HTTP_REFERER=%s", hc->referer );
    if ( hc->useragent[0] != '\0' )
	envp[envn++] = build_env( "HTTP_USER_AGENT=%s", hc->useragent );
    if ( hc->accept[0] != '\0' )
	envp[envn++] = build_env( "HTTP_ACCEPT=%s", hc->accept );
    if ( hc->accepte[0] != '\0' )
	envp[envn++] = build_env( "HTTP_ACCEPT_ENCODING=%s", hc->accepte );
    if ( hc->acceptl[0] != '\0' )
	envp[envn++] = build_env( "HTTP_ACCEPT_LANGUAGE=%s", hc->acceptl );
    if ( hc->cookie[0] != '\0' )
	envp[envn++] = build_env( "HTTP_COOKIE=%s", hc->cookie );
    if ( hc->contenttype[0] != '\0' )
	envp[envn++] = build_env( "CONTENT_TYPE=%s", hc->contenttype );
    if ( hc->hdrhost[0] != '\0' )
	envp[envn++] = build_env( "HTTP_HOST=%s", hc->hdrhost );
    if ( hc->contentlength != -1 )
	{
	(void) my_snprintf( buf, sizeof(buf),
	    "%ld", (long) hc->contentlength );
	envp[envn++] = build_env( "CONTENT_LENGTH=%s", buf );
	}
    if ( hc->remoteuser[0] != '\0' )
	envp[envn++] = build_env( "REMOTE_USER=%s", hc->remoteuser );
#ifdef DVR
	if ( hc->authorization[0] != '\0' )
		envp[envn++] = build_env( "REMOTE_AUTH=%s", hc->authorization );
#endif
    if ( hc->authorization[0] != '\0' )
	envp[envn++] = build_env( "AUTH_TYPE=%s", "Basic" );
	/* We only support Basic auth at the moment. */
    if ( getenv( "TZ" ) != (char*) 0 )
	envp[envn++] = build_env( "TZ=%s", getenv( "TZ" ) );
    envp[envn++] = build_env( "CGI_PATTERN=%s", hc->hs->cgi_pattern );

    envp[envn] = (char*) 0;

	/* // Ralph Added
	for (i = 0; i < envn; i++)
		if (envp[i])
			DebugOutput("[make_envp] envp[%d]=%s\n", i, envp[i]);
	*/
    return envp;
    }


/* Set up argument vector.  Again, we don't have to worry about freeing stuff
** since we're a sub-process.  This gets done after make_envp() because we
** scribble on hc->query.
*/
static char**
make_argp( httpd_conn* hc )
    {
    char** argp;
    int argn;
    char* cp1;
    char* cp2;
	//int i;

    /* By allocating an arg slot for every character in the query, plus
    ** one for the filename and one for the NULL, we are guaranteed to
    ** have enough.  We could actually use strlen/2.
    */
    argp = NEW( char*, strlen( hc->query ) + 2 );
    if ( argp == (char**) 0 )
	return (char**) 0;

    argp[0] = strrchr( hc->expnfilename, '/' );
    if ( argp[0] != (char*) 0 )
	++argp[0];
    else
	argp[0] = hc->expnfilename;

    argn = 1;
    /* According to the CGI spec at http://hoohoo.ncsa.uiuc.edu/cgi/cl.html,
    ** "The server should search the query information for a non-encoded =
    ** character to determine if the command line is to be used, if it finds
    ** one, the command line is not to be used."
    */
    if ( strchr( hc->query, '=' ) == (char*) 0 )
	{
	for ( cp1 = cp2 = hc->query; *cp2 != '\0'; ++cp2 )
	    {
	    if ( *cp2 == '+' )
		{
		*cp2 = '\0';
		strdecode( cp1, cp1 );
		argp[argn++] = cp1;
		cp1 = cp2 + 1;
		}
	    }
	if ( cp2 != cp1 )
	    {
	    strdecode( cp1, cp1 );
	    argp[argn++] = cp1;
	    }
	}

    argp[argn] = (char*) 0;

	/* // Ralph Added
	for (i = 0; i < argn; i++)
		if (argp[i])
			DebugOutput("[make_argp] argp[%d]=%s\n", i, argp[i]);
	*/
    return argp;
    }


/* This routine is used only for POST requests.  It reads the data
** from the request and sends it to the child process.  The only reason
** we need to do it this way instead of just letting the child read
** directly is that we have already read part of the data into our
** buffer.
*/
static void
cgi_interpose_input( httpd_conn* hc, int wfd )
    {
    int c, r;
    char buf[1024];
	//char* p = 0;
	//int i = 0;

	//DebugOutput("[cgi_interpose_input] ...\n");
    /* restore block-mode (if it was lost) */
    httpd_clear_ndelay( hc->conn_fd );

    c = hc->read_idx - hc->checked_idx;
    if ( c > 0 )
	{
		//DebugOutput("[cgi_interpose_input] wfd=%d, c=%d, hc->checked_idx=%d\n", wfd, c, hc->checked_idx);
		if ( write( wfd, &(hc->read_buf[hc->checked_idx]), c ) != c )
			return;

		/* // Ralph Added
		p = &(hc->read_buf[hc->checked_idx]);
		for (i = 0; i < c; i++)
			DebugOutput("%c", p[i]);
		DebugOutput("\n");
		*/
	}
	//DebugOutput("[cgi_interpose_input] c=%d, hc->contentlength=%d\n", c, hc->contentlength);
    while ( c < hc->contentlength )
	{
	r = read( hc->conn_fd, buf, MIN( sizeof(buf), hc->contentlength - c ) );
	//DebugOutput("[cgi_interpose_input] hc->conn_fd=%d, buf=%s\n", hc->conn_fd, buf);
	if ( r == 0 )
	    return;
	if ( r < 0 )
	    {
	    if ( errno == EAGAIN )
		sleep( 1 );
	    else
		return;
	    }
	else
	    {
		//if (buf)
		//	DebugOutput("[cgi_interpose_input] buf=%s\n", buf);

	    if ( write( wfd, buf, r ) != r )
		return;
	    c += r;
	    }
	}
#ifdef NAS
	/* add by Kent 2001/12/03			*/
	/* prevent the name of object too long...	*/
      	if (strstr(hc->encodedurl, "cgi-bin/filemanager/filemanager.cgi") ||
      		strstr(hc->encodedurl, "cgi-bin/filemanager/function.cgi"))
#endif
    post_post_garbage_hack( hc );
    }


/* Special hack to deal with broken browsers that send a LF or CRLF
** after POST data, causing TCP resets - we just read and discard up
** to 2 bytes.  Unfortunately this doesn't fix the problem for CGIs
** which avoid the interposer process due to their POST data being
** short.  Creating an interposer process for all POST CGIs is
** unacceptably expensive.  The eventual fix will come when interposing
** gets integrated into the main loop as a tasklet instead of a process.
*/
static void
post_post_garbage_hack( httpd_conn* hc )
    {
    char buf[2];

    httpd_set_ndelay( hc->conn_fd );
    (void) read( hc->conn_fd, buf, sizeof(buf) );
    }


/* This routine is used for parsed-header CGIs.  The idea here is that the
** CGI can return special headers such as "Status:" and "Location:" which
** change the return status of the response.  Since the return status has to
** be the very first line written out, we have to accumulate all the headers
** and check for the special ones before writing the status.  Then we write
** out the saved headers and proceed to echo the rest of the response.
*/
static void
cgi_interpose_output( httpd_conn* hc, int rfd )
    {
    int r;
    char buf[1024];
    int headers_size, headers_len;
    char* headers;
    char* br;
    int status;
    char* title;
    char* cp;
    //pid_t pid = getppid();

    //DebugOutput("cgi_interpose_output\n", buf);

    /* restore block-mode (if it was lost) */
    httpd_clear_ndelay( hc->conn_fd );

    /* we start with a normal condition */
    status = 200;

    /* Slurp in all headers. */
    headers_size = 0;
    headers = (char *) 0;
    httpd_realloc_str( &headers, &headers_size, 512 );
    headers_len = 0;

	//DebugOutput("[cgi_interpose_output] headers=%s\n", headers);

    for (;;)
	{
	r = read( rfd, buf, sizeof(buf) );
	if ( r <= 0 )
	    {
	    br = &(headers[headers_len]);
	    break;
	    }
	httpd_realloc_str( &headers, &headers_size, headers_len + r );
	(void) memcpy( &(headers[headers_len]), buf, r );
	headers_len += r;
	headers[headers_len] = '\0';
	if ( ( br = strstr( headers, "\r\n\r\n" ) ) != (char*) 0 ||
	     ( br = strstr( headers, "\n\n" ) ) != (char*) 0 )
	    /* OK, headers have just been read */
	    break;
#if	defined(CGI_MAX_HEADERS_LENGTH) && (CGI_MAX_HEADERS_LENGTH > 0)
	/* no, check for headers too long */
	if (headers_len > CGI_MAX_HEADERS_LENGTH)
	    /* yeah, we don't want to go out of memory because of */
	    /* a buggy CGI which is writing huge amounts of crappy data */
	    /* instead of HTTP headers */
	    /* NOTE: we let a small CGI response without HTTP headers */
	    break;
#endif
	}

    /* if there were no headers, bail */
    if ( headers_len < 1 )
	return;

#if	defined(CGI_MAX_HEADERS_LENGTH) && (CGI_MAX_HEADERS_LENGTH > 0)
    if ( headers_len > CGI_MAX_HEADERS_LENGTH )
	{
	/* bad HTTP headers, write an error response and stop */
	/* so buggy CGI is hit by a signal (SIGPIPE) */

	status = 500;
	(void) my_snprintf( buf, sizeof(buf), "HTTP/1.0 %d %s\r\n",
		status, err500title );
	(void) write( hc->conn_fd, buf, strlen( buf ) );
	httpd_close_conn_wr( hc );
	syslog( LOG_ERR,
	"CGI '%.80s': HTTP headers too long %d > %d (missing empty line)",
		hc->expnfilename, headers_len, CGI_MAX_HEADERS_LENGTH );
	return;
	}
#endif

    /* Figure out the status.  Look for a Status: or Location: header; */
    /* else if there's and HTTP header line, get it from there; else */
    /* use default status (200 or 500) */
    if ( strncmp( headers, "HTTP/", 5 ) == 0)
	{
	cp = headers;
	cp += strcspn( cp, " \t" );
	status = atoi( cp );
	}
    if ( ( cp = strstr( headers, "Status:" ) ) != (char*) 0 &&
	   cp < br &&
	 ( cp == headers || *(cp-1) == '\n' ) )
	{
	cp += 7;
	cp += strspn( cp, " \t" );
	status = atoi( cp );
	}
    if ( ( cp = strstr( headers, "Location:" ) ) != (char*) 0 &&
	   cp < br &&
	 ( cp == headers || *(cp-1) == '\n' ) )
	status = 302;

    /* Write the status line. */
    switch ( status )
	{
	case 200: title = ok200title; break;
	case 302: title = err302title; break;
	case 304: title = err304title; break;
	case 400: title = httpd_err400title; break;
#ifdef AUTH_FILE
	case 401: title = err401title; break;
#endif /* AUTH_FILE */
	case 403: title = err403title; break;
	case 404: title = err404title; break;
	case 408: title = httpd_err408title; break;
	case 500: title = err500title; break;
	case 501: title = err501title; break;
	case 503: title = httpd_err503title; break;
#ifdef NAS
	case 999: title = httpd_err999title; break;
#endif
	default: title = "Something"; break;
	}
    (void) my_snprintf( buf, sizeof(buf), "HTTP/1.0 %d %s\r\n", status, title );
    //DebugOutput("[cgi_interpose_output] buf=%s\n", buf);
    (void) write( hc->conn_fd, buf, strlen( buf ) );
#if defined(NAS) || defined(DVR)
	if (strlen(hc->session) > 0) {
		(void) my_snprintf(buf, sizeof(buf), "Set-Cookie: +ID=%s; path=/; expires=Thursday, 31-Jan-1980 23:59:59 GMT\r\n", hc->session);
    		(void) write(hc->conn_fd, buf, strlen(buf));
		//Conf_Remove_Field(SESSION_ID_LOG_FILE, NULL, hc->session);
	}
	if (strlen(hc->remoteuser) > 0) {
		sprintf(buf, "%s=TEST", hc->remoteuser);
		if (strstr(hc->cookie, buf)) {
			(void) my_snprintf(buf, sizeof(buf), "Set-Cookie: %s=TEST; path=/; expires=Thursday, 31-Jan-1980 23:59:59 GMT\r\n", hc->remoteuser);
    			(void) write(hc->conn_fd, buf, strlen(buf));
		}
	}
#endif
    //DebugOutput("[cgi_interpose_output] headers=%s\n", headers);
    /* Write the saved headers. */
    (void) write( hc->conn_fd, headers, headers_len );

	//DebugOutput("%s\n", buf);
	//DebugOutput("%s\n", headers);
    	free((void*) headers);

    /* we don't care about freeing headers because this is a subprocess */
    /* and allocated memory is automatically fred after exit() */

    /* Echo the rest of the output. */
    for (;;)
	{
	//DebugOutput("Echo the rest of the output\n");
	r = read( rfd, buf, sizeof(buf) );
	if ( r <= 0 )
	    break;
	//DebugOutput("[cgi_interpose_output] read: %d\n", r);
	if ( write( hc->conn_fd, buf, r ) != r ) {
#if 0
		//DebugOutput("Try to send signal to parent process: %d\n", (int) pid);
		if (hc->broadcast == 1 && kill(pid, SIGTERM) == 0) {
			while (read(rfd, buf, sizeof(buf)) > 0)
				;
			sleep(3);
		}
#endif
	    break;
	}
	//DebugOutput("write\n");
	}
	//DebugOutput("httpd_close_conn_wr\n");
	//DebugOutput("End process: %d\n", (int) getpid());
    httpd_close_conn_wr( hc );
    }


/* CGI child process. */
static void
cgi_child( httpd_conn* hc )
    {
    int r;
    char** argp;
    char** envp;
    char* binary;
    char* directory;

#if defined(DVR) && defined(STATISTICS)
	add_httpd_user(hc);
#endif
    /* Unset close-on-exec flag for this socket.  This actually shouldn't
    ** be necessary, according to POSIX a dup()'d file descriptor does
    ** *not* inherit the close-on-exec flag, its flag is always clear.
    ** However, Linux messes this up and does copy the flag to the
    ** dup()'d descriptor, so we have to clear it.  This could be
    ** ifdeffed for Linux only.
    */
    (void) fcntl( hc->conn_fd, F_SETFD, 0 );

    /* Close the syslog descriptor so that the CGI program can't
    ** mess with it.  All other open descriptors should be either
    ** the listen socket(s), sockets from accept(), or the file-logging
    ** fd, and all of those are set to close-on-exec, so we don't
    ** have to close anything else.
    */
    closelog();

    /* If the socket happens to be using one of the stdin/stdout/stderr
    ** descriptors, move it to another descriptor so that the dup2 calls
    ** below don't screw things up.  We arbitrarily pick fd 3 - if there
    ** was already something on it, we clobber it, but that doesn't matter
    ** since at this point the only fd of interest is the connection.
    ** All others will be closed on exec.
    */
    if ( hc->conn_fd == STDIN_FILENO ||
         hc->conn_fd == STDOUT_FILENO ||
         hc->conn_fd == STDERR_FILENO )
	{
	int newfd = dup2( hc->conn_fd, STDERR_FILENO + 1 );
	if ( newfd >= 0 )
	    hc->conn_fd = newfd;
	/* If the dup2 fails, shrug.  We'll just take our chances.
	** Shouldn't happen though.
	*/
	}

	//DebugOutput("[cgi_child] make_envp ...\n");
    /* Make the environment vector. */
    envp = make_envp( hc );

    /* Make the argument vector. */
    argp = make_argp( hc );


    /* Set up stdin.  For POSTs we may have to set up a pipe from an
    ** interposer process, depending on if we've read some of the data
    ** into our buffer.
    */
    if ( hc->method == METHOD_POST && hc->read_idx > hc->checked_idx )
	{
	int p[2];

	if ( pipe( p ) < 0 )
	    {
	    syslog( LOG_ERR, "pipe - %m" );
	    httpd_send_err( hc, 500, err500title, "", err500form, hc->encodedurl );
	    httpd_write_blk_response( hc );
	    exit( 108 );
	    }
	r = fork( );
	if ( r < 0 )
	    {
	    syslog( LOG_ERR, "fork - %m" );
	    httpd_send_err( hc, 500, err500title, "", err500form, hc->encodedurl );
	    httpd_write_blk_response( hc );
	    exit( 109 );
	    }
	if ( r == 0 )
	    {
		//DebugOutput("[cgi_child] fork1 Ralph cgi_interpose_input ... p[0]=%d, p[1]=%d\n", p[0], p[1]);
	    /* Interposer process. */
	    (void) close( p[0] );
	    cgi_interpose_input( hc, p[1] );
	    exit( 0 );
	    }
	/* Need to schedule a kill for process r; but in the main process! */
	(void) close( p[1] );
	if ( p[0] != STDIN_FILENO )
	    {
	    (void) dup2( p[0], STDIN_FILENO );
	    (void) close( p[0] );
	    }
	}
    else
	{
	/* Otherwise, the request socket is stdin. */
	if ( hc->conn_fd != STDIN_FILENO )
	    (void) dup2( hc->conn_fd, STDIN_FILENO );
	}

    /* Set up stdout/stderr.  If we're doing CGI header parsing,
    ** we need an output interposer too.
    */
    if ( strncmp( argp[0], "nph-", 4 ) != 0 && hc->mime_flag )
	{
	int p[2];

	if ( pipe( p ) < 0 )
	    {
	    syslog( LOG_ERR, "pipe - %m" );
	    httpd_send_err( hc, 500, err500title, "", err500form, hc->encodedurl );
	    httpd_write_blk_response( hc );
	    exit( 110 );
	    }
	r = fork( );
	if ( r < 0 )
	    {
	    syslog( LOG_ERR, "fork - %m" );
	    httpd_send_err( hc, 500, err500title, "", err500form, hc->encodedurl );
	    httpd_write_blk_response( hc );
	    exit( 111 );
	    }
	if ( r == 0 )
	    {
	    /* Interposer process. */
	    (void) close( p[1] );
	    //DebugOutput("[cgi_child] fork2 cgi_interpose_output ...\n");
	    cgi_interpose_output( hc, p[0] );
	    exit( 0 );
	    }
	/* Need to schedule a kill for process r; but in the main process! */
	(void) close( p[0] );
	if ( p[1] != STDOUT_FILENO )
	    (void) dup2( p[1], STDOUT_FILENO );
	if ( p[1] != STDERR_FILENO )
	    (void) dup2( p[1], STDERR_FILENO );
	if ( p[1] != STDOUT_FILENO && p[1] != STDERR_FILENO )
	    (void) close( p[1] );
	}
    else
	{
	/* Otherwise, the request socket is stdout/stderr. */
	if ( hc->conn_fd != STDOUT_FILENO )
	    (void) dup2( hc->conn_fd, STDOUT_FILENO );
	if ( hc->conn_fd != STDERR_FILENO )
	    (void) dup2( hc->conn_fd, STDERR_FILENO );
	}

    /* At this point we would like to set close-on-exec again for hc->conn_fd
    ** (see previous comments on Linux's broken behavior re: close-on-exec
    ** and dup.)  Unfortunately there seems to be another Linux problem, or
    ** perhaps a different aspect of the same problem - if we do this
    ** close-on-exec in Linux, the socket stays open but stderr gets
    ** closed - the last fd duped from the socket.  What a mess.  So we'll
    ** just leave the socket as is, which under other OSs means an extra
    ** file descriptor gets passed to the child process.  Since the child
    ** probably already has that file open via stdin stdout and/or stderr,
    ** this is not a problem.
    */
    /* (void) fcntl( hc->conn_fd, F_SETFD, 1 ); */

#ifdef CGI_NICE
    /* Set priority. */
    (void) nice( CGI_NICE );
#endif /* CGI_NICE */

	//DebugOutput("1\n");

    /* Split the program into directory and binary, so we can chdir()
    ** to the program's own directory.  This isn't in the CGI 1.1
    ** spec, but it's what other HTTP servers do.
    */
    directory = strdup( hc->expnfilename );
    if ( directory == (char*) 0 )
	binary = hc->expnfilename;      /* ignore errors */
    else
	{
	binary = strrchr( directory, '/' );
	if ( binary == (char*) 0 )
	    binary = hc->expnfilename;
	else
	    {
	    *binary++ = '\0';
	    (void) chdir( directory );  /* ignore errors */
	    }
	}

	//DebugOutput("2\n");

    /* Default behavior for SIGPIPE. */
    (void) signal( SIGPIPE, SIG_DFL );

	DebugOutput("[cgi_child] execve: %s\n", binary);

    /* Run the program. */
    (void) execve( binary, argp, envp );

    	//DebugOutput("Failed\n", binary);

    /* Something went wrong. */
    syslog( LOG_ERR, "execve '%.80s' - %m", hc->expnfilename );
    httpd_send_err( hc, 500, err500title, "", err500form, hc->encodedurl );
    httpd_write_blk_response( hc );
    exit( 112 );
    }


static off_t
cgi( httpd_conn* hc )
    {
    int r;
    ClientData client_data;
//#if defined(NAS) || defined(DVR)
#ifdef NAS
	char tmpchar;

	//modify by Kent Tsai 2001/02/06
	//check if it is under the cgi-bin folder
	//if not, show the error message
	tmpchar = hc->encodedurl[8];
        hc->encodedurl[8] = 0x0;
        if (!strcmp(hc->encodedurl, "/cgi-bin"))
                hc->encodedurl[8] = tmpchar;
	else {
        	hc->encodedurl[8] = tmpchar;
		if (hc->referer[0] != 0) { //user want to exec it
			httpd_send_err(hc, 501, httpd_err504title, "", httpd_err504form, hc->encodedurl);
			return -1;
		}
	}
#endif
    /*
    **  We are not going to leave the socket open after a CGI... too hard
    */
    hc->do_keep_alive = 0;

    if ( hc->method == METHOD_GET || hc->method == METHOD_POST )
	{
	r = fork( );
	if ( r < 0 )
	    {
	    syslog( LOG_ERR, "fork - %m" );
	    httpd_send_err( hc, 500, err500title, "", err500form, hc->encodedurl );
	    return -1;
	    }
	if ( r == 0 )
	    {
	    unlisten( hc->hs );
	    httpd_clear_ndelay( hc->conn_fd );
	    cgi_child( hc );
	    //DebugOutput("End of cgi_child\n");
	    }
	//DebugOutput("spawned CGI process %d\n", r);
	/* Parent process. */
	httpd_clear_response( hc );
	syslog( LOG_INFO, "spawned CGI process %d for file '%.200s'", r, hc->expnfilename );
#ifdef CGI_TIMELIMIT
#ifdef NAS
	//======================================
	//	add by Kent Tsai 2001/03/07
	//	if we want to upload file
	//	don't set cgi time limit
	//=======================================
	if (!strstr(hc->contenttype, "multipart/form-data") && !strstr(hc->encodedurl, "security.cgi")) {
#endif
#ifdef DVR
	if (!strstr(hc->contenttype, "multipart/form-data") && !strstr(hc->encodedurl, "monitor_ch") && !strstr(hc->encodedurl, "cfg_sys_tool.cgi")) {
#endif
	/* Schedule a kill for the child process, in case it runs too long */
	client_data.i = r;
	if ( tmr_create( (struct timeval*) 0, cgi_kill, client_data, CGI_TIMELIMIT * 1000L, 0 ) == (Timer*) 0 )
	    {
	    syslog( LOG_CRIT, "tmr_create(cgi_kill child) failed" );
	    exit( 113 );
	    }
#if defined(NAS) || defined(DVR)
	}
#endif
#endif /* CGI_TIMELIMIT */
	hc->status = 200;
	hc->bytes_sent = CGI_BYTECOUNT;
	hc->should_linger = 0;
	}
    else
	{
	httpd_send_err(
	    hc, 501, err501title, "", err501form, httpd_method_str( hc->method ) );
	return -1;
	}

    return 0;
    }

#ifdef DVR

#define STRID_LEN 64

static int master_cgi_reject(httpd_conn* hc)
{
	int i, c;
	char strid[STRID_LEN] = {0};
	char* p;

	if (!hc)
		return 1;
	else
	{
		if (strstr(hc->encodedurl, "cgi-bin/sacc.cgi") == 0)
			return 0;
		//DebugOutput("[master_cgi_reject]...\n");
	}

	p = &(hc->read_buf[hc->checked_idx]);
	//DebugOutput("[master_cgi_reject] p=%x\n", p);
    c = hc->read_idx - hc->checked_idx;
    if ( c > 0 )
	{
		int id = 0;
		//DebugOutput("[master_cgi_reject] c=%d\n", c);

		for (i = 0; i < c; i++)
		{
			//DebugOutput("[master_cgi_reject] i=%d ", i);
			if (i >= STRID_LEN)
				return 1;

			if (p[i] == ' ')
			{
				strid[i] = 0;
				break;
			}
			else
				strid[i] = p[i];
		}

		//DebugOutput("\n[master_cgi_reject] strid=%s\n", strid);
		id = atoi(strid);
		switch (id)
		{
		case HRPC_ID_VS_GET_SYSTEM_INFO:
		case HRPC_ID_VS_PING:
		case HRPC_ID_VS_GET_HARDWARE_RESET:
		case HRPC_ID_VS_IS_SUPPORT_LOCAL_STORAGE:
		case HRPC_ID_VS_DISK_STATUS_GET:
		case HRPC_ID_VS_DISK_FORMAT_START:
		case HRPC_ID_VS_DISK_FORMAT_STOP:
		case HRPC_ID_VS_DISK_CHECK_START:
		case HRPC_ID_VS_DISK_CHECK_STOP:
		case HRPC_ID_VS_DISK_AVAILABLE_SPACE_GET:
		case HRPC_ID_VS_GET_USER_ADVANCE_INFO:
			DebugOutput("[master_cgi_reject] id=%d pass\n", id);
			return 0;
		default:
			DebugOutput("[master_cgi_reject] id=%d reject\n", id);
			return 1;
		}
	}
	
	return 0;
}

#define  IS_LOCAL_DISK_BUSY(ds) (ds == INITIALIZING || ds == FORMATTING || ds == DISK_CHECKING || ds == FS_CHECKING_DISK || ds == FS_CHECKING_FORMAT)

static int local_disk_busy_reject(httpd_conn* hc)
{
	void *handle = 0;
	typedef int (*LOCAL_STORAGE)();
	LOCAL_STORAGE disk_status_get;
	DISK_STATUS ds;
	char *get_error;
	int ret = 1;
	int istr = 0;

	//add by Ralph Nieh 2003/07/17
	handle = dlopen("/usr/lib/libdvrutil.so", RTLD_LAZY);
	if (!handle) 
	{
		//printf("dlopen fail\n");
		//reply(530, "dlopen access denied.");
		return LOCAL_DISK_MODULE_ERROR;
	}

	disk_status_get = (LOCAL_STORAGE)dlsym(handle, "cf_disk_status_get__Fv");
	if ((get_error = dlerror()) != NULL)
	{
		//reply(530, "dlsym disk_status_get access denied. (%s)", get_error);
		//DebugOutput("[local_disk_busy_reject] dlsym error\n");
		dlclose(handle);
		ret = LOCAL_DISK_MODULE_ERROR;
		goto exitpoint;
	}

	ds = (*disk_status_get)();
	DebugOutput("[local_disk_busy_reject] disk_status_get OK (status=%d)\n", ds);
	//when disk status is any INITIALIZING/FORMATTING/DISK_CHECKING/FS_CHECKING_DISK status..
	//if it is not found these stings as follows, 
	//reject it immediately.
	if ( IS_LOCAL_DISK_BUSY(ds) && //Not found these strings
		 strstr(hc->encodedurl, "cgi-bin/sacc.cgi") == 0 && 
		 strstr(hc->encodedurl, "cgi-bin/ch_info_update.cgi") == 0 && 
		 strstr(hc->encodedurl, "cgi-bin/logout.cgi") == 0 &&
		 //strstr(hc->encodedurl, "cgi-bin/cfg_sys_get.cgi") == 0 
		 strstr(hc->encodedurl, "cgi-bin/cfg_sys_tool.cgi?Item=0") == 0 &&
		 strstr(hc->encodedurl, "cgi-bin/cfg_sys_tool.cgi?Item=1") == 0 &&
		 strstr(hc->encodedurl, "cgi-bin/cfg_sys_tool.cgi?Item=2") == 0 &&
		 strstr(hc->encodedurl, "cgi-bin/cfg_sys_tool.cgi?Item=3") == 0 &&
		 strstr(hc->encodedurl, "cgi-bin/cfg_sys_tool.cgi?Item=4") == 0 &&
		 //strstr(hc->encodedurl, "cgi-bin/data/tool.js") == 0 &&
		 strstr(hc->encodedurl, "cgi-bin/data/") == 0 &&
		 strstr(hc->encodedurl, "cgi-bin/cfg_main.cgi") == 0 &&
		 strstr(hc->encodedurl, ".jpg") == 0 &&
		 strstr(hc->encodedurl, ".gif") == 0 &&
		 strstr(hc->encodedurl, "cgi-bin/monitor.cgi") == 0 && 
		 strstr(hc->encodedurl, "enter.html") == 0 && 
		 strstr(hc->encodedurl, "cgi-bin/data/images/") == 0 &&
		 strstr(hc->encodedurl, "cgi-bin/l2_format_disk.cgi") == 0 &&
		 strstr(hc->encodedurl, "cgi-bin/cfg_sys_debug.cgi") == 0 &&
		 ! ( (strlen(hc->encodedurl) == 1) && hc->encodedurl[0] == '/' )) 
	{
	  	 ret = LOCAL_DISK_BUSY_REJECT;

		 DebugOutput("[local_disk_busy_reject] LOCAL_DISK_BUSY_REJECT OK (ds=%d, hc->encodedurl=%s, is_not_img=%d, istr=%x)\n", 
			 ds, hc->encodedurl, (istr = strstr(hc->encodedurl, "cgi-bin/data/images/")) == 0, istr);
		 
		 goto exitpoint;
	}

	if (IS_LOCAL_DISK_BUSY(ds) && master_cgi_reject(hc))
	{
	  	 ret = LOCAL_DISK_MASTER_CGI_REJECT;	
		 goto exitpoint;
	}

	if (IS_LOCAL_DISK_BUSY(ds) && strstr(hc->encodedurl, "cgi-bin/monitor.cgi") != 0)
	{
		DebugOutput("[local_disk_busy_reject] %s\n", hc->encodedurl);
		sleep(10);

		ds = (*disk_status_get)();
		if (IS_LOCAL_DISK_BUSY(ds))
		{
			ret = LOCAL_DISK_MONITOR_REDIRECT;//LOCAL_DISK_BUSY_REJECT;//;

			DebugOutput("[local_disk_busy_reject] LOCAL_DISK_BUSY_REJECT (status=%d, ret=%d)\n", ds, ret);

			goto exitpoint;
		}
	}
	/* // Ralph Added
	if (hc->read_buf)
		DebugOutput("[local_disk_busy_pass] read_buf=%s\n", hc->read_buf);

	if (hc->encodedurl)
		DebugOutput("[local_disk_busy_pass] encodedurl=%s\n", hc->encodedurl);

	if (hc->decodedurl)
		DebugOutput("[local_disk_busy_pass] decodedurl=%s\n", hc->decodedurl);

	if (hc->protocol)
		DebugOutput("[local_disk_busy_pass] protocol=%s\n", hc->protocol);

	if (hc->origfilename)
		DebugOutput("[local_disk_busy_pass] origfilename=%s\n", hc->origfilename);

	if (hc->expnfilename)
		DebugOutput("[local_disk_busy_pass] expnfilename=%s\n", hc->expnfilename);

	if (hc->encodings)
		DebugOutput("[local_disk_busy_pass] encodings=%s\n", hc->encodings);

	if (hc->pathinfo)
		DebugOutput("[local_disk_busy_pass] pathinfo=%s\n", hc->pathinfo);

	if (hc->query)
		DebugOutput("[local_disk_busy_pass] query=%s\n", hc->query);

	if (hc->referer)
		DebugOutput("[local_disk_busy_pass] referer=%s\n", hc->referer);

	if (hc->useragent)
		DebugOutput("[local_disk_busy_pass] useragent=%s\n", hc->useragent);

	if (hc->accept)
		DebugOutput("[local_disk_busy_pass] accept=%s\n", hc->accept);

	if (hc->accepte)
		DebugOutput("[local_disk_busy_pass] accepte=%s\n", hc->accepte);

	if (hc->acceptl)
		DebugOutput("[local_disk_busy_pass] acceptl=%s\n", hc->acceptl);

	if (hc->cookie)
		DebugOutput("[local_disk_busy_pass] cookie=%s\n", hc->cookie);

	if (hc->expnfilename)
		DebugOutput("[local_disk_busy_pass] expnfilename=%s\n", hc->expnfilename);

	if (hc->contenttype)
		DebugOutput("[local_disk_busy_pass] contenttype=%s\n", hc->contenttype);

	if (hc->reqhost)
		DebugOutput("[local_disk_busy_pass] reqhost=%s\n", hc->reqhost);

	if (hc->hdrhost)
		DebugOutput("[local_disk_busy_pass] hdrhost=%s\n", hc->hdrhost);

	if (hc->hostdir)
		DebugOutput("[local_disk_busy_pass] hostdir=%s\n", hc->hostdir);

	if (hc->authorization)
		DebugOutput("[local_disk_busy_pass] authorization=%s\n", hc->authorization);

	if (hc->response)
		DebugOutput("[local_disk_busy_pass] response=%s\n\n", hc->response);

	if (hc->type)
		DebugOutput("[local_disk_busy_pass] type=%s\n", hc->type);

	if (hc->hostname)
		DebugOutput("[local_disk_busy_pass] hostname=%s\n\n", hc->hostname);
	*/
	//DebugOutput("[local_disk_busy_pass] ds=%d, hc->encodedurl=%s\n", ds, hc->encodedurl);
exitpoint:
	if (handle)
		dlclose(handle);
	return ret;
}
#endif

static int
really_start_request( httpd_conn* hc, struct timeval* nowP,
			int numconn, int maxconn )
    {
    static char* indexname;
    static int maxindexname = 0;
    static const char* index_names[] = { INDEX_NAMES };
    int i;
    int isRegOrigFile = 0;
#ifdef AUTH_FILE
    static char* dirname;
    static int maxdirname = 0;
#endif /* AUTH_FILE */
    int expnlen, indxlen;
    char* cp;
    char* pi;

    //DebugOutput("%s\n", hc->expnfilename);

    expnlen = strlen( hc->expnfilename );

    if ( hc->method != METHOD_GET && hc->method != METHOD_HEAD &&
	 hc->method != METHOD_POST )
	{
	httpd_send_err(
	    hc, 501, err501title, "", err501form, httpd_method_str( hc->method ) );
	return -1;
	}
#ifdef DVR
	if (hc->send_jpeg_buffer != 0)
		goto send_jpeg_response;
#endif
    /* Stat the file. */
    if ( stat( hc->expnfilename, &hc->sb ) < 0 )
	{
	httpd_send_err( hc, 500, err500title, "", err500form, hc->encodedurl );
	return -1;
	}

    /* Is it world-readable or world-executable?  We check explicitly instead
    ** of just trying to open it, so that no one ever gets surprised by
    ** a file that's not set world-readable and yet somehow is
    ** readable by the HTTP server and therefore the *whole* world.
    */
    if ( ! ( hc->sb.st_mode & ( S_IRACC | S_IXACC ) ) )
	{
	syslog(
	    LOG_INFO,
	    "%.80s URL \"%.80s\" resolves to a non world-readable file",
	    httpd_ntoa( &hc->client_addr ), hc->encodedurl );
	httpd_send_err(
	    hc, 403, err403title, "",
	    ERROR_FORM( err403form, "The requested URL '%.80s' resolves to a file that is not world-readable.\n" ),
	    hc->encodedurl );
	return -1;
	}

    /* Is it a directory? */
    if ( S_ISDIR(hc->sb.st_mode) )
	{
	/* If there's pathinfo, it's just a non-existent file. */
	if ( hc->pathinfo[0] != '\0' )
	    {
	    httpd_send_err( hc, 404, err404title, "", err404form, hc->encodedurl );
	    return -1;
	    }

	/* Special handling for directory URLs that don't end in a slash.
	** We send back an explicit redirect with the slash, because
	** otherwise many clients can't build relative URLs properly.
	*/
	if ( hc->decodedurl[strlen( hc->decodedurl ) - 1] != '/' )
	    {
	    if ( hc->query[0] != '\0')
		httpd_send_err( hc, 400, httpd_err400title, "",
			httpd_err400form, "" );
	    else
		send_dirredirect( hc );
	    return -1;
	    }

	/* Check for an index file. */
	for ( i = 0; i < sizeof(index_names) / sizeof(char*); ++i )
	    {
	    httpd_realloc_str(
		&indexname, &maxindexname,
		expnlen + 1 + strlen( index_names[i] ) );
	    (void) strcpy( indexname, hc->expnfilename );
	    indxlen = strlen( indexname );
	    if ( indxlen == 0 || indexname[indxlen - 1] != '/' )
		(void) strcat( indexname, "/" );
	    if ( strcmp( indexname, "./" ) == 0 )
		indexname[0] = '\0';
	    (void) strcat( indexname, index_names[i] );
	    if ( stat( indexname, &hc->sb ) >= 0 )
		goto got_one;
	    }

	/* Nope, no index file, so it's an actual directory request. */
#ifdef GENERATE_INDEXES
	/* Directories must be readable for indexing. */
	if ( ! ( hc->sb.st_mode & S_IRACC ) )
	    {
	    syslog(
		LOG_INFO,
		"%.80s URL \"%.80s\" tried to index a directory with indexing disabled",
		httpd_ntoa( &hc->client_addr ), hc->encodedurl );
	    httpd_send_err(
		hc, 403, err403title, "",
		ERROR_FORM( err403form, "The requested URL '%.80s' resolves to a directory that has indexing disabled.\n" ),
		hc->encodedurl );
	    return -1;
	    }
#ifdef AUTH_FILE
	/* Check authorization for this directory. */
	if ( (i = auth_check( hc, hc->expnfilename )) <= -1 )
		return i;
	    //return -1;
#endif /* AUTH_FILE */
	/* Referer check. */
	if ( ! check_referer( hc ) )
	    return -1;
	/* Ok, generate an index. */
	return ls( hc );
#else /* GENERATE_INDEXES */
	syslog(
	    LOG_INFO, "%.80s URL \"%.80s\" tried to index a directory",
	    httpd_ntoa( &hc->client_addr ), hc->encodedurl );
	httpd_send_err(
	    hc, 403, err403title, "",
	    ERROR_FORM( err403form, "The requested URL '%.80s' is a directory, and directory indexing is disabled on this server.\n" ),
	    hc->encodedurl );
	return -1;
#endif /* GENERATE_INDEXES */

	got_one: ;
	/* Got an index file.  Expand symlinks again.  More pathinfo means
	** something went wrong.
	*/
	cp = expand_symlinks( indexname, &pi, hc->hs->no_symlink, hc->tildemapped );
	if ( cp == (char*) 0 || pi[0] != '\0' )
	    {
	    httpd_send_err( hc, 500, err500title, "", err500form, hc->encodedurl );
	    return -1;
	    }
	expnlen = strlen( cp );
	httpd_realloc_str( &hc->expnfilename, &hc->maxexpnfilename, expnlen );
	(void) strcpy( hc->expnfilename, cp );

	/* Now, is the index version a regular file ? */
	if ( !S_ISREG(hc->sb.st_mode) )
	    {
	    syslog(
		LOG_INFO,
		"%.80s URL \"%.80s\" resolves to a non-regular index file",
		httpd_ntoa( &hc->client_addr ), hc->encodedurl );
	    httpd_send_err(
		hc, 403, err403title, "",
		ERROR_FORM( err403form, "The requested URL '%.80s' resolves to a non-regular index file.\n" ),
		hc->encodedurl );
	    return -1;
	    }

	/* Now, is the index version world-readable or world-executable? */
	if ( ! ( hc->sb.st_mode & ( S_IRACC | S_IXACC ) ) )
	    {
	    syslog(
		LOG_INFO,
		"%.80s URL \"%.80s\" resolves to a non-world-readable index file",
		httpd_ntoa( &hc->client_addr ), hc->encodedurl );
	    httpd_send_err(
		hc, 403, err403title, "",
		ERROR_FORM( err403form, "The requested URL '%.80s' resolves to an index file that is not world-readable.\n" ),
		hc->encodedurl );
	    return -1;
	    }
	}
    else
    if ( S_ISREG(hc->sb.st_mode) )
	isRegOrigFile = 1;
    else
	{
	syslog(
		LOG_INFO,
		"%.80s URL \"%.80s\" does not resolve to a directory or a regular file",
		httpd_ntoa( &hc->client_addr ), hc->encodedurl );
	httpd_send_err(
		hc, 403, err403title, "",
		ERROR_FORM( err403form, "The requested URL '%.80s' does not resolve to a directory or a regular file.\n" ),
		hc->encodedurl );
	return -1;
	}

#ifdef AUTH_FILE
    /* Check authorization for this directory. */
    httpd_realloc_str( &dirname, &maxdirname, expnlen );
    (void) strcpy( dirname, hc->expnfilename );
    cp = strrchr( dirname, '/' );
    if ( cp == (char*) 0 )
	(void) strcpy( dirname, "." );
    else
	*cp = '\0';
#if defined(NAS) || defined(DVR)
	strcpy(dirname, ".");
#endif
    if ( (i = auth_check( hc, dirname )) <= -1 )
    	return i;
	//return -1;

    /* Check if the filename is the AUTH_FILE itself - that's verboten. */
    if ( expnlen == sizeof(AUTH_FILE) - 1 )
	{
	if ( strcmp( hc->expnfilename, AUTH_FILE ) == 0 )
	    {
	    syslog(
		LOG_NOTICE,
		"%.80s URL \"%.80s\" tried to retrieve an auth file",
		httpd_ntoa( &hc->client_addr ), hc->encodedurl );
	    httpd_send_err(
		hc, 403, err403title, "",
		ERROR_FORM( err403form, "The requested URL '%.80s' is an authorization file, retrieving it is not permitted.\n" ),
		hc->encodedurl );
	    return -1;
	    }
	}
    else if ( expnlen >= sizeof(AUTH_FILE) &&
	      strcmp( &(hc->expnfilename[expnlen - sizeof(AUTH_FILE) + 1]), AUTH_FILE ) == 0 &&
	      hc->expnfilename[expnlen - sizeof(AUTH_FILE)] == '/' )
	{
	syslog(
	    LOG_NOTICE,
	    "%.80s URL \"%.80s\" tried to retrieve an auth file",
	    httpd_ntoa( &hc->client_addr ), hc->encodedurl );
	httpd_send_err(
	    hc, 403, err403title, "",
	    ERROR_FORM( err403form, "The requested URL '%.80s' is an authorization file, retrieving it is not permitted.\n" ),
	    hc->encodedurl );
	return -1;
	}
#endif /* AUTH_FILE */

    /* Referer check. */
    if ( ! check_referer( hc ) )
	return -1;

#ifdef DVR
	if ( (i = local_disk_busy_reject(hc)) <= -1)
		return i;
#endif

    /* Is it world-executable and in the CGI area? */
    if ( hc->hs->cgi_pattern != (char*) 0 &&
	 ( hc->sb.st_mode & S_IXACC ) &&
	 match( hc->hs->cgi_pattern, hc->expnfilename ) )
	{
	return cgi( hc );
	}

    /* It's not CGI.  If it's executable or there's pathinfo, someone's
    ** trying to either serve or run a non-CGI file as CGI.   Either case
    ** is prohibited.
    */
    if ( hc->sb.st_mode & S_IXACC )
	{
#if defined(NAS) || defined(DVR)
	//add by Kent Tsai 2001/02/05
	//execuable file and we want to save it
	hc->file_address = 0x0;
	return 0;
#endif
	syslog(
	    LOG_NOTICE, "%.80s URL \"%.80s\" is executable but isn't CGI",
	    httpd_ntoa( &hc->client_addr ), hc->encodedurl );
	httpd_send_err(
	    hc, 403, err403title, "",
	    ERROR_FORM( err403form, "The requested URL '%.80s' resolves to a file which is marked executable but is not a CGI file; retrieving it is forbidden.\n" ),
	    hc->encodedurl );
	return -1;
	}
    if ( hc->pathinfo[0] != '\0' )
	{
	syslog(
	    LOG_INFO, "%.80s URL \"%.80s\" has pathinfo but isn't CGI",
	    httpd_ntoa( &hc->client_addr ), hc->encodedurl );
	httpd_send_err(
	    hc, 403, err403title, "",
	    ERROR_FORM( err403form, "The requested URL '%.80s' resolves to a file plus CGI-style pathinfo, but the file is not a valid CGI file.\n" ),
	    hc->encodedurl );
	return -1;
	}

    if ( isRegOrigFile )
	{
	size_t origfilename_len = strlen( hc->origfilename );
	if ( origfilename_len > 1 &&
	    hc->origfilename[origfilename_len - 1] == '/' )
	    {
	    size_t encodedurl_len0;
	    size_t encodedurl_len = strlen( hc->encodedurl );
	    encodedurl_len0 = encodedurl_len;
	    while ( encodedurl_len > 1 &&
		    hc->encodedurl[encodedurl_len - 1] == '/' )
		{
		hc->encodedurl[--encodedurl_len] = '\0';
		}
	    if ( encodedurl_len0 != encodedurl_len )
		{
		send_redirect( hc, hc->encodedurl );
		return -1;
		}
	    }
	}

#ifdef DVR
send_jpeg_response:
#endif
    figure_mime( hc );

#ifndef USE_LAYOUT
    /* Fill in end_byte_loc, if necessary. */
    if ( hc->got_range &&
	 ( hc->end_byte_loc == -1L || hc->end_byte_loc >= hc->sb.st_size ) )
	hc->end_byte_loc = hc->sb.st_size - 1;
#else /* USE_LAYOUT */
    /* For Layout, check if it is a text/html file */
    if ( ( lheaderfile != NULL || lfooterfile != NULL ) &&
		hc->sb.st_size > 0 &&
		strncmp( hc->type, "text/html", 9 ) == 0 )
	{
	hc->layout = 1; /* we need to modify output and Content-length */
	hc->lheaderfile_len = lheaderfile_len;
	hc->lfooterfile_len = lfooterfile_len;
	}
    else
	{
	hc->layout = 0; /* we don't need to modify output and Content-length */
	hc->lheaderfile_len = 0;
	hc->lfooterfile_len = 0;
	}

    /* Fill in end_byte_loc, if necessary. */
    /* NOTE: don't change hc->lheaderfile_len and/or hc->lfooterfile_len */
    /* because, later, we use them inside send_mime() */

    if ( hc->got_range )
	{
	/* avoid values of end_byte_loc inside header or footer range */
	if ( hc->end_byte_loc == -1L || hc->end_byte_loc >= hc->sb.st_size +
	    hc->lheaderfile_len )
	    /* set it at the end of hdr + file + ftr */
	    hc->end_byte_loc = hc->sb.st_size +
				hc->lheaderfile_len + hc->lfooterfile_len - 1;
	else
	if ( hc->end_byte_loc < hc->lheaderfile_len - 1 )
	    /* set it at the end of hdr */
	    hc->end_byte_loc = hc->lheaderfile_len - 1;
	}
#endif /* USE_LAYOUT */

    if ( hc->method == METHOD_HEAD )
	{
	send_mime(
	    hc, 200, ok200title, hc->encodings, "", hc->type, hc->sb.st_size,
	    hc->sb.st_mtime );
	}
    else if ( hc->send_jpeg_buffer == 0 && hc->if_modified_since != (time_t) -1 &&
	 hc->if_modified_since >= hc->sb.st_mtime )
	{
	hc->method = METHOD_HEAD;
	send_mime(
	    hc, 304, err304title, hc->encodings, "", hc->type, hc->sb.st_size,
	    hc->sb.st_mtime );
	}
    else
	{
#ifdef DVR
	if (hc->send_jpeg_buffer == 2) {
		char usr_name[128];
		hc->sb.st_size = 1;
		hc->file_address = (char *) malloc(1);
		hc->file_address[0] = ocx_auth_check(hc, usr_name);
//DebugOutput2("IEIEVRS for %s, ret=%X!\n", usr_name, hc->file_address[0]);
	}
	else if (hc->send_jpeg_buffer == 1) {
		channel_info *ci = NULL;
		//double begin = timeGet();
		int buffer_id, count;

#ifdef AUTH_IN_SERVER
		// do authentication check
		if( __security_level>0 && do_auth_for_live_stream( hc )<0 )
		{
	 	    hc->sb.st_size = 2;
		    hc->file_address = (char *) malloc(2);
		    hc->file_address[0]='0';
		    hc->file_address[1]='0';
		}
		else
		{
#endif
		//DebugOutput("card %d, ch %d begin: %f\n", hc->current_card, hc->current_ch, begin);
		ci = Open_Videomm(hc->current_card, hc->current_ch);
		if (ci != NULL) {
		  	//Read_Lock_Videomm(hc->current_card, hc->current_ch);
		  	buffer_id = ci->current;
		  	//Read_Unlock_Videomm(hc->current_card, hc->current_ch);
		  	if (buffer_id == -1) {
		  		hc->sb.st_size = 1;
				hc->file_address = (char *) malloc(1);
		  	}
		  	else {
		  		if (ci->size[buffer_id] > 0) {
		  			count = ci->count[buffer_id];
		  			hc->sb.st_size = ci->size[buffer_id];
			  		hc->file_address = (char *) malloc(hc->sb.st_size);
			  		memcpy(hc->file_address, ci->data[buffer_id], hc->sb.st_size);
			  		//save_jpeg_file(ci->data[buffer_id], hc->sb.st_size, hc->current_card, hc->current_ch, count, (int) hc->sb.st_mtime);
	  			}
	  			else {
	  				hc->sb.st_size = 1;
					hc->file_address = (char *) malloc(1);
					hc->file_address[0]='0';
	  			}
		  	}
			Close_Videomm(ci);
			//usleep(5000);
			//hc->sb.st_mtime = time(NULL);
			//DebugOutput("size: %d, count: %d, buffer: %d\n", hc->sb.st_size, count, buffer_id);
  			//DebugOutput("Time used: %f\n", timeGet() - begin);
		}
#ifdef AUTH_IN_SERVER
		}
#endif
	}
	else
#endif
	{ /* METHOD_GET */
		void* file_address = (void*) 0;
	//DebugOutput2("in METHOD_GET...\n");
		hc->file_fd = EOF;
		hc->file_address = (char*) 0;
		if ( mmc_is_fdmap( hc->sb.st_size ) &&
	     ( numconn + mmc_get_value( MMC_V_FD_MAP_COUNT ) ) >= maxconn )
	    	{ /* we have to keep another file opened over the limit */
		    char szTmp[32];
		    my_snprintf( szTmp, sizeof(szTmp), "%d", numconn );
	    	httpd_send_err(
			hc, 503, httpd_err503title, "",
			ERROR_FORM( err503form2, "Too many (%s) connections. Retry later.\n" ),
			szTmp );
	//DebugOutput2("sned_err 503\n");
	    	return -1;
	    	}
		if ( mmc_map( &(hc->file_fd), &file_address,
			hc->expnfilename, &(hc->sb), nowP ) != MMC_NORMAL )
	    	{
	    	httpd_send_err( hc, 500, err500title, "", err500form, hc->encodedurl );
	//DebugOutput2("sned_err 500\n");
	    	return -1;
	    	}
		hc->file_address = file_address;
	}
	//DebugOutput2("will send_mine %s, %s(pid=%d)\n", hc->encodings, hc->type, getpid());
	send_mime(
	    hc, 200, ok200title, hc->encodings, "", hc->type, hc->sb.st_size,
	    hc->sb.st_mtime );
	//DebugOutput2("end send_mine %s, %s\n", hc->encodings, hc->type);
	}

    return 0;
    }


int
httpd_start_request( httpd_conn* hc, struct timeval* nowP,
			int numconn, int maxconn )
    {
    int r;

    /* Really start the request. */
    r = really_start_request( hc, nowP, numconn, maxconn );

    /* And return the status. */
    return r;
    }


static void
make_log_entry( httpd_conn* hc, struct timeval* nowP )
    {
    char* ru;
    char url[305];
    char bytes[40];

    if ( hc->hs->no_log )
	return;

    /* This is straight CERN Combined Log Format - the only tweak
    ** being that if we're using syslog() we leave out the date, because
    ** syslogd puts it in.  The included syslogtocern script turns the
    ** results into true CERN format.
    */

    /* Format remote user. */
    if ( hc->remoteuser[0] != '\0' )
	ru = hc->remoteuser;
    else
	ru = "-";
    /* If we're vhosting, prepend the hostname to the url.  This is
    ** a little weird, perhaps writing separate log files for
    ** each vhost would make more sense.
    */
    if ( hc->hs->vhost && ! hc->tildemapped )
	(void) my_snprintf( url, sizeof(url),
	    "/%.100s%.200s",
	    hc->hostname == (char*) 0 ? hc->hs->server_hostname : hc->hostname,
	    hc->encodedurl );
    else
	(void) my_snprintf( url, sizeof(url),
	    "%.200s", hc->encodedurl );
    /* Format the bytes. */
    if ( (long) hc->bytes_sent >= 0 )
	(void) my_snprintf( bytes, sizeof(bytes),
	    "%ld", (long) hc->bytes_sent );
    else
	(void) strcpy( bytes, "-" );

    /* Logfile or syslog? */
    if ( hc->hs->logfp != (FILE*) 0 )
	{
	time_t now;
	struct tm* t;
	const char  cernfmt_nozone[] = "%d/%b/%Y:%H:%M:%S";
	char date_nozone[100];
	int zone;
	char sign;
	char date[100];

	/* Get the current time, if necessary. */
	if ( nowP != (struct timeval*) 0 )
	    now = nowP->tv_sec;
	else
	    now = time( (time_t*) 0 );
	/* Format the time, forcing a numeric timezone (some log analyzers
	** are stoooopid about this).
	*/
	t = localtime( &now );
	(void) strftime( date_nozone, sizeof(date_nozone), cernfmt_nozone, t );
#ifdef HAVE_TM_GMTOFF
	zone = t->tm_gmtoff / 60L;
#else
	zone = -timezone / 60L;
	/* Probably have to add something about daylight time here. */
#endif
	if ( zone >= 0 )
	    sign = '+';
	else
	    {
	    sign = '-';
	    zone = -zone;
	    }
	zone = ( zone / 60 ) * 100 + zone % 60;
	(void) my_snprintf( date, sizeof(date),
	    "%s %c%04d", date_nozone, sign, zone );
	/* And write the log entry. */
	(void) fprintf( hc->hs->logfp,
	    "%.80s - %.80s [%s] \"%.80s %.300s %.80s\" %d %s \"%.200s\" \"%.80s\"\n",
	    httpd_ntoa( &hc->client_addr ), ru, date,
	    httpd_method_str( hc->method ), url, hc->protocol,
	    hc->status, bytes, hc->referer, hc->useragent );
	(void) fflush( hc->hs->logfp );	/* don't need to flush every time */
	}
    else
	syslog( LOG_INFO,
	    "%.80s - %.80s \"%.80s %.200s %.80s\" %d %s \"%.200s\" \"%.80s\"",
	    httpd_ntoa( &hc->client_addr ), ru,
	    httpd_method_str( hc->method ), url, hc->protocol,
	    hc->status, bytes, hc->referer, hc->useragent );
    }


/* Returns 1 if ok to serve the url, 0 if not. */
static int
check_referer( httpd_conn* hc )
    {
    int r;

    /* Are we doing referer checking at all? */
    if ( hc->hs->url_pattern == (char*) 0 )
	return 1;

    r = really_check_referer( hc );

    if ( ! r )
	{
	syslog(
	    LOG_INFO, "%.80s non-local referer \"%.80s\" \"%.80s\"",
	    httpd_ntoa( &hc->client_addr ), hc->encodedurl,
	    hc->referer );
	httpd_send_err(
	    hc, 403, err403title, "",
	    ERROR_FORM( err403form, "You must supply a local referer to get URL '%.80s' from this server.\n" ),
	    hc->encodedurl );
	}
    return r;
    }


/* Returns 1 if ok to serve the url, 0 if not. */
static int
really_check_referer( httpd_conn* hc )
    {
    httpd_server* hs;
    char* cp1;
    char* cp2;
    char* cp3;
    static char* refhost = (char*) 0;
    static int refhost_size = 0;
    char *lp;

    hs = hc->hs;

    /* Check for an empty referer. */
    if ( hc->referer == (char*) 0 || hc->referer[0] == '\0' ||
	 ( cp1 = strstr( hc->referer, "//" ) ) == (char*) 0 )
	{
	/* Disallow if we require a referer and the url matches. */
	if ( hs->no_empty_referers && match( hs->url_pattern, hc->decodedurl ) )
	    return 0;
	/* Otherwise ok. */
	return 1;
	}

    /* Extract referer host. */
    cp1 += 2;
    for ( cp2 = cp1; *cp2 != '/' && *cp2 != ':' && *cp2 != '\0'; ++cp2 )
	continue;
    httpd_realloc_str( &refhost, &refhost_size, cp2 - cp1 );
    for ( cp3 = refhost; cp1 < cp2; ++cp1, ++cp3 )
	if ( isupper(*cp1) )
	    *cp3 = tolower(*cp1);
	else
	    *cp3 = *cp1;
    *cp3 = '\0';

    /* Local pattern? */
    if ( hs->local_pattern != (char*) 0 )
	lp = hs->local_pattern;
    else
	{
	/* No local pattern.  What's our hostname? */
	if ( ! hs->vhost )
	    {
	    /* Not vhosting, use the server name. */
	    lp = hs->server_hostname;
	    if ( lp == (char*) 0 )
		/* Couldn't figure out local hostname - give up. */
		return 1;
	    }
	else
	    {
	    /* We are vhosting, use the hostname on this connection. */
	    lp = hc->hostname;
	    if ( lp == (char*) 0 )
		/* Oops, no hostname.  Maybe it's an old browser that
		** doesn't send a Host: header.  We could figure out
		** the default hostname for this IP address, but it's
		** not worth it for the few requests like this.
		*/
		return 1;
	    }
	}

    /* If the referer host doesn't match the local host pattern, and
    ** the URL does match the url pattern, it's an illegal reference.
    */
    if ( ! match( lp, refhost ) && match( hs->url_pattern, hc->decodedurl ) )
	return 0;
    /* Otherwise ok. */
    return 1;
    }


char*
httpd_ntoa( httpd_sockaddr* saP )
    {
#ifdef HAVE_GETNAMEINFO
    static char str[200];

    if ( getnameinfo( &saP->sa, sockaddr_len( saP ), str, sizeof(str), 0, 0, NI_NUMERICHOST ) != 0 )
	{
	str[0] = '?';
	str[1] = '\0';
	}
    return str;

#else /* HAVE_GETNAMEINFO */

    return inet_ntoa( saP->sa_in.sin_addr );

#endif /* HAVE_GETNAMEINFO */
    }


static int
sockaddr_check( httpd_sockaddr* saP )
    {
    switch ( saP->sa.sa_family )
	{
	case AF_INET: return 1;
#if defined(AF_INET6) && defined(HAVE_SOCKADDR_IN6)
	case AF_INET6: return 1;
#endif /* AF_INET6 && HAVE_SOCKADDR_IN6 */
	default:
	return 0;
	}
    }


static size_t
sockaddr_len( httpd_sockaddr* saP )
    {
    switch ( saP->sa.sa_family )
	{
	case AF_INET: return sizeof(struct sockaddr_in);
#if defined(AF_INET6) && defined(HAVE_SOCKADDR_IN6)
	case AF_INET6: return sizeof(struct sockaddr_in6);
#endif /* AF_INET6 && HAVE_SOCKADDR_IN6 */
	default:
	return 0;	/* shouldn't happen */
	}
    }


/* Some systems don't have snprintf(), so we make our own that uses
** either vsnprintf() or vsprintf().  If your system doesn't have
** vsnprintf(), it is probably vulnerable to buffer overruns.
** Upgrade!
*/
static int
my_snprintf( char* str, size_t size, const char* format, ... )
    {
    va_list ap;
    int r;

    va_start( ap, format );
#ifdef HAVE_VSNPRINTF
    r = vsnprintf( str, size, format, ap );
#else /* HAVE_VSNPRINTF */
    r = vsprintf( str, format, ap );
#endif /* HAVE_VSNPRINTF */
    va_end( ap );
    return r;
    }


/* Generate debugging statistics syslog message. */
void
httpd_logstats( long secs )
    {

    if (str_alloc_count == 0)
	str_alloc_count++;

    syslog( LOG_NOTICE,
	"  libhttpd - %d strings allocated, %ld bytes (%g bytes/str)",
	str_alloc_count, str_alloc_size,
	(float) str_alloc_size / str_alloc_count );
    }
